;**************************************************************
;     Operno
;
; Autor: Josef Sniatecki
;   Josef Sniatecki 2009 (c)
;**************************************************************





;******************************************
; Macros
;******************************************

Macro _OpernoQ
"
EndMacro
Macro _OpernoImport(Function)
  Global Function.Function = *FunctionTable\L : *FunctionTable+4
EndMacro

Macro _OpernoError(Message, Action1 = , Action2 = , Action3 = )
  OpernoRaiseError(Message)
  Action1
  Action2
  Action3
  ProcedureReturn
EndMacro
Macro _OpernoArgumentError(MinimalCount)
  OpernoRaiseError("The minimal argument count is " + Str(MinimalCount) + "!")
  ProcedureReturn
EndMacro



;******************************************
; Constants
;******************************************

; Value types:
Enumeration
  #OpernoType_Integer
  #OpernoType_Float
  #OpernoType_Quad
  #OpernoType_Double
  #OpernoType_String
  #OpernoType_Bool
  #OpernoType_Variant
EndEnumeration
; Object types:
Enumeration 1
  #OpernoObject_Group
  #OpernoObject_Function
  #OpernoObject_Variable
EndEnumeration
; Operators:
Enumeration
  #OpernoOperator_Set
  #OpernoOperator_Add
  #OpernoOperator_Subtract
  #OpernoOperator_Multiply
  #OpernoOperator_Divide
  #OpernoOperator_Power
  #OpernoOperator_Root
  #OpernoOperator_DoAdd
  #OpernoOperator_DoSubtract
  #OpernoOperator_DoMultiply
  #OpernoOperator_DoDivide
  #OpernoOperator_Equal
  #OpernoOperator_Less
  #OpernoOperator_Greater
  #OpernoOperator_NotEqual
  #OpernoOperator_LessEqual
  #OpernoOperator_GreaterEqual
  #OpernoOperator_Increment
  #OpernoOperator_Decrement
  #OpernoOperator_Format
  #OpernoOperator_Input
  #OpernoOperator_Reference
EndEnumeration
; Class methods:
Enumeration
  #OpernoClassMethod_New
  #OpernoClassMethod_Define
  #OpernoClassMethod_Delete
  #OpernoClassMethod_IsTrue
  #OpernoClassMethod_Set
  #OpernoClassMethod_FriendSet
  #OpernoClassMethod_Add
  #OpernoClassMethod_FriendAdd
  #OpernoClassMethod_Subtract
  #OpernoClassMethod_Negative
  #OpernoClassMethod_FriendSubtract
  #OpernoClassMethod_Multiply
  #OpernoClassMethod_FriendMultiply
  #OpernoClassMethod_Divide
  #OpernoClassMethod_FriendDivide
  #OpernoClassMethod_Power
  #OpernoClassMethod_FriendPower
  #OpernoClassMethod_DoAdd
  #OpernoClassMethod_FriendDoAdd
  #OpernoClassMethod_DoSubtract
  #OpernoClassMethod_FriendDoSubtract
  #OpernoClassMethod_DoMultiply
  #OpernoClassMethod_FriendDoMultiply
  #OpernoClassMethod_DoDivide
  #OpernoClassMethod_FriendDoDivide
  #OpernoClassMethod_Equal
  #OpernoClassMethod_Less
  #OpernoClassMethod_FriendLess
  #OpernoClassMethod_Greater
  #OpernoClassMethod_FriendGreater
  #OpernoClassMethod_Increment
  #OpernoClassMethod_Decrement
  #OpernoClassMethod_Format
  #OpernoClassMethod_Arguments
  #OpernoClassMethod_FileRead
  #OpernoClassMethod_FileWrite
EndEnumeration : #OpernoClassMethodCount = #PB_Compiler_EnumerationValue - 1
; Value scopes:
#OpernoValue_Reference  = %0001
#OpernoValue_Binary     = %0010
#OpernoValue_Searchable = %0100
#OpernoValue_Literal    = %1000
; Callback:
Enumeration
  #OpernoCallback_Output
  #OpernoCallback_Input
  #OpernoCallback_ClearOutput
  #OpernoCallback_Wait
  #OpernoCallback_WaitForReturn
EndEnumeration
; Symbols for Output:
Enumeration
  #OpernoSymbol_None
  #OpernoSymbol_Default
  #OpernoSymbol_ETC
  #OpernoSymbol_Point
  #OpernoSymbol_Output
  #OpernoSymbol_Input
  #OpernoSymbol_Error
  #OpernoSymbol_Message
EndEnumeration



;******************************************
; Structures
;******************************************

Structure OpernoObject
  Type.i
  Name.s
  StructureUnion
    *Group.OpernoGroup
    *Function.OpernoFunction
    *Variable.OpernoVariable
    *Memory
  EndStructureUnion
  *Super.OpernoObject
  *Item
  *Reference.OpernoObject
EndStructure
Structure OpernoGroup
  *ObjectQueue.Queue
  *ClassMethod[#OpernoClassMethodCount]
EndStructure
Structure OpernoFunction
  Type.i
  StructureUnion
    *Address
    *TokenQueue
  EndStructureUnion
  *Class.OpernoObject
EndStructure
Structure OpernoVariable
  *Value.OpernoValue
EndStructure
Structure OpernoValue
  *Class.OpernoObject
  StructureUnion
    *Memory
    *Integer.Integer
    *Float.Float
    *Quad.Quad
    *Double.Double
    *String.String
    *Queue.Queue
    *Object.OpernoObject
  EndStructureUnion
  Scope.i
  *Variable.OpernoObject
EndStructure

Structure OpernoCoordinateSystem
  MinX.f
  MaxX.f
  MinY.f
  MaxY.f
  XFactor.f
  YFactor.f
  
  Image.i
  Width.i
  Height.i
EndStructure



;******************************************
; Variables
;******************************************

Global *OpernoMainObject.OpernoObject
Global *OpernoPreprocessorObject.OpernoObject
Global *OpernoStandardLibrary.OpernoObject
Global *OpernoStandardClassLibrary.OpernoObject
Global *OpernoIntegerClass.OpernoObject
Global *OpernoFloatClass.OpernoObject
Global *OpernoQuadClass.OpernoObject
Global *OpernoDoubleClass.OpernoObject
Global *OpernoBoolClass.OpernoObject
Global *OpernoStringClass.OpernoObject
Global *OpernoQueueClass.OpernoObject
Global *OpernoBlockClass.OpernoObject
Global *OpernoPointerClass.OpernoObject
Global *OpernoObjectClass.OpernoObject



;******************************************
; Procedures
;******************************************

Procedure.i OpernoStringCallback(String.s, *Result.String)
  ; Strings aus dem Client werden nicht innerhalb der DLL modifiziert, da
  ; der Heap nur innerhalb des Clients aktuallisiert werden kann.
  *Result\S = String
EndProcedure



;******************************************
; Imports
;******************************************

Prototype.i OpernoMainObject()
Prototype.i OpernoPreprocessorObject()
Prototype.i OpernoStandardLibrary()
Prototype.i OpernoStandardClassLibrary()
Prototype.i OpernoIntegerClass()
Prototype.i OpernoFloatClass()
Prototype.i OpernoQuadClass()
Prototype.i OpernoDoubleClass()
Prototype.i OpernoBoolClass()
Prototype.i OpernoStringClass()
Prototype.i OpernoQueueClass()
Prototype.i OpernoBlockClass()
Prototype.i OpernoPointerClass()
Prototype.i OpernoObjectClass()

Prototype.i OpernoThis()
Prototype.i OpernoMemory()
Prototype.i OpernoTable()
Prototype.i OpernoClassName_(*Class.OpernoObject)

Prototype.i OpernoOpenLevel(*ArgumentQueue, *Object.OpernoObject, *This.OpernoValue)
Prototype.i OpernoUseObject(*Object.OpernoObject)
Prototype.i OpernoCloseLevel()
Prototype.i OpernoClearStack()
Prototype.i OpernoTruncation()
Prototype.i OpernoSetTruncation(Level.i)
Prototype.i OpernoExit()

Prototype.i OpernoError()
Prototype.i OpernoErrorMessage_()
Prototype.i OpernoRaiseError(Message.s)

Prototype.i OpernoEval(Code.s)
Prototype.i OpernoEvalI(Code.s)
Prototype.f OpernoEvalF(Code.s)
Prototype.q OpernoEvalQ(Code.s)
Prototype.d OpernoEvalD(Code.s)
Prototype.i OpernoEvalS_(Code.s)
Prototype.i OpernoEvalCommands(Code.s)
Prototype.i OpernoEvalFile(Name.s)

Prototype.i OpernoParse(Code.s)
Prototype.i OpernoRun(*TokenQueue)
Prototype.i OpernoRunI(*TokenQueue)
Prototype.f OpernoRunF(*TokenQueue)
Prototype.q OpernoRunQ(*TokenQueue)
Prototype.d OpernoRunD(*TokenQueue)
Prototype.i OpernoRunS_(*TokenQueue)
Prototype.i OpernoRunCommands(*TokenQueue)
Prototype.i OpernoFreeTokenQueue(*TokenQueue)

Prototype.i OpernoValueClass(*Value.OpernoValue)
Prototype.i OpernoValueI(*Value.OpernoValue)
Prototype.f OpernoValueF(*Value.OpernoValue)
Prototype.q OpernoValueQ(*Value.OpernoValue)
Prototype.d OpernoValueD(*Value.OpernoValue)
Prototype.i OpernoValueB(*Value.OpernoValue)
Prototype.i OpernoValueS_(*Value.OpernoValue)
Prototype.i OpernoValueP(*Value.OpernoValue)
Prototype.i OpernoValueScope(*Value.OpernoValue)
Prototype.i OpernoSetValueScope(*Value.OpernoValue, Scope.i)
Prototype.i OpernoAddValueScope(*Value.OpernoValue, Scope.i)
Prototype.i OpernoValueName_(*Value.OpernoValue)

Prototype.i OpernoSetValue(*Value.OpernoValue, *SourceValue.OpernoValue)
Prototype.i OpernoOperateValue(Operation.i, *LHS.OpernoValue, *RHS.OpernoValue)
Prototype.i OpernoDuplicateValue(*Value.OpernoValue)
Prototype.i OpernoPushDuplicatedValue(*Value.OpernoValue)
Prototype.i OpernoFormatValue_(*Value.OpernoValue)

Prototype.i OpernoCreateValue(*Class.OpernoObject)
Prototype.i OpernoCreateIValue(Integer.i)
Prototype.i OpernoCreateFValue(Float.f)
Prototype.i OpernoCreateQValue(Quad.q)
Prototype.i OpernoCreateDValue(Double.d)
Prototype.i OpernoCreateBValue(State.i)
Prototype.i OpernoCreateSValue(String.s)
Prototype.i OpernoCreateQueueValue()
Prototype.i OpernoCreateBlockValue(Code.s)
Prototype.i OpernoCreatePValue(*Object.OpernoObject)
Prototype.i OpernoCreateReferenceValue(*Original.OpernoValue)

Prototype.i OpernoPushValue(*Class.OpernoObject)
Prototype.i OpernoPushIValue(Integer.i)
Prototype.i OpernoPushFValue(Float.f)
Prototype.i OpernoPushQValue(Quad.q)
Prototype.i OpernoPushDValue(Double.d)
Prototype.i OpernoPushBValue(State.i)
Prototype.i OpernoPushSValue(String.s)
Prototype.i OpernoPushQueueValue()
Prototype.i OpernoPushBlockValue(Code.s)
Prototype.i OpernoPushPValue(*Object.OpernoObject)
Prototype.i OpernoPushReferenceValue(*Original.OpernoValue)

Prototype.i OpernoPushQueue1(*Value1.OpernoValue)
Prototype.i OpernoPushQueue2(*Value1.OpernoValue, *Value2.OpernoValue)
Prototype.i OpernoPushQueue3(*Value1.OpernoValue, *Value2.OpernoValue, *Value3.OpernoValue)
Prototype.i OpernoPushGroup1(Name1.s, *Value1.OpernoValue)
Prototype.i OpernoPushGroup2(Name1.s, *Value1.OpernoValue, Name2.s, *Value2.OpernoValue)
Prototype.i OpernoPushGroup3(Name1.s, *Value1.OpernoValue, Name2.s, *Value2.OpernoValue, Name3.s, *Value3.OpernoValue)

Prototype.i OpernoArgumentCount()
Prototype.i OpernoPopValue(*Result.Integer)
Prototype.i OpernoPopIValue(*Result.Integer)
Prototype.i OpernoPopFValue(*Result.Float)
Prototype.i OpernoPopQValue(*Result.Quad)
Prototype.i OpernoPopDValue(*Result.Double)
Prototype.i OpernoPopBValue(*Result.Integer)
Prototype.i OpernoPopSValue(*Result.String)
Prototype.i OpernoPopPValue(*Result.Integer)
Prototype.i OpernoPopVariable(*Result.OpernoObject)

Prototype.i OpernoItemCount(*Value.OpernoValue)
Prototype.i OpernoFirstItem(*Value.OpernoValue)
Prototype.i OpernoLastItem(*Value.OpernoValue)
Prototype.i OpernoNextItem(*Item)
Prototype.i OpernoPreviousItem(*Item)
Prototype.i OpernoItem(*Value.OpernoValue, Index.i)
Prototype.i OpernoItemValue(*Item)
Prototype.i OpernoClearQueue(*Value.OpernoValue)
Prototype.i OpernoPrependItem(*Value.OpernoValue, *NewMember.OpernoValue)
Prototype.i OpernoAppendItem(*Value.OpernoValue, *NewMember.OpernoValue)
Prototype.i OpernoFreeItem(*Value.OpernoValue, *Item)

Prototype.i OpernoCreateGroup(Name.s, *Super.OpernoObject = -1)
Prototype.i OpernoChildCount(*Object.OpernoObject)
Prototype.i OpernoFirstChild(*Object.OpernoObject)
Prototype.i OpernoLastChild(*Object.OpernoObject)
Prototype.i OpernoGroupVariableName_(*Object.OpernoObject)

Prototype.i OpernoCreateVirtualTable(TableSize.i)
Prototype.i OpernoGetVirtualTable(*Object.OpernoObject)

Prototype.i OpernoCreateFunction(Name.s, *Address, *Super.OpernoObject = -1)
Prototype.i OpernoCall(*Object.OpernoObject, *ArgumentQueue = -1, *This.OpernoValue = 0)
Prototype.i OpernoArguments1(*Value1.OpernoValue)
Prototype.i OpernoArguments2(*Value1.OpernoValue, *Value2.OpernoValue)

Prototype.i OpernoCreateVariable(Name.s, *Value.OpernoValue, *Super.OpernoObject = -1)
Prototype.i OpernoCreateIVariable(Name.s, Integer.i, *Super.OpernoObject = -1)
Prototype.i OpernoCreateFVariable(Name.s, Float.f, *Super.OpernoObject = -1)
Prototype.i OpernoCreateQVariable(Name.s, Quad.q, *Super.OpernoObject = -1)
Prototype.i OpernoCreateDVariable(Name.s, Double.d, *Super.OpernoObject = -1)
Prototype.i OpernoCreateBVariable(Name.s, State.i, *Super.OpernoObject = -1)
Prototype.i OpernoCreateSVariable(Name.s, String.s, *Super.OpernoObject = -1)
Prototype.i OpernoCreatePVariable(Name.s, *Address.OpernoObject, *Super.OpernoObject = -1)
Prototype.i OpernoCreateVVariable(Name.s, *Super.OpernoObject = -1)
Prototype.i OpernoVariableValue(*Object.OpernoObject)
Prototype.i OpernoVariableMemory(*Object.OpernoObject)
Prototype.i OpernoVariableI(*Object.OpernoObject)
Prototype.f OpernoVariableF(*Object.OpernoObject)
Prototype.q OpernoVariableQ(*Object.OpernoObject)
Prototype.d OpernoVariableD(*Object.OpernoObject)
Prototype.i OpernoVariableB(*Object.OpernoObject)
Prototype.i OpernoVariableS_(*Object.OpernoObject)
Prototype.i OpernoVariableP(*Object.OpernoObject)
Prototype.i OpernoSetVariable(*Object.OpernoObject, *Value.OpernoValue)
Prototype.i OpernoSetIVariable(*Object.OpernoObject, Integer.i)
Prototype.i OpernoSetFVariable(*Object.OpernoObject, Float.f)
Prototype.i OpernoSetQVariable(*Object.OpernoObject, Quad.q)
Prototype.i OpernoSetDVariable(*Object.OpernoObject, Double.d)
Prototype.i OpernoSetBVariable(*Object.OpernoObject, State.i)
Prototype.i OpernoSetSVariable(*Object.OpernoObject, String.s)
Prototype.i OpernoSetPVariable(*Object.OpernoObject, *Address.OpernoObject)

Prototype.i OpernoObjectName_(*Object.OpernoObject)
Prototype.i OpernoRenameObject(*Object.OpernoObject, Name.s)
Prototype.i OpernoObjectType(*Object.OpernoObject)
Prototype.i OpernoObjectPath_(*Object.OpernoObject)
Prototype.i OpernoNextObject(*Object.OpernoObject)
Prototype.i OpernoPreviousObject(*Object.OpernoObject)
Prototype.i OpernoGet(Name.s, *Super.OpernoObject = -1)
Prototype.i OpernoCreateReferenceObject(Name.s, *Original.OpernoObject, *Super.OpernoObject = -1)
Prototype.i OpernoExtractObject(*Super.OpernoObject, *Memory)
Prototype.i OpernoFreeObject(*Object.OpernoObject)

Prototype.i OpernoImportLibrary(LibraryName.s)

Prototype.i OpernoOpenUI()
Prototype.i OpernoOpenGUI()
Prototype.i OpernoOutput(Symbol.i, Text.s, Indent.i = 0)
Prototype.i OpernoOutputHL()
Prototype.i OpernoInput_(Text.s, Indent.i = 0)
Prototype.i OpernoInputCode_(Indent.i = 0)
Prototype.i OpernoCallback(*Callback)
Prototype.i OpernoEnableSymbolOutput(State.i)
Prototype.i OpernoOpenConsole(Title.s = "Operno")

Prototype.i OpernoVersion_()
Prototype.i OpernoOpenHelp()
Prototype.i OpernoGenerateHTMLFile(Filename.s, HTMLFilename.s)
Prototype.i OpernoConfigureHTMLGenerator()
Prototype.i OpernoOpenPreview()

Prototype.i OpernoClassMethodID(Method.s)

Prototype.i OpernoCreateCoordinateSystem(Width.i, Height.i, MinX.f, MinY.f, MaxX.f, MaxY.f)
Prototype.i OpernoCoordinateSystemDrawBackground(*System, BackgroundColor.i = $FFFFFF, AxeColor.i = $808080, GridColor.i = $E0E0E0)
Prototype.i OpernoCoordinateSystemGraph(*System, Graph.s, Color.i = $000000, Thickness.i = 2)
Prototype.i OpernoCoordinateSystemImageID(*System)
Prototype.i OpernoCoordinateSystemSaveImage(*System, Filename.s)
Prototype.i OpernoFreeCoordinateSystem(*System)

Prototype.i OpernoSetStringCallback(*Address)



ProcedureDLL.i OpernoGetFunctions(*FunctionTable.Long)
  _OpernoImport(OpernoMainObject)
  _OpernoImport(OpernoPreprocessorObject)
  _OpernoImport(OpernoStandardLibrary)
  _OpernoImport(OpernoStandardClassLibrary)
  _OpernoImport(OpernoIntegerClass)
  _OpernoImport(OpernoFloatClass)
  _OpernoImport(OpernoQuadClass)
  _OpernoImport(OpernoDoubleClass)
  _OpernoImport(OpernoBoolClass)
  _OpernoImport(OpernoStringClass)
  _OpernoImport(OpernoQueueClass)
  _OpernoImport(OpernoBlockClass)
  _OpernoImport(OpernoPointerClass)
  _OpernoImport(OpernoObjectClass)
  
  _OpernoImport(OpernoThis)
  _OpernoImport(OpernoMemory)
  _OpernoImport(OpernoTable)
  _OpernoImport(OpernoClassName_)
  
  _OpernoImport(OpernoOpenLevel)
  _OpernoImport(OpernoUseObject)
  _OpernoImport(OpernoCloseLevel)
  _OpernoImport(OpernoClearStack)
  _OpernoImport(OpernoTruncation)
  _OpernoImport(OpernoSetTruncation)
  _OpernoImport(OpernoExit)
  
  _OpernoImport(OpernoError)
  _OpernoImport(OpernoErrorMessage_)
  _OpernoImport(OpernoRaiseError)
  
  _OpernoImport(OpernoEval)
  _OpernoImport(OpernoEvalI)
  _OpernoImport(OpernoEvalF)
  _OpernoImport(OpernoEvalQ)
  _OpernoImport(OpernoEvalD)
  _OpernoImport(OpernoEvalS_)
  _OpernoImport(OpernoEvalCommands)
  _OpernoImport(OpernoEvalFile)
  
  _OpernoImport(OpernoParse)
  _OpernoImport(OpernoRun)
  _OpernoImport(OpernoRunI)
  _OpernoImport(OpernoRunF)
  _OpernoImport(OpernoRunQ)
  _OpernoImport(OpernoRunD)
  _OpernoImport(OpernoRunS_)
  _OpernoImport(OpernoRunCommands)
  _OpernoImport(OpernoFreeTokenQueue)
  
  _OpernoImport(OpernoValueClass)
  _OpernoImport(OpernoValueI)
  _OpernoImport(OpernoValueF)
  _OpernoImport(OpernoValueQ)
  _OpernoImport(OpernoValueD)
  _OpernoImport(OpernoValueB)
  _OpernoImport(OpernoValueS_)
  _OpernoImport(OpernoValueP)
  _OpernoImport(OpernoValueScope)
  _OpernoImport(OpernoSetValueScope)
  _OpernoImport(OpernoAddValueScope)
  _OpernoImport(OpernoValueName_)
  
  _OpernoImport(OpernoSetValue)
  _OpernoImport(OpernoOperateValue)
  _OpernoImport(OpernoDuplicateValue)
  _OpernoImport(OpernoPushDuplicatedValue)
  _OpernoImport(OpernoFormatValue_)
  
  _OpernoImport(OpernoCreateValue)
  _OpernoImport(OpernoCreateIValue)
  _OpernoImport(OpernoCreateFValue)
  _OpernoImport(OpernoCreateQValue)
  _OpernoImport(OpernoCreateDValue)
  _OpernoImport(OpernoCreateBValue)
  _OpernoImport(OpernoCreateSValue)
  _OpernoImport(OpernoCreateQueueValue)
  _OpernoImport(OpernoCreateBlockValue)
  _OpernoImport(OpernoCreatePValue)
  _OpernoImport(OpernoCreateReferenceValue)
  
  _OpernoImport(OpernoPushValue)
  _OpernoImport(OpernoPushIValue)
  _OpernoImport(OpernoPushFValue)
  _OpernoImport(OpernoPushQValue)
  _OpernoImport(OpernoPushDValue)
  _OpernoImport(OpernoPushBValue)
  _OpernoImport(OpernoPushSValue)
  _OpernoImport(OpernoPushQueueValue)
  _OpernoImport(OpernoPushBlockValue)
  _OpernoImport(OpernoPushPValue)
  _OpernoImport(OpernoPushReferenceValue)
  
  _OpernoImport(OpernoPushQueue1)
  _OpernoImport(OpernoPushQueue2)
  _OpernoImport(OpernoPushQueue3)
  _OpernoImport(OpernoPushGroup1)
  _OpernoImport(OpernoPushGroup2)
  _OpernoImport(OpernoPushGroup3)
  
  _OpernoImport(OpernoArgumentCount)
  _OpernoImport(OpernoPopValue)
  _OpernoImport(OpernoPopIValue)
  _OpernoImport(OpernoPopFValue)
  _OpernoImport(OpernoPopQValue)
  _OpernoImport(OpernoPopDValue)
  _OpernoImport(OpernoPopBValue)
  _OpernoImport(OpernoPopSValue)
  _OpernoImport(OpernoPopPValue)
  _OpernoImport(OpernoPopVariable)
  
  _OpernoImport(OpernoItemCount)
  _OpernoImport(OpernoFirstItem)
  _OpernoImport(OpernoLastItem)
  _OpernoImport(OpernoNextItem)
  _OpernoImport(OpernoPreviousItem)
  _OpernoImport(OpernoItem)
  _OpernoImport(OpernoItemValue)
  _OpernoImport(OpernoClearQueue)
  _OpernoImport(OpernoPrependItem)
  _OpernoImport(OpernoAppendItem)
  _OpernoImport(OpernoFreeItem)
  
  _OpernoImport(OpernoCreateGroup)
  _OpernoImport(OpernoChildCount)
  _OpernoImport(OpernoFirstChild)
  _OpernoImport(OpernoLastChild)
  _OpernoImport(OpernoGroupVariableName_)
  
  _OpernoImport(OpernoCreateVirtualTable)
  _OpernoImport(OpernoGetVirtualTable)
  
  _OpernoImport(OpernoCreateFunction)
  _OpernoImport(OpernoCall)
  _OpernoImport(OpernoArguments1)
  _OpernoImport(OpernoArguments2)
  
  _OpernoImport(OpernoCreateVariable)
  _OpernoImport(OpernoCreateIVariable)
  _OpernoImport(OpernoCreateFVariable)
  _OpernoImport(OpernoCreateQVariable)
  _OpernoImport(OpernoCreateDVariable)
  _OpernoImport(OpernoCreateBVariable)
  _OpernoImport(OpernoCreateSVariable)
  _OpernoImport(OpernoCreatePVariable)
  _OpernoImport(OpernoCreateVVariable)
  _OpernoImport(OpernoVariableValue)
  _OpernoImport(OpernoVariableMemory)
  _OpernoImport(OpernoVariableI)
  _OpernoImport(OpernoVariableF)
  _OpernoImport(OpernoVariableQ)
  _OpernoImport(OpernoVariableD)
  _OpernoImport(OpernoVariableB)
  _OpernoImport(OpernoVariableS_)
  _OpernoImport(OpernoVariableP)
  _OpernoImport(OpernoSetVariable)
  _OpernoImport(OpernoSetIVariable)
  _OpernoImport(OpernoSetFVariable)
  _OpernoImport(OpernoSetQVariable)
  _OpernoImport(OpernoSetDVariable)
  _OpernoImport(OpernoSetBVariable)
  _OpernoImport(OpernoSetSVariable)
  _OpernoImport(OpernoSetPVariable)
  
  _OpernoImport(OpernoObjectName_)
  _OpernoImport(OpernoRenameObject)
  _OpernoImport(OpernoObjectType)
  _OpernoImport(OpernoObjectPath_)
  _OpernoImport(OpernoNextObject)
  _OpernoImport(OpernoPreviousObject)
  _OpernoImport(OpernoGet)
  _OpernoImport(OpernoCreateReferenceObject)
  _OpernoImport(OpernoExtractObject)
  _OpernoImport(OpernoFreeObject)
  
  _OpernoImport(OpernoImportLibrary)
  
  _OpernoImport(OpernoOpenUI)
  _OpernoImport(OpernoOpenGUI)
  _OpernoImport(OpernoOutput)
  _OpernoImport(OpernoOutputHL)
  _OpernoImport(OpernoInput_)
  _OpernoImport(OpernoInputCode_)
  _OpernoImport(OpernoCallback)
  _OpernoImport(OpernoEnableSymbolOutput)
  _OpernoImport(OpernoOpenConsole)
  
  _OpernoImport(OpernoVersion_)
  _OpernoImport(OpernoOpenHelp)
  _OpernoImport(OpernoGenerateHTMLFile)
  _OpernoImport(OpernoConfigureHTMLGenerator)
  _OpernoImport(OpernoOpenPreview)
  
  _OpernoImport(OpernoClassMethodID)
  
  _OpernoImport(OpernoCreateCoordinateSystem)
  _OpernoImport(OpernoCoordinateSystemDrawBackground)
  _OpernoImport(OpernoCoordinateSystemGraph)
  _OpernoImport(OpernoCoordinateSystemImageID)
  _OpernoImport(OpernoCoordinateSystemSaveImage)
  _OpernoImport(OpernoFreeCoordinateSystem)
  
  _OpernoImport(OpernoSetStringCallback)
  
  ; Get addresses of some important objects:
  *OpernoMainObject           = OpernoMainObject()
  *OpernoPreprocessorObject   = OpernoPreprocessorObject()
  *OpernoStandardLibrary      = OpernoStandardLibrary()
  *OpernoStandardClassLibrary = OpernoStandardClassLibrary()
  *OpernoIntegerClass         = OpernoIntegerClass()
  *OpernoFloatClass           = OpernoFloatClass()
  *OpernoQuadClass            = OpernoQuadClass()
  *OpernoDoubleClass          = OpernoDoubleClass()
  *OpernoBoolClass            = OpernoBoolClass()
  *OpernoStringClass          = OpernoStringClass()
  *OpernoQueueClass           = OpernoQueueClass()
  *OpernoBlockClass           = OpernoBlockClass()
  *OpernoPointerClass         = OpernoPointerClass()
  *OpernoObjectClass          = OpernoObjectClass()
  
  ; Send the address of the "OpernoSetStringCallback":
  OpernoSetStringCallback(@OpernoStringCallback())
EndProcedure

Procedure.s OpernoClassName(*Class.OpernoObject)
  ProcedureReturn PeekS(OpernoClassName_(*Class))
EndProcedure
Procedure.s OpernoErrorMessage()
  ProcedureReturn PeekS(OpernoErrorMessage_())
EndProcedure
Procedure.s OpernoEvalS(Code.s)
  ProcedureReturn PeekS(OpernoEvalS_(Code))
EndProcedure
Procedure.s OpernoRunS(*TokenQueue)
  ProcedureReturn PeekS(OpernoRunS_(*TokenQueue))
EndProcedure
Procedure.s OpernoValueS(*Value.OpernoValue)
  ProcedureReturn PeekS(OpernoValueS_(*Value))
EndProcedure
Procedure.s OpernoValueName(*Value.OpernoValue)
  ProcedureReturn PeekS(OpernoValueName_(*Value))
EndProcedure
Procedure.s OpernoFormatValue(*Value.OpernoValue)
  ProcedureReturn PeekS(OpernoFormatValue_(*Value))
EndProcedure
Procedure.s OpernoGroupVariableName(*Object.OpernoObject)
  ProcedureReturn PeekS(OpernoGroupVariableName_(*Object))
EndProcedure
Procedure.s OpernoVariableS(*Object.OpernoObject)
  ProcedureReturn PeekS(OpernoVariableS_(*Object))
EndProcedure
Procedure.s OpernoObjectName(*Object.OpernoObject)
  ProcedureReturn PeekS(OpernoObjectName_(*Object))
EndProcedure
Procedure.s OpernoObjectPath(*Object.OpernoObject)
  ProcedureReturn PeekS(OpernoObjectPath_(*Object))
EndProcedure
Procedure.s OpernoInput(Text.s, Indent.i = 0)
  ProcedureReturn PeekS(OpernoInput_(Text, Indent))
EndProcedure
Procedure.s OpernoInputCode(Indent.i = 0)
  ProcedureReturn PeekS(OpernoInputCode_(Indent))
EndProcedure
Procedure.s OpernoVersion()
  ProcedureReturn PeekS(OpernoVersion_())
EndProcedure



;******************************************
; Master functions
;******************************************

Global OpernoLibrary.i



Procedure.i InitOperno(DLLName.s = #PB_Compiler_FilePath + "Operno.dll")
  OpernoLibrary = OpenLibrary(#PB_Any, DLLName)
  If OpernoLibrary
    OpernoGetFunctions(CallFunction(OpernoLibrary, "OpernoFunctionTable"))
    ProcedureReturn OpernoLibrary
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i ReloadOperno()
  CloseLibrary(OpernoLibrary)
  ProcedureReturn InitOperno()
EndProcedure
Procedure.i CloseOperno()
  CloseLibrary(OpernoLibrary)
EndProcedure
