;**************************************************************
;     Object (class)
;
; Autor: Josef Sniatecki
; Creation date: 08.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i SetObject(*Object.Object, *SourceObject.Object)
  Protected *Child.Object, *SourceChild.Object
  Protected *Item.QueueItem, *SourceItem.QueueItem
  
  If *Object\Type <> *SourceObject\Type
    _Error("Assigment of two different object types: " + #Q + ObjectPath(*Object) + #Q + " = " + #Q + ObjectPath(*SourceObject) + #Q)
  EndIf
  
  Select *Object\Type
    Case #Object_Group
      *SourceItem = *SourceObject\Group\ObjectQueue\First
      While *SourceItem
        *SourceChild = *SourceItem\Memory
        *Item = *Object\Group\ObjectQueue\First
        While *Item
          *Child = *Item\Memory
          If *Child\Name = *SourceChild\Name
            SetObject(*Child, *SourceChild)
            *SourceChild = 0
            Break
          EndIf
          _Move(*Item, N)
        Wend
        If *SourceChild
          Select *SourceChild\Type
            Case #Object_Group
              *Child = CreateGroup(*SourceChild\Name, *Object)
              SetObject(*Child, *SourceChild)
            Case #Object_Variable
              *Child = CreateVariable(*SourceChild\Name, *SourceChild\Variable\Value, *Object)
            Case #Object_Function
              *Child = CreateFunction(*SourceChild\Name, *Object)
              SetObject(*Child, *SourceChild)
          EndSelect
        EndIf
        _Move(*SourceItem, N)
      Wend
    Case #Object_Variable
      SetValue(*Object\Variable\Value, *SourceObject\Variable\Value)
    Case #Object_Function
      *Object\Function\Type = *SourceObject\Function\Type
      Select *Object\Function\Type
        Case #Function_Address
          *Object\Function\Address = *SourceObject\Function\Address
        Case #Function_Block
          *Object\Function\TokenQueue = CopyQueue(*SourceObject\Function\TokenQueue, CreateQueue(), 0, @CopyToken())
          *Object\Function\Class = *SourceObject\Function\Class
      EndSelect
  EndSelect
EndProcedure

Procedure.i Std_Object__New()
  *Current\This\Object = CreateGroup("")
  *Current\This\Scope  | #Value_Binary|#Value_Searchable
EndProcedure
Procedure.i Std_Object__Def()
  Protected String.String
  
  If PopSValue(@String)
    *Current\This\Object\Name = String\S
  EndIf
EndProcedure
Procedure.i Std_Object__Del()
  FreeObject(*Current\This\Object)
EndProcedure
Procedure.i Std_Object__Set()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *ObjectClass, *PointerClass
      SetObject(*Current\This\Object, *Value\Object)
    Default
      ProcedureReturn 0
  EndSelect
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Object__Form()
  ProcedureReturn PushSValue(FormatObject(*Current\This\Object))
EndProcedure

DataSection
  ObjectClass:
    Data.c #Object_Function : Data.s "class" ;{
      Data.i @Std_Pointer_Class()
    ;}
    Data.c #Object_Function : Data.s "all" ;{
      Data.i @Std_Pointer_All()
    ;}
    Data.c #Object_Function : Data.s "childCount" ;{
      Data.i @Std_Pointer_ChildCount()
    ;}
    Data.c #Object_Function : Data.s "next" ;{
      Data.i @Std_Pointer_Next()
    ;}
    Data.c #Object_Function : Data.s "previous" ;{
      Data.i @Std_Pointer_Previous()
    Data.s "prev"
    ;}
    Data.c #Object_Function : Data.s "children" ;{
      Data.i @Std_Pointer_Children()
    ;}
    Data.c #Object_Function : Data.s "firstChild" ;{
      Data.i @Std_Pointer_FirstChild()
    ;}
    Data.c #Object_Function : Data.s "lastChild" ;{
      Data.i @Std_Pointer_LastChild()
    ;}
    Data.c #Object_Function : Data.s "name" ;{
      Data.i @Std_Pointer_Name()
    ;}
    Data.c #Object_Function : Data.s "rename" ;{
      Data.i @Std_Pointer_Rename()
    ;}
    Data.c #Object_Function : Data.s "group?" ;{
      Data.i @Std_Pointer_IsGroup()
    ;}
    Data.c #Object_Function : Data.s "function?" ;{
      Data.i @Std_Pointer_IsFunction()
    Data.s "func?"
    ;}
    Data.c #Object_Function : Data.s "variable?" ;{
      Data.i @Std_Pointer_IsVariable()
    Data.s "var?"
    ;}
    Data.c #Object_Function : Data.s "import" ;{
      Data.i @Std_Pointer_Import()
    Data.s "imp"
    ;}
    Data.c #Object_Function : Data.s "clear" ;{
      Data.i @Std_Pointer_Clear()
    ;}
    Data.c #Object_Function : Data.s "block" ;{
      Data.i @Std_Pointer_Block()
    ;}
    Data.c #Object_Function : Data.s "path" ;{
      Data.i @Std_Pointer_Path()
    ;}
    Data.c #Object_Function : Data.s _CM(new) ;{
      Data.i @Std_Object__New()
    ;}
    Data.c #Object_Function : Data.s _CM(def) ;{
      Data.i @Std_Pointer_Rename()
    Data.s "rename"
    ;}
    Data.c #Object_Function : Data.s _CM(del) ;{
      Data.i @Std_Object__Del()
    ;}
    Data.c #Object_Function : Data.s _CM(set) ;{
      Data.i @Std_Object__Set()
    ;}
    Data.c #Object_Function : Data.s _CM(form) ;{
      Data.i @Std_Object__Form()
    ;}
    Data.c #Object_Function : Data.s _CM(args) ;{
      Data.i @Std_Pointer__Args()
    ;}
    Data.c #Null
EndDataSection

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 109
; FirstLine = 1
; Folding = ADAAg
; EnableXP
; CompileSourceDirectory