;**************************************************************
;     Pointer (class)
;
; Autor: Josef Sniatecki
; Creation date: 07.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i Std_Pointer_Class()
  Protected *This.Object = *Current\This\Object
  
  If *This\Type = #Object_Variable
    ProcedureReturn PushPValue(*This\Variable\Value\Class)
  Else
    _Error("Only variables can use this function!")
  EndIf
EndProcedure
Procedure.i Std_Pointer_All()
  Protected *ThisObject.Object = *Current\This\Object, *ResultQueue.Queue
  Protected *Item.QueueItem, *Child.Object
  
  If *ThisObject\Type = #Object_Group
    *Item = *ThisObject\Group\ObjectQueue\First
    *ResultQueue = CreateQueue(@FreeValue())
    While *Item
      *Child = *Item\Memory
      If *Child\Type = #Object_Variable
        AppendQueueItem(*ResultQueue, CreateReferenceValue(*Child\Variable\Value))
      EndIf
      _Move(*Item, N)
    Wend
    ProcedureReturn PushQueueValue(*ResultQueue)
  Else
    _Error("Only groups can use " + #Q + "all" + #Q + "!")
  EndIf
EndProcedure
Procedure.i Std_Pointer_ChildCount()
  If *Current\This\Object\Type = #Object_Group
    ProcedureReturn PushIValue(*Current\This\Object\Group\ObjectQueue\Count)
  Else
    _Error("Only groups can use this function!")
  EndIf
EndProcedure
Procedure.i Std_Pointer_Next()
  Protected *Object.Object = *Current\This\Object
  
  If *Object\Super And *Object\Item\N
    ProcedureReturn PushPValue(*Object\Item\N\Object)
  EndIf
  ProcedureReturn PushPValue(#Null)
EndProcedure
Procedure.i Std_Pointer_Previous()
  Protected *Object.Object = *Current\This\Object
  
  If *Object\Super And *Object\Item\P
    ProcedureReturn PushPValue(*Object\Item\P\Object)
  EndIf
  ProcedureReturn PushPValue(#Null)
EndProcedure
Procedure.i Std_Pointer_Children()
  Protected *ThisObject.Object = *Current\This\Object, *Item.QueueItem
  Protected *ResultQueue.Queue
  
  If *ThisObject\Type = #Object_Group
    *Item = *ThisObject\Group\ObjectQueue\First
    *ResultQueue = CreateQueue()
    While *Item
      AppendQueueItem(*ResultQueue, CreatePValue(*Item\Object))
      _Move(*Item, N)
    Wend
    ProcedureReturn PushQueueValue(*ResultQueue)
  Else
    _Error("Only groups can use this function!")
  EndIf
EndProcedure
Procedure.i Std_Pointer_FirstChild()
  Protected *Object.Object = *Current\This\Object
  
  If *Object\Type = #Object_Group
    If *Object\Group\ObjectQueue\First
      ProcedureReturn PushPValue(*Object\Group\ObjectQueue\First\Object)
    Else
      ProcedureReturn PushPValue(#Null)
    EndIf
  Else
    _Error("Only groups can use this function!")
  EndIf
EndProcedure
Procedure.i Std_Pointer_LastChild()
  Protected *Object.Object = *Current\This\Object
  
  If *Object\Type = #Object_Group
    If *Object\Group\ObjectQueue\Last
      ProcedureReturn PushPValue(*Object\Group\ObjectQueue\Last\Object)
    Else
      ProcedureReturn PushPValue(#Null)
    EndIf
  Else
    _Error("Only groups can use this function!")
  EndIf
EndProcedure
Procedure.i Std_Pointer_Name()
  ProcedureReturn PushSValue(*Current\This\Object\Name)
EndProcedure
Procedure.i Std_Pointer_Rename()
  Protected String.String
  
  If PopSValue(@String)
    *Current\This\Object\Name = String\S
  EndIf
EndProcedure
Procedure.i Std_Pointer_IsGroup()
  If *Current\This\Object\Type = #Object_Group
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_Pointer_IsFunction()
  If *Current\This\Object\Type = #Object_Function
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_Pointer_IsVariable()
  If *Current\This\Object\Type = #Object_Variable
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_Pointer_Clear()
  Protected *Item.QueueItem, *Object.Object
  
  If *Current\This\Object\Type = #Object_Group
    *Item = *Current\This\Object\Group\ObjectQueue\First
    While *Item
      *Object = *Item\Memory
      _Move(*Item, N)
      FreeObject(*Object)
    Wend
  Else
    _Error("Only groups can use this function!")
  EndIf
EndProcedure
Procedure.i Std_Pointer_Import()
  Protected *DestinationObject.Object
  Protected *SourceObject.Object, *SourceItem.QueueItem
  
  If *Current\This\Object\Type = #Object_Group
    *DestinationObject = *Current\This\Object
    While PopPValue(@*SourceObject)
      If *SourceObject And *SourceObject\Type = #Object_Group
        *SourceItem = *SourceObject\Group\ObjectQueue\First
        While *SourceItem
          CreateReferenceObject(*SourceItem\Object\Name, *SourceItem\Object, *DestinationObject)
          _Move(*SourceItem, N)
        Wend
      Else
        _Error("Only groups can be imported by " + #Q + "import" + #Q + "!")
      EndIf
    Wend
  Else
    _Error("Only groups can use this function!")
  EndIf
EndProcedure
Procedure.i Std_Pointer_Block()
  If *Current\This\Object\Type = #Object_Function
    If *Current\This\Object\Function\Type = #Function_Block
      ProcedureReturn PushBlockValue(*Current\This\Object\Function\TokenQueue, #True)
    Else
      ProcedureReturn PushIValue(*Current\This\Object\Function\Address)
    EndIf
  Else
    _Error("Only functions can use this function!")
  EndIf
EndProcedure
Procedure.i Std_Pointer_Path()
  ProcedureReturn PushSValue(ObjectPath(*Current\This\Object))
EndProcedure
Procedure.i Std_Pointer__New()
  *Current\This\Scope | #Value_Binary|#Value_Searchable
EndProcedure
Procedure.i Std_Pointer__Def()
  PopPValue(@*Current\This\Object)
EndProcedure
Procedure.i Std_Pointer__IsTrue()
  If *Current\This\Object
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_Pointer__Set()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *PointerClass, *ObjectClass
      *Current\This\Object = *Value\Object
    Case *IntegerClass
      If *Value\Integer\I = 0
        *Current\This\Object = 0
      EndIf
  EndSelect
  UpdateReferences(*Current\This)
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Pointer__Eq()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *PointerClass, *ObjectClass
      If (*Current\This\Object = *Value\Object) Or (*Value\Object\Reference And *Current\This\Object = *Value\Object\Reference)
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
  EndSelect
EndProcedure
Procedure.i Std_Pointer__Inc()
  Protected *This.Value = *Current\This
  
  If *This\Object\Super
    If *This\Object\Item\N
      *This\Object = *This\Object\Item\N\Object
    Else
      *This\Object = #Null
    EndIf
    UpdateReferences(*This)
  EndIf
  ProcedureReturn *This
EndProcedure
Procedure.i Std_Pointer__Dec()
  Protected *This.Value = *Current\This
  
  If *This\Object\Super
    If *This\Object\Item\P
      *This\Object = *This\Object\Item\P\Object
    Else
      *This\Object = #Null
    EndIf
    UpdateReferences(*This)
  EndIf
  ProcedureReturn *This
EndProcedure
Procedure.i Std_Pointer__Form()
  If *Current\This\Object
    ProcedureReturn PushSValue("@"+ObjectPath(*Current\This\Object))
  Else
    ProcedureReturn PushSValue("<NULL>")
  EndIf
EndProcedure
Procedure.i Std_Pointer__Args()
  Protected *Object.Object, *Value.Value
  Protected *ResultObject.Object, *ResultQueue.Value
  
  If *Current\This\Object\Type = #Object_Group
    *Object = *Current\This\Object
    
    If *Current\ArgumentQueue\Count = 1
      PopValueAddress(@*Value)
      Select *Value\Class
        Case *IntegerClass
          SelectQueueItem(*Object\Group\ObjectQueue, *Value\Integer\I, @*ResultObject)
        Case *StringClass
          *ResultObject = FindObject(*Object, *Value\String\S)
      EndSelect
      ProcedureReturn PushPValue(*ResultObject)
    Else
      *ResultQueue = CreateQueue()
      While PopValueAddress(@*Value)
        Select *Value\Class
          Case *IntegerClass
            SelectQueueItem(*Object\Group\ObjectQueue, *Value\Integer\I, @*ResultObject)
          Case *StringClass
            *ResultObject = FindObject(*Object, *Value\String\S)
        EndSelect
        AppendQueueItem(*ResultQueue, CreatePValue(*ResultObject))
      Wend
      ProcedureReturn PushQueueValue(*ResultQueue)
    EndIf
  Else
    _Error("Only groups can use " + #Q + _CM(args) + #Q + "!")
  EndIf
EndProcedure

DataSection
  PointerClass:
    Data.c #Object_Function : Data.s "class" ;{
      Data.i @Std_Pointer_Class()
    ;}
    Data.c #Object_Function : Data.s "all" ;{
      Data.i @Std_Pointer_All()
    ;}
    Data.c #Object_Function : Data.s "childCount" ;{
      Data.i @Std_Pointer_ChildCount()
    ;}
    Data.c #Object_Function : Data.s "next" ;{
      Data.i @Std_Pointer_Next()
    ;}
    Data.c #Object_Function : Data.s "previous" ;{
      Data.i @Std_Pointer_Previous()
    Data.s "prev"
    ;}
    Data.c #Object_Function : Data.s "children" ;{
      Data.i @Std_Pointer_Children()
    ;}
    Data.c #Object_Function : Data.s "firstChild" ;{
      Data.i @Std_Pointer_FirstChild()
    ;}
    Data.c #Object_Function : Data.s "lastChild" ;{
      Data.i @Std_Pointer_LastChild()
    ;}
    Data.c #Object_Function : Data.s "name" ;{
      Data.i @Std_Pointer_Name()
    ;}
    Data.c #Object_Function : Data.s "rename" ;{
      Data.i @Std_Pointer_Rename()
    ;}
    Data.c #Object_Function : Data.s "group?" ;{
      Data.i @Std_Pointer_IsGroup()
    ;}
    Data.c #Object_Function : Data.s "function?" ;{
      Data.i @Std_Pointer_IsFunction()
    Data.s "func?"
    ;}
    Data.c #Object_Function : Data.s "variable?" ;{
      Data.i @Std_Pointer_IsVariable()
    Data.s "var?"
    ;}
    Data.c #Object_Function : Data.s "import" ;{
      Data.i @Std_Pointer_Import()
    Data.s "imp"
    ;}
    Data.c #Object_Function : Data.s "clear" ;{
      Data.i @Std_Pointer_Clear()
    ;}
    Data.c #Object_Function : Data.s "block" ;{
      Data.i @Std_Pointer_Block()
    ;}
    Data.c #Object_Function : Data.s "path" ;{
      Data.i @Std_Pointer_Path()
    ;}
    Data.c #Object_Function : Data.s _CM(new) ;{
      Data.i @Std_Pointer__New()
    ;}
    Data.c #Object_Function : Data.s _CM(def) ;{
      Data.i @Std_Pointer__Def()
    ;}
    Data.c #Object_Function : Data.s _CM(true?) ;{
      Data.i @Std_Pointer__IsTrue()
    ;}
    Data.c #Object_Function : Data.s _CM(set) ;{
      Data.i @Std_Pointer__Set()
    ;}
    Data.c #Object_Function : Data.s _CM(eq) ;{
      Data.i @Std_Pointer__Eq()
    ;}
    Data.c #Object_Function : Data.s _CM(inc) ;{
      Data.i @Std_Pointer__Inc()
    ;}
    Data.c #Object_Function : Data.s _CM(dec) ;{
      Data.i @Std_Pointer__Dec()
    ;}
    Data.c #Object_Function : Data.s _CM(form) ;{
      Data.i @Std_Pointer__Form()
    ;}
    Data.c #Object_Function : Data.s _CM(args) ;{
      Data.i @Std_Pointer__Args()
    Data.s "get"
    ;}
    Data.c #Null
EndDataSection

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 118
; FirstLine = 35
; Folding = BEAAEAA+-
; EnableXP
; CompileSourceDirectory