;**************************************************************
;     Queue (class)
;
; Autor: Josef Sniatecki
; Creation date: 07.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i DimensionQueue(*Queue.Queue, *Class.Object, *DefinitionQueue.Queue, *DimensionItem.QueueItem = 0)
  ; Generates a queue with the specified dimensions:
  
  Protected Index.i, Count.i, *Value.Value, *CallableDefinitionQueue.Queue
  
  Count = *DimensionItem\Value\Integer\I
  If *DimensionItem\N
    While Index < Count
      *Value = CreateQueueValue()
      AppendQueueItem(*Queue, *Value)
      DimensionQueue(*Value\Queue, *Class, *DefinitionQueue, *DimensionItem\N) : _E()
      Index + 1
    Wend
  Else
    If *DefinitionQueue
      While Index < Count
        *Value = CreateValue(*Class)
        AppendQueueItem(*Queue, *Value)
        *CallableDefinitionQueue = CreateQueue(#Queue_SaveMemory)
        CopyQueue(*DefinitionQueue, *CallableDefinitionQueue, #Queue_CopyAddress)
        CallClassMethod(*Class, #ClassMethod_Define, *CallableDefinitionQueue, *Value, #False) : _E()
        Index + 1
      Wend
    Else
      While Index < Count
        AppendQueueItem(*Queue, CreateValue(*Class))
        Index + 1
      Wend
    EndIf
  EndIf
EndProcedure

Procedure.i Std_Queue_Count()
  ProcedureReturn PushIValue(*Current\This\Queue\Count)
EndProcedure
Procedure.i Std_Queue_First()
  ProcedureReturn *Current\This\Queue\First\Value
EndProcedure
Procedure.i Std_Queue_Last()
  ProcedureReturn *Current\This\Queue\Last\Value
EndProcedure
Procedure.i Std_Queue_Row()
  Protected *Item.QueueItem, *Value.Value, *ResultQueue.Queue
  Protected Row.i
  
  If PopIValue(@Row)
    *ResultQueue = CreateQueue(@FreeValue())
    
    *Item = *Current\This\Queue\First
    While *Item
      *Value = *Item\Value
      If *Value\Class = *QueueClass And Row < *Value\Queue\Count
        SelectQueueItem(*Value\Queue, Row, @*Value)
        AppendQueueItem(*ResultQueue, CreateReferenceValue(*Value))
      EndIf
      _Move(*Item, N)
    Wend
    
    ProcedureReturn PushQueueValue(*ResultQueue)
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_Queue_Sum()
  Protected *Item.QueueItem, *Result.Value
  
  *Item = *Current\This\Queue\First
  If *Item
    *Result = PushDuplicatedValue(*Item\Value)
    _Move(*Item, N)
    While *Item
      OperateValue(#Operator_DoAdd, *Result, *Item\Value)
      _Move(*Item, N)
    Wend
    ProcedureReturn *Result
  Else
    _Error("An empty queue used!")
  EndIf
EndProcedure
Procedure.i Std_Queue_Join()
  Protected *Item.QueueItem, *Value.Value
  Protected Result.s, Seperator.String
  
  Seperator\S = ""
  
  PopSValue(@Seperator)
  
  *Item = *Current\This\Queue\First
  While *Item
    Result + FormatValue(*Item\Value)
    If *Item\N
      Result + Seperator\S
    EndIf
    _Move(*Item, N)
  Wend
  ProcedureReturn PushSValue(Result)
EndProcedure
Procedure.i Std_Queue_Average()
  Protected *Item.QueueItem, *Result.Value, Float.f
  
  *Item = *Current\This\Queue\First
  If *Item
    *Result = PushDuplicatedValue(*Item\Value)
    _Move(*Item, N)
    While *Item
      OperateValue(#Operator_DoAdd, *Result, *Item\Value)
      _Move(*Item, N)
    Wend
    Select *Result\Class
      Case *IntegerClass
        Float = *Result\Integer\I
        ProcedureReturn PushFValue(Float / *Current\This\Queue\Count)
      Case *FloatClass
        ProcedureReturn PushFValue(*Result\Float\F / *Current\This\Queue\Count)
      Default
        OperateValue(#Operator_DoDivide, *Result, PushIValue(*Current\This\Queue\Count))
        ProcedureReturn *Result
    EndSelect
  Else
    _Error(#Q + "queue:average" + #Q + " used on an empty queue!")
  EndIf
EndProcedure
Procedure.i Std_Queue_Random()
  Protected *Value.Value
  
  If *Current\This\Queue\Count
    RandomQueueItem(*Current\This\Queue, @*Value)
    ProcedureReturn *Value
  Else
    _Error(#Q + "queue:random" + #Q + " used on an empty queue!")
  EndIf
EndProcedure
Procedure.i Std_Queue_Analyse_Min()
  Protected *Item.QueueItem, *OtherItem.QueueItem
  Protected *Value.Value, *Min.Value
  
  *Item = *Current\This\Queue\First
  If *Item
    *Min = *Item\Value
    _Move(*Item, N)
    
    While *Item
      If IsValueLess(*Item\Value, *Min)
        *Min = *Item\Value
      EndIf
      _Move(*Item, N)
    Wend
    
    ProcedureReturn *Min
  Else
    _Error("No items are avialable.")
  EndIf
EndProcedure
Procedure.i Std_Queue_Analyse_Max()
  Protected *Item.QueueItem, *OtherItem.QueueItem
  Protected *Value.Value, *Max.Value
  
  *Item = *Current\This\Queue\First
  If *Item
    *Max = *Item\Value
    _Move(*Item, N)
    
    While *Item
      If IsValueGreater(*Item\Value, *Max)
        *Max = *Item\Value
      EndIf
      _Move(*Item, N)
    Wend
    
    ProcedureReturn *Max
  Else
    _Error("No items are avialable.")
  EndIf
EndProcedure
Procedure.i Std_Queue_Analyse_Variations()
  Protected *Item.QueueItem
  Protected *CopyQueue.Queue, *VariationQueue.Queue
  Protected *SubQueue.Queue
  Protected Count.i, *Variation.Value
  
  *CopyQueue = CreateQueue(@FreeValue())
  *Item      = *Current\This\Queue\First
  While *Item
    AppendQueueItem(*CopyQueue, CreateReferenceValue(*Item\Value))
    _Move(*Item, N)
  Wend
  
  *VariationQueue = CreateQueue(@FreeValue())
  *Item           = *CopyQueue\First
  While *Item
    *Variation = *Item\Value
    Count = 1
    
    _Move(*Item, N)
    While *Item
      If *Variation\Class = *Item\Value\Class And IsValueEqual(*Variation, *Item\Value)
        Count + 1
        *Item = FreeQueueItem(*CopyQueue, *Item)
      Else
        _Move(*Item, N)
      EndIf
    Wend
    
    *SubQueue = CreateQueue(@FreeValue())
    AppendQueueItem(*SubQueue, CreateReferenceValue(*Variation\Original))
    AppendQueueItem(*SubQueue, CreateIValue(Count))
    AppendQueueItem(*VariationQueue, CreateQueueValue(*SubQueue))
    FreeFirstQueueItem(*CopyQueue)
    
    *Item = *CopyQueue\First
  Wend
  
  FreeQueue(*CopyQueue)
  ProcedureReturn PushQueueValue(*VariationQueue)
EndProcedure
Procedure.i Std_Queue_Combine()
  Protected *This.Queue = *Current\This\Queue, *Value.Value
  Protected *Item.QueueItem, *OtherItem.QueueItem
  Protected *Child.Value, *OtherChild.Value, *NewChild.Value
  
  While PopValueAddress(@*Value)
    If *Value\Class <> *QueueClass
      _Error("Only allowed with queues.")
    ElseIf *This\Count > *Value\Queue\Count
      _Error(#Q + ValueName(*Current\This) + #Q + " is smaller than an other queue.")
    EndIf
    
    *Item      = *This\First
    *OtherItem = *Value\Queue\First
    
    While *Item And *OtherItem
      *Child      = *Item\Memory
      *OtherChild = *OtherItem\Memory
      
      If *Child\Class = *QueueClass
        AppendQueueItem(*Child\Queue, CreateReferenceValue(*OtherChild))
      Else
        *NewChild = CreateQueueValue()
        AppendQueueItem(*NewChild\Queue, *Child)
        AppendQueueItem(*NewChild\Queue, *OtherChild)
        *Item\Memory = *NewChild
      EndIf
      
      _Move(*Item, N)
      _Move(*OtherItem, N)
    Wend
  Wend
  
  ProcedureReturn *Current\This
EndProcedure
Procedure.i Std_Queue_Append()
  Protected *Value.Value
  
  While PopValueAddress(@*Value)
    ; Push a copy of the given values:
    *Value = DuplicateValue(*Value) : _E()
    AppendQueueItem(*Current\This\Queue, *Value)
  Wend
EndProcedure
Procedure.i Std_Queue_AppendReference()
  Protected *Value.Value
  
  While PopValueAddress(@*Value)
    ; Push a reference of the given values:
    AppendQueueItem(*Current\This\Queue, CreateReferenceValue(*Value))
  Wend
EndProcedure
Procedure.i Std_Queue_AppendValue()
  Protected *Class.Object, *Value.Value
  
  If PopPValue(@*Class)
    *Value = CreateValue(*Class) : _E()
    If *Current\ArgumentQueue\Count
      CallClassMethod(*Class, #ClassMethod_Define, 0, *Value) : _E()
    EndIf
    AppendQueueItem(*Current\This\Queue, *Value)
  EndIf
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Queue_Prepend()
  Protected *Value.Value
  
  While PopValueAddress(@*Value)
    ; Push a copy of the given values:
    *Value = DuplicateValue(*Value) : _E()
    PrependQueueItem(*Current\This\Queue, *Value)
  Wend
EndProcedure
Procedure.i Std_Queue_PrependReference()
  Protected *Value.Value
  
  While PopValueAddress(@*Value)
    ; Push a reference of the given values:
    PrependQueueItem(*Current\This\Queue, CreateReferenceValue(*Value))
  Wend
EndProcedure
Procedure.i Std_Queue_PrependValue()
  Protected *Class.Object, *Value.Value
  
  If PopPValue(@*Class)
    *Value = CreateValue(*Class) : _E()
    If *Current\ArgumentQueue\Count
      CallClassMethod(*Class, #ClassMethod_Define, 0, *Value) : _E()
    EndIf
    PrependQueueItem(*Current\This\Queue, *Value)
  EndIf
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_Queue_Remove()
  Protected *Index.Value, *Stack.Queue, *Item.QueueItem
  
  *Stack = CreateQueue(#Queue_SaveMemory)
  
  While PopValueAddress(@*Index)
    ; Save the item into stack:
    AppendQueueItem(*Stack, SelectQueueItem(*Current\This\Queue, *Index\Integer\I))
  Wend
  
  While *Stack\Count
    *Item = PopQueueItem(*Stack)
    ; Remove the item at the saved item:
    If *Item
      FreeQueueItem(*Current\This\Queue, *Item)
    EndIf
  Wend
  
  FreeQueue(*Stack)
EndProcedure
Procedure.i Std_Queue_RemoveValue()
  Protected *Item.QueueItem, *NextItem.QueueItem, *Value.Value
  Protected *ThisQueue.Queue = *Current\This\Queue, *Result.Value
  
  While PopValueAddress(@*Value)
    *Item = *ThisQueue\First
    While *Item
      If *Item\Value\Class = *Value\Class And IsValueEqual(*Item\Value, *Value)
        *NextItem = *Item\N
        FreeQueueItem(*ThisQueue, *Item)
        *Item = *NextItem
        Continue
      EndIf
      _Move(*Item, N)
    Wend
  Wend
  
  ProcedureReturn *Current\This
EndProcedure
Procedure.i Std_Queue_Enumerate()
  Protected Index.i
  Protected *Item.QueueItem, *ResultQueue.Queue, *SubQueue.Queue
  
  *ResultQueue = CreateQueue(@FreeValue())
  
  *Item = *Current\This\Queue\First
  While *Item
    *SubQueue = CreateQueue(@FreeValue())
    AppendQueueItem(*SubQueue, CreateReferenceValue(*Item\Value))
    AppendQueueItem(*SubQueue, CreateIValue(Index))
    AppendQueueItem(*ResultQueue, CreateQueueValue(*SubQueue))
    Index + 1
    _Move(*Item, N)
  Wend
  
  ProcedureReturn PushQueueValue(*ResultQueue)
EndProcedure
Procedure.i Std_Queue_Fill()
  Protected Count.i, *Value.Value, *Copy.Value
  
  If PopValueAddress(@*Value)
    Count = *Current\This\Queue\Count
    ClearQueue(*Current\This\Queue)
    While Count
      *Copy = DuplicateValue(*Value) : _E()
      AppendQueueItem(*Current\This\Queue, *Copy)
      Count-1
    Wend
  EndIf
  ProcedureReturn *Current\This
EndProcedure
Procedure.i Std_Queue_Clear()
  ClearQueue(*Current\This\Queue)
EndProcedure
Procedure.i Std_Queue_Sort()
  Protected N.i, Swapped.i
  Protected *Item.QueueItem, *OtherItem.QueueItem, *TempItem.QueueItem
  Protected *This.Queue = *Current\This\Queue, *Result.Value, *Callback.Value
  Protected *Object.Object, *A.Object, *B.Object
  
  *Callback = CreateBlockValue()
  If Not PopBlockValue(*Callback\Queue)
    AddStringToken(*Callback\Queue, #Token_Name, "a")
    AddIntegerToken(*Callback\Queue, #Token_Operator, #Operator_Less)
    AddStringToken(*Callback\Queue, #Token_Name, "b")
    AddIntegerToken(*Callback\Queue, #Token_Seperator, #Seperator_Command)
  EndIf
  
  *Object = CreateGroup("_sort")
    *A = CreateVariable("a", #Null, *Object)
    *B = CreateVariable("b", #Null, *Object)
  
  N = *This\Count
  
  Repeat
    Swapped = #False
    *Item   = *This\First
    While *Item
      *OtherItem = *Item\N
      If *OtherItem
        ; Examine:
        *A\Variable\Value = *Item\Value
        *B\Variable\Value = *OtherItem\Value
        OpenLevel(*Callback\Queue, 0, *Object, 0)
          *Result = Run() : _E()
          *Result = ValueB(*Result)
        CloseLevel()
        
        If Not *Result
          ; Swap items:
          *OtherItem\P = *Item\P
          *Item\P      = *OtherItem
          *Item\N      = *OtherItem\N
          *OtherItem\N = *Item
          If *OtherItem\P
            *OtherItem\P\N = *OtherItem
          Else
            *This\First = *OtherItem
          EndIf
          If *Item\N
            *Item\N\P = *Item
          Else
            *This\Last = *Item
          EndIf
          Swapped = #True
        EndIf
      EndIf
      _Move(*Item, N)
    Wend
    N - 1
  Until Not (Swapped And N => 1)
  
  *A\Variable\Value = #Null
  *B\Variable\Value = #Null
  FreeObject(*Object)
  
  ProcedureReturn *Current\This
EndProcedure
Procedure.i Std_Queue_AppendQueue()
  Protected *Value.Value, *ThisQueue.Queue = *Current\This\Queue
  
  While PopValueAddress(@*Value)
    If *Value\Class = *QueueClass
      CopyQueue(*Value\Queue, *ThisQueue, @CopyValue(), *ThisQueue\Last)
    Else
      _Error("Only queues can by append by "+#Q+"appendQueue()"+#Q)
    EndIf
  Wend
EndProcedure
Procedure.i Std_Queue_PrependQueue()
  Protected *Value.Value, *ThisQueue.Queue = *Current\This\Queue
  
  While PopValueAddress(@*Value)
    If *Value\Class = *QueueClass
      CopyQueue(*Value\Queue, *ThisQueue, @CopyValue())
    Else
      _Error("Only queues can by append by "+#Q+"prependQueue()"+#Q)
    EndIf
  Wend
EndProcedure
Procedure.i Std_Queue_Push()
  Protected *ArgumentQueue.Queue
  
  *ArgumentQueue = *Current\ArgumentQueue
  CopyQueue(*Current\This\Queue, *ArgumentQueue, @CopyValue(), *ArgumentQueue\Last)
EndProcedure
Procedure.i Std_Queue_PushReference()
  Protected *ArgumentQueue.Queue
  
  *ArgumentQueue = *Current\ArgumentQueue
  CopyQueue(*Current\This\Queue, *ArgumentQueue, #Queue_CopyAddress, *ArgumentQueue\Last)
EndProcedure
Procedure.i Std_Queue_Table()
  Protected *Value.Value, *Item.QueueItem, *SubItem.QueueItem, Result.s, SubResult.s
  Protected Length.i, Seperator.String
  
  Length      = 8
  Seperator\S = ", "
  
  PopIValue(@Length)
  PopSValue(@Seperator)
  
  *Item = *Current\This\Queue\First
  While *Item
    *Value = *Item\Value
    If *Value\Class = *QueueClass
      If Result
        Result + #N
      EndIf
      
      *SubItem  = *Value\Queue\First
      SubResult = ""
      While *SubItem
        If SubResult
          SubResult + Seperator\S
        EndIf
        SubResult + LSet(FormatValue(*SubItem\Value), Length)
        _Move(*SubItem, N)
      Wend
      
      Result + SubResult
    Else
      If Result
        Result + Seperator\S
      EndIf
      
      Result + LSet(FormatValue(*Value), Length)
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn PushSValue(Result)
EndProcedure
Procedure.i Std_Queue__New()
  *Current\This\Queue = CreateQueue(@FreeValue())
  *Current\This\Scope|#Value_Binary
EndProcedure
Procedure.i Std_Queue__Def()
  CopyQueue(*Current\ArgumentQueue, *Current\This\Queue, @CopyValue())
  ClearQueue(*Current\ArgumentQueue)
EndProcedure
Procedure.i Std_Queue__Del()
  FreeQueue(*Current\This\Queue)
EndProcedure
Procedure.i Std_Queue__IsTrue()
  Protected *Item.QueueItem
  
  *Item = *Current\This\Queue\First
  While *Item
    If IsValueTrue(*Item\Value)
      ProcedureReturn PushBValue(#True)
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn PushBValue(#False)
EndProcedure
Procedure.i Std_Queue__Form()
  Protected String.s, *Item.QueueItem
  Protected Value.s, Line.s
  
  If *Current\This\Queue\Count
    ; List all items:
    String = "["
    *Item = *Current\This\Queue\First
    While *Item
      Value = FormatValue(*Item\Value)
      _Move(*Item, N)
      If CountString(Value, #N)
        If Line
          String + Line + "," + #N + " "
          Line = ""
        EndIf
        String + ReplaceString(Value, #N, #N + " ")
        If *Item
          String + "," + #N + " "
        EndIf
      ElseIf Len(Line) + Len(Value) > 30
        String + Line + Value
        Line = ""
        If *Item
          String + "," + #N + " "
        EndIf
      Else
        Line + Value
        If *Item
          Line + ", "
        EndIf
      EndIf
    Wend
    String + Line + "]"
  Else
    ; Return an empty queue:
    String = "[]"
  EndIf
  ProcedureReturn PushSValue(String)
EndProcedure
Procedure.i Std_Queue__Args()
  Protected *Result.Queue, Index.i, *Value.Value, *Item.QueueItem
  
  If *Current\ArgumentQueue\Count > 1
    *Result = CreateQueue(@FreeValue())
    While PopIValue(@Index)
      If SelectQueueItem(*Current\This\Queue, Index, @*Value)
        AppendQueueItem(*Result, CreateReferenceValue(*Value))
      EndIf
    Wend
    ProcedureReturn PushQueueValue(*Result)
  ElseIf *Current\ArgumentQueue\Count
    PopIValue(@Index)
    SelectQueueItem(*Current\This\Queue, Index, @*Result)
    ProcedureReturn *Result
  EndIf
EndProcedure

DataSection
  QueueClass:
    Data.c #Object_Function : Data.s "count" ;{
      Data.i @Std_Queue_Count()
    ;}
    Data.c #Object_Function : Data.s "first" ;{
      Data.i @Std_Queue_First()
    ;}
    Data.c #Object_Function : Data.s "last" ;{
      Data.i @Std_Queue_Last()
    ;}
    Data.c #Object_Function : Data.s "row" ;{
      Data.i @Std_Queue_Row()
    ;}
    Data.c #Object_Function : Data.s "sum" ;{
      Data.i @Std_Queue_Sum()
    ;}
    Data.c #Object_Function : Data.s "join" ;{
      Data.i @Std_Queue_Join()
    ;}
    Data.c #Object_Function : Data.s "average" ;{
      Data.i @Std_Queue_Average()
    ;}
    Data.c #Object_Function : Data.s "random" ;{
      Data.i @Std_Queue_Random()
    Data.s "rand"
    ;}
    Data.c #Object_Group : Data.s "analyse" ;{
      Data.c #Object_Function : Data.s "min" ;{
        Data.i @Std_Queue_Analyse_Min()
      ;}
      Data.c #Object_Function : Data.s "max" ;{
        Data.i @Std_Queue_Analyse_Max()
      ;}
      Data.c #Object_Function : Data.s "variations" ;{
        Data.i @Std_Queue_Analyse_Variations()
      ;}
      Data.c #Null
    ;}
    Data.c #Object_Function : Data.s "combine" ;{
      Data.i @Std_Queue_Combine()
    ;}
    Data.c #Object_Function : Data.s "append" ;{
      Data.i @Std_Queue_Append()
    ;}
    Data.c #Object_Function : Data.s "appendReference" ;{
      Data.i @Std_Queue_AppendReference()
    ;}
    Data.c #Object_Function : Data.s "appendValue" ;{
      Data.i @Std_Queue_AppendValue()
    ;}
    Data.c #Object_Function : Data.s "prepend" ;{
      Data.i @Std_Queue_Prepend()
    ;}
    Data.c #Object_Function : Data.s "prependReference" ;{
      Data.i @Std_Queue_PrependReference()
    ;}
    Data.c #Object_Function : Data.s "prependValue" ;{
      Data.i @Std_Queue_PrependValue()
    ;}
    Data.c #Object_Function : Data.s "remove" ;{
      Data.i @Std_Queue_Remove()
    ;}
    Data.c #Object_Function : Data.s "removeValue" ;{
      Data.i @Std_Queue_RemoveValue()
    ;}
    Data.c #Object_Function : Data.s "clear" ;{
      Data.i @Std_Queue_Clear()
    ;}
    Data.c #Object_Function : Data.s "enum" ;{
      Data.i @Std_Queue_Enumerate()
    ;}
    Data.c #Object_Function : Data.s "fill" ;{
      Data.i @Std_Queue_Fill()
    ;}
    Data.c #Object_Function : Data.s "sort" ;{
      Data.i @Std_Queue_Sort()
    ;}
    Data.c #Object_Function : Data.s "appendQueue" ;{
      Data.i @Std_Queue_AppendQueue()
    ;}
    Data.c #Object_Function : Data.s "prependQueue" ;{
      Data.i @Std_Queue_PrependQueue()
    ;}
    Data.c #Object_Function : Data.s "push" ;{
      Data.i @Std_Queue_Push()
    ;}
    Data.c #Object_Function : Data.s "pushReference" ;{
      Data.i @Std_Queue_PushReference()
    ;}
    Data.c #Object_Function : Data.s "table" ;{
      Data.i @Std_Queue_Table()
    ;}
    Data.c #Object_Function : Data.s _CM(new) ;{
      Data.i @Std_Queue__New()
    ;}
    Data.c #Object_Function : Data.s _CM(def) ;{
      Data.i @Std_Queue__Def()
    ;}
    Data.c #Object_Function : Data.s _CM(del) ;{
      Data.i @Std_Queue__Del()
    ;}
    Data.c #Object_Function : Data.s _CM(true?) ;{
      Data.i @Std_Queue__IsTrue()
    ;}
    Data.c #Object_Function : Data.s _CM(form) ;{
      Data.i @Std_Queue__Form()
    ;}
    Data.c #Object_Function : Data.s _CM(args) ;{
      Data.i @Std_Queue__Args()
    ;}
    Data.c #False
EndDataSection

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 670
; FirstLine = 71
; Folding = AACAAAAAAAAA-
; EnableXP