;**************************************************************
;     String (class)
;
; Author: Josef Sniatecki
; Creation date: 07.04.2009
;**************************************************************
;   Copyright (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i Std_String_Ascii()
  Protected *Pointer.Character, *ResultQueue.Queue
  
  If Len(*Current\This\String\S) = 1
    ProcedureReturn PushIValue(Asc(*Current\This\String\S))
  Else
    *Pointer     = @*Current\This\String\S
    *ResultQueue = CreateQueue(@FreeValue())
    
    While *Pointer\C
      AppendQueueItem(*ResultQueue, CreateIValue(*Pointer\C))
      *Pointer + _Sz(Character)
    Wend
    
    ProcedureReturn PushQueueValue(*ResultQueue)
  EndIf
EndProcedure
Procedure.i Std_String_Left()
  Protected Length.i
  
  If PopIValue(@Length)
    ProcedureReturn PushSValue(Left(*Current\This\String\S, Length))
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_String_Right()
  Protected Length.i
  
  If PopIValue(@Length)
    ProcedureReturn PushSValue(Right(*Current\This\String\S, Length))
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_String_Range()
  Protected IndexA.i, IndexB.i
  
  If PopIValue(@IndexA) And PopIValue(@IndexB)
    ProcedureReturn PushSValue(Mid(*Current\This\String\S, IndexA + 1, IndexB - IndexA + 1))
  Else
    _ErrorP(2)
  EndIf
EndProcedure
Procedure.i Std_String_Insert()
  Protected String.String, Index.i
  
  If PopSValue(@String) And PopIValue(@Index)
    ProcedureReturn PushSValue(InsertString(*Current\This\String\S, String\S, Index))
  Else
    _ErrorP(2)
  EndIf
EndProcedure
Procedure.i Std_String_DoInsert()
  Protected String.String, Index.i, *ThisString.String = *Current\This\String
  
  If PopSValue(@String) And PopIValue(@Index)
    *ThisString\S = InsertString(*ThisString\S, String\S, Index)
  Else
    _ErrorP(2)
  EndIf
EndProcedure
Procedure.i Std_String_Put()
  Protected String.String, Index.i
  
  If PopSValue(@String) And PopIValue(@Index)
    CopyMemory(@String\S, @*Current\This\String\S + Index, Len(String\S))
  EndIf
EndProcedure
Procedure.i Std_String_Length()
  ProcedureReturn PushIValue(Len(*Current\This\String\S))
EndProcedure
Procedure.i Std_String_Fix()
  Protected Length.i
  
  If PopIValue(@Length)
    If Length > 0
      ProcedureReturn PushSValue(LSet(*Current\This\String\S, Length))
    Else
      ProcedureReturn PushSValue(RSet(*Current\This\String\S, -Length))
    EndIf
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_String_UpperCase()
  ProcedureReturn PushSValue(UCase(*Current\This\String\S))
EndProcedure
Procedure.i Std_String_LowerCase()
  ProcedureReturn PushSValue(LCase(*Current\This\String\S))
EndProcedure
Procedure.i Std_String_Replace()
  Protected ToFind.String, ToReplace.String, Result.s
  
  Result = *Current\This\String\S
  While PopSValue(@ToFind) And PopSValue(@ToReplace)
    Result = ReplaceString(Result, ToFind\S, ToReplace\S)
  Wend
  ProcedureReturn PushSValue(Result)
EndProcedure
Procedure.i Std_String_DoReplace()
  Protected ToFind.String, ToReplace.String
  
  While PopSValue(@ToFind) And PopSValue(@ToReplace)
    *Current\This\String\S = ReplaceString(*Current\This\String\S, ToFind\S, ToReplace\S)
  Wend
  ProcedureReturn *Current\This     
EndProcedure
Procedure.i Std_String_Chars()
  Protected *ResultQueue.Queue, *Pointer.Character
  
  *ResultQueue = CreateQueue()
  *Pointer = @*Current\This\String\S
  While *Pointer\C
    AppendQueueItem(*ResultQueue, CreateSValue(Chr(*Pointer\C)))
    *Pointer + _Sz(Character)
  Wend
  
  ProcedureReturn PushQueueValue(*ResultQueue)
EndProcedure
Procedure.i Std_String_Eval()
  Protected *TokenQueue.Queue, *Result.Value
  
  *TokenQueue = CreateQueue(@FreeToken())
  Parse(*TokenQueue, @*Current\This\String\S) : _E("", FreeQueue(*TokenQueue))
  
  OpenLevel(*TokenQueue, 0, 0, 0, #True)
    *Result = RunCommands()
    If *Result
      If *Result\Scope&#Value_Literal
        *Result = DuplicateValue(*Result)
      EndIf
    Else
      *Result = PushVValue()
    EndIf
  CloseLevel()
  
  FreeQueue(*TokenQueue)
  ProcedureReturn *Result
EndProcedure
Procedure.i Std_String_Format_CamelCase()
  ProcedureReturn PushSValue(CamelCase(*Current\This\String\s))
EndProcedure
Procedure.i Std_String_Format_SpacedCase()
  ProcedureReturn PushSValue(SpacedCase(*Current\This\String\s))
EndProcedure
Procedure.i Std_String_Format_Capitalise()
  ProcedureReturn PushSValue(UCase(Left(*Current\This\String\s, 1)) + SStr(*Current\This\String\s, -1))
EndProcedure
Procedure.i Std_String_Format_Decimals()
  Protected *Character.Character, String.s, Decimals.i, Count.i
  
  If PopIValue(@Decimals)
    *Character = @*Current\This\String\S
    While *Character\C
      Select *Character\C
        Case '.'
          String     + "."
          *Character + 1
          While *Character\C
            String + Chr(*Character\C)
            Count  + 1
            If Count = Decimals
              ProcedureReturn PushSValue(String)
            EndIf
            *Character + 1
          Wend
          If Count < Decimals
            ProcedureReturn PushSValue(String + RSet("", Decimals - Count, "0"))
          EndIf
        Default
          String + Chr(*Character\C)
      EndSelect
      *Character + 1
    Wend
    
    ProcedureReturn PushSValue(String + "." + RSet("", Decimals, "0"))
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_String_Split_Lines()
  Protected *Char.Character, *Start.Character
  Protected *Result.Queue = CreateQueue(@FreeValue())
  
  *Char  = @*Current\This\String\S
  *Start = *Char
  
  While *Char\C
    If *Char\C = 13
      AppendQueueItem(*Result, CreateSValue(PeekS(*Start, *Char - *Start)))
      *Char + _Sz(Character)
      If *Char\C <> 10
        *Char - _Sz(Character)
      EndIf
      *Start = *Char + _Sz(Character)
    EndIf
    *Char + _Sz(Character)
  Wend
  
  If *Char <> *Start
    AppendQueueItem(*Result, CreateSValue(PeekS(*Start, *Char - *Start)))
  EndIf
  
  ProcedureReturn PushQueueValue(*Result)
EndProcedure
Procedure.i Std_String_Indent()
  Protected Length.i
  
  If PopIValue(@Length)
    If Length > 0
      ProcedureReturn PushSValue(ReplaceString(*Current\This\String\S, #N, #N + Space(Length)))
    Else
      ProcedureReturn PushSValue(ReplaceString(*Current\This\String\S, #N + Space(-Length), #N))
    EndIf
  EndIf
EndProcedure
Procedure.i Std_String_DoIndent()
  Protected Length.i
  
  If PopIValue(@Length)
    If Length > 0
      *Current\This\String\S = ReplaceString(*Current\This\String\S, #N, #N + Space(Length))
    Else
      *Current\This\String\S = ReplaceString(*Current\This\String\S, #N + Space(-Length), #N)
    EndIf
  EndIf
EndProcedure
Procedure.i Std_String_Reverse()
  ProcedureReturn PushSValue(ReverseString(*Current\This\String\S))
EndProcedure
Procedure.i Std_String__New()
  *Current\This\String = _New(String)
  *Current\This\Scope  | #Value_Binary
EndProcedure
Procedure.i Std_String__Def()
  PopSValue(*Current\This\String)
EndProcedure
Procedure.i Std_String__Del()
  ClearStructure(*Current\This\String, String)
  _Free(*Current\This\Memory)
EndProcedure
Procedure.i Std_String__IsTrue()
  If *Current\This\String\S
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_String__Set()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass
      *Current\This\String\S = Str(*Value\Integer\I)
    Case *FloatClass
      *Current\This\String\S = StrF(*Value\Float\F)
    Case *QuadClass
      *Current\This\String\S = Str(*Value\Quad\Q)
    Case *DoubleClass
      *Current\This\String\S = StrD(*Value\Double\D)
    Case *BooleanClass
      If *Value\Integer\I
        *Current\This\String\S = "True"
      Else
        *Current\This\String\S = "False"
      EndIf
    Case *StringClass
      *Current\This\String\S = *Value\String\S
    Default
      ProcedureReturn #Null
  EndSelect
  
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_String__Add()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *StringClass
      ProcedureReturn PushSValue(*Current\This\String\S + *Value\String\S)
  EndSelect
EndProcedure
Procedure.i Std_String__Sub()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass
      ProcedureReturn PushSValue(Left(*Current\This\String\S, Len(*Current\This\String\S)-*Value\Integer\I))
    Case *QuadClass
      ProcedureReturn PushSValue(Left(*Current\This\String\S, Len(*Current\This\String\S)-*Value\Quad\Q))
    Case *StringClass
      ProcedureReturn PushSValue(RemoveString(*Current\This\String\S, *Value\String\S))
  EndSelect
EndProcedure
Procedure.i Std_String__Mul()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass
      ProcedureReturn PushSValue(MStr(*Current\This\String\S, *Value\Integer\I))
    Case *QuadClass
      ProcedureReturn PushSValue(MStr(*Current\This\String\S, *Value\Quad\Q))
  EndSelect
EndProcedure
Procedure.i Std_String__Div()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *StringClass
      ProcedureReturn PushIValue(CountString(*Current\This\String\S, *Value\String\S))
  EndSelect
EndProcedure
Procedure.i Std_String__DoAdd()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *StringClass
      *Current\This\String\S + *Value\String\S
  EndSelect
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_String__DoSub()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass
      *Current\This\String\S = Left(*Current\This\String\S, Len(*Current\This\String\S)-*Value\Integer\I)
    Case *QuadClass
      *Current\This\String\S = Left(*Current\This\String\S, Len(*Current\This\String\S)-*Value\Quad\Q)
    Case *StringClass
      *Current\This\String\S = RemoveString(*Current\This\String\S, *Value\String\S)
  EndSelect
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_String__DoMul()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *IntegerClass
      *Current\This\String\S = MStr(*Current\This\String\S, *Value\Integer\I)
    Case *QuadClass
      *Current\This\String\S = MStr(*Current\This\String\S, *Value\Quad\Q)
  EndSelect
  ProcedureReturn *Value
EndProcedure
Procedure.i Std_String__Eq()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *StringClass
      If *Current\This\String\S = *Value\String\S
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
  EndSelect
EndProcedure
Procedure.i Std_String__Le()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *StringClass
      If *Current\This\String\S < *Value\String\S
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
  EndSelect
EndProcedure
Procedure.i Std_String__Gr()
  Protected *Value.Value
  
  PopValueAddress(@*Value)
  Select *Value\Class
    Case *StringClass
      If *Current\This\String\S > *Value\String\S
        ProcedureReturn PushBValue(#True)
      Else
        ProcedureReturn PushBValue(#False)
      EndIf
  EndSelect
EndProcedure
Procedure.i Std_String__Form()
  ProcedureReturn PushSValue(*Current\This\String\S)
EndProcedure
Procedure.i Std_String__Args()
  Protected *Value.Value, Result.s

  While PopValueAddress(@*Value)
    If *Value\Class = *IntegerClass
      Result + Mid(*Current\This\String\S, *Value\Integer\I + 1, 1)
    ElseIf *Value\Class = *StringClass
      ProcedureReturn PushIValue(FindString(*Current\This\String\S, *Value\String\S, 1)-1)
    EndIf
  Wend
  ProcedureReturn PushSValue(Result)
EndProcedure

DataSection
  StringClass:
    Data.c #Object_Function : Data.s "ascii" ;{
      Data.i @Std_String_Ascii()
    ;}
    Data.c #Object_Function : Data.s "left" ;{
      Data.i @Std_String_Left()
    ;}
    Data.c #Object_Function : Data.s "right" ;{
      Data.i @Std_String_Right()
    ;}
    Data.c #Object_Function : Data.s "range" ;{
      Data.i @Std_String_Range()
    ;}
    Data.c #Object_Function : Data.s "insert" ;{
      Data.i @Std_String_Insert()
    ;}
    Data.c #Object_Function : Data.s "insert!" ;{
      Data.i @Std_String_DoInsert()
    ;}
    Data.c #Object_Function : Data.s "put" ;{
      Data.i @Std_String_Put()
    ;}
    Data.c #Object_Function : Data.s "length" ;{
      Data.i @Std_String_Length()
    ;}
    Data.c #Object_Function : Data.s "fix" ;{
      Data.i @Std_String_Fix()
    ;}
    Data.c #Object_Function : Data.s "lCase" ;{
      Data.i @Std_String_LowerCase()
    ;}
    Data.c #Object_Function : Data.s "uCase" ;{
      Data.i @Std_String_UpperCase()
    ;}
    Data.c #Object_Function : Data.s "replace" ;{
      Data.i @Std_String_Replace()
    ;}
    Data.c #Object_Function : Data.s "replace!" ;{
      Data.i @Std_String_DoReplace()
    ;}
    Data.c #Object_Function : Data.s "chars" ;{
      Data.i @Std_String_Chars()
    ;}
    Data.c #Object_Function : Data.s "eval" ;{
      Data.i @Std_String_Eval()
    ;}
    Data.c #Object_Group : Data.s "format" ;{
      Data.c #Object_Function : Data.s "camelCase" ;{
        Data.i @Std_String_Format_CamelCase()
      ;}
      Data.c #Object_Function : Data.s "spacedCase" ;{
        Data.i @Std_String_Format_SpacedCase()
      ;}
      Data.c #Object_Function : Data.s "capitalise" ;{
        Data.i @Std_String_Format_Capitalise()
      ;}
      Data.c #Object_Function : Data.s "decimals" ;{
        Data.i @Std_String_Format_Decimals()
      ;}
      Data.c #Null
    ;}
    Data.c #Object_Group : Data.s "split" ;{
      Data.c #Object_Function : Data.s "lines" ;{
        Data.i @Std_String_Split_Lines()
      ;}
      Data.c #Null
    ;}
    Data.c #Object_Function : Data.s "indent" ;{
      Data.i @Std_String_Indent()
    ;}
    Data.c #Object_Function : Data.s "indent!" ;{
      Data.i @Std_String_DoIndent()
    ;}
    Data.c #Object_Function : Data.s "reverse" ;{
      Data.i @Std_String_Reverse()
    ;}
    Data.c #Object_Function : Data.s _CM(new) ;{
      Data.i @Std_String__New()
    ;}
    Data.c #Object_Function : Data.s _CM(def) ;{
      Data.i @Std_String__Def()
    ;}
    Data.c #Object_Function : Data.s _CM(del) ;{
      Data.i @Std_String__Del()
    ;}
    Data.c #Object_Function : Data.s _CM(true?) ;{
      Data.i @Std_String__IsTrue()
    ;}
    Data.c #Object_Function : Data.s _CM(set) ;{
      Data.i @Std_String__Set()
    ;}
    Data.c #Object_Function : Data.s _CM(add) ;{
      Data.i @Std_String__Add()
    ;}
    Data.c #Object_Function : Data.s _CM(sub) ;{
      Data.i @Std_String__Sub()
    ;}
    Data.c #Object_Function : Data.s _CM(mul) ;{
      Data.i @Std_String__Mul()
    ;}
    Data.c #Object_Function : Data.s _CM(div) ;{
      Data.i @Std_String__Div()
    ;}
    Data.c #Object_Function : Data.s _CM(add!) ;{
      Data.i @Std_String__DoAdd()
    ;}
    Data.c #Object_Function : Data.s _CM(sub!) ;{
      Data.i @Std_String__DoSub()
    ;}
    Data.c #Object_Function : Data.s _CM(mul!) ;{
      Data.i @Std_String__DoMul()
    ;}
    Data.c #Object_Function : Data.s _CM(eq) ;{
      Data.i @Std_String__Eq()
    ;}
    Data.c #Object_Function : Data.s _CM(le) ;{
      Data.i @Std_String__Le()
    ;}
    Data.c #Object_Function : Data.s _CM(gr) ;{
      Data.i @Std_String__Gr()
    ;}
    Data.c #Object_Function : Data.s _CM(form) ;{
      Data.i @Std_String__Form()
    ;}
    Data.c #Object_Function : Data.s _CM(args) ;{
      Data.i @Std_String__Args()
    Data.s "indexOf", "charOf"
    ;}
    Data.c #Null
EndDataSection

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 225
; FirstLine = 41
; Folding = AAACAAAAA9AAAw
; EnableXP
; CompileSourceDirectory