;**************************************************************
;     String pointer (class)
;
; Autor: Josef Sniatecki
; Creation date: 15.11.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i Std_StringPointer_Ascii()
  ProcedureReturn PushIValue(*Current\This\StringPointer\Character\C)
EndProcedure
Procedure.i Std_StringPointer_Character()
  ProcedureReturn PushSValue(Chr(*Current\This\StringPointer\Character\C))
EndProcedure
Procedure.i Std_StringPointer_Index()
  Protected *StringPointer.StringPointer = *Current\This\StringPointer
  
  ProcedureReturn PushIValue(*StringPointer\Character - *StringPointer\String)
EndProcedure
Procedure.i Std_StringPointer_Next()
  Protected *StringPointer.StringPointer = *Current\This\StringPointer
  
  If *StringPointer\Character < *StringPointer\EndOfString
    ProcedureReturn PushSPValue(*StringPointer\String, *StringPointer\Character + 1, *StringPointer\EndOfString)
  Else
    _Error("String pointer overflow!")
  EndIf
EndProcedure
Procedure.i Std_StringPointer_Previous()
  Protected *StringPointer.StringPointer = *Current\This\StringPointer
  
  If *StringPointer\Character > *StringPointer\String
    ProcedureReturn PushSPValue(*StringPointer\String, *StringPointer\Character - 1, *StringPointer\EndOfString)
  Else
    _Error("String pointer is behind of string's start!")
  EndIf
EndProcedure
Procedure.i Std_StringPointer__New()
  *Current\This\StringPointer = _New(StringPointer)
EndProcedure
Procedure.i Std_StringPointer__Def()
  Protected *StringPointer.StringPointer = *Current\This\StringPointer, *Value.Value
  
  If PopValueAddress(@*Value)
    Select *Value\Class
      Case *StringClass
        *StringPointer\String      = @*Value\String\S
        *StringPointer\Character   = @*Value\String\S
        *StringPointer\EndOfString = *StringPointer\Character + Len(*Value\String\S)
      Case *StringPointerClass
        *StringPointer\String      = *Value\StringPointer\String
        *StringPointer\Character   = *Value\StringPointer\Character
        *StringPointer\EndOfString = *Value\StringPointer\EndOfString
    EndSelect
  Else
    _ErrorP(1)
  EndIf
EndProcedure
Procedure.i Std_StringPointer__Del()
  _Free(*Current\This\StringPointer)
EndProcedure
Procedure.i Std_StringPointer__IsTrue()
  If *Current\This\StringPointer\Character\C
    ProcedureReturn PushBValue(#True)
  Else
    ProcedureReturn PushBValue(#False)
  EndIf
EndProcedure
Procedure.i Std_StringPointer__Set()
  Protected *StringPointer.StringPointer = *Current\This\StringPointer, *Value.Value
  
  If PopValueAddress(@*Value)
    Select *Value\Class
      Case *IntegerClass
        *StringPointer\Character\C = *Value\Integer\I
      Case *QuadClass
        *StringPointer\Character\C = *Value\Quad\Q
      Case *StringClass
        *StringPointer\String      = @*Value\String\S
        *StringPointer\Character   = @*Value\String\S
        *StringPointer\EndOfString = *StringPointer\Character + Len(*Value\String\S)
      Case *StringPointerClass
        *StringPointer\String      = *Value\StringPointer\String
        *StringPointer\Character   = *Value\StringPointer\Character
        *StringPointer\EndOfString = *Value\StringPointer\EndOfString
      Default
        ProcedureReturn #Null
    EndSelect
    ProcedureReturn *Value
  EndIf
EndProcedure
Procedure.i Std_StringPointer__Add()
  Protected *StringPointer.StringPointer = *Current\This\StringPointer, *Value.Value, Movement.i
  
  If PopValueAddress(@*Value)
    Select *Value\Class
      Case *IntegerClass
        Movement = *Value\Integer\I
      Case *QuadClass
        Movement = *Value\Quad\Q
      Default
        ProcedureReturn #Null
    EndSelect
    If Movement > 0
      If *StringPointer\Character + Movement <= *StringPointer\EndOfString
        ProcedureReturn PushSPValue(*StringPointer\String, *StringPointer\Character + Movement, *StringPointer\EndOfString)
      Else
        _Error("String pointer overflow!")
      EndIf
    ElseIf Movement < 0
      If *StringPointer\Character + Movement => *StringPointer\String
        ProcedureReturn PushSPValue(*StringPointer\String, *StringPointer\Character + Movement, *StringPointer\EndOfString)
      Else
        _Error("String pointer is behind of string's start!")
      EndIf
    EndIf
  EndIf
EndProcedure
Procedure.i Std_StringPointer__Sub()
  Protected *StringPointer.StringPointer = *Current\This\StringPointer, *Value.Value, Movement.i
  
  If PopValueAddress(@*Value)
    Select *Value\Class
      Case *IntegerClass
        Movement = *Value\Integer\I
      Case *QuadClass
        Movement = *Value\Quad\Q
      Default
        ProcedureReturn #Null
    EndSelect
    If Movement > 0
      If *StringPointer\Character - Movement => *StringPointer\String
        ProcedureReturn PushSPValue(*StringPointer\String, *StringPointer\Character - Movement, *StringPointer\EndOfString)
      Else
        _Error("String pointer is behind of string's start!")
      EndIf
    ElseIf Movement < 0
      If *StringPointer\Character - Movement <= *StringPointer\EndOfString
        PushSPValue(*StringPointer\String, *StringPointer\Character - Movement, *StringPointer\EndOfString)
      Else
        _Error("String pointer overflow!")
      EndIf
    EndIf
  EndIf
EndProcedure
Procedure.i Std_StringPointer__Inc()
  Protected *StringPointer.StringPointer = *Current\This\StringPointer
  
  If *StringPointer\Character < *StringPointer\EndOfString
    *StringPointer\Character + _Sz(Character)
    ProcedureReturn *Current\This
  Else
    _Error("String pointer overflow!")
  EndIf
EndProcedure
Procedure.i Std_StringPointer__Dec()
  Protected *StringPointer.StringPointer = *Current\This\StringPointer
  
  If *StringPointer\Character > *StringPointer\String
    *StringPointer\Character - _Sz(Character)
    ProcedureReturn *Current\This
  Else
    _Error("String pointer is behind of string's start!")
  EndIf
EndProcedure
Procedure.i Std_StringPointer__Form()
  Protected *StringPointer.StringPointer = *Current\This\StringPointer
  
  ProcedureReturn PushSValue(ReplaceString(PeekS(*StringPointer\String), #N, "#n") + #N + Space(*StringPointer\Character - *StringPointer\String) + "^" + Space(*StringPointer\EndOfString - *StringPointer\Character - 1))
EndProcedure



DataSection
  StringPointerClass:
    Data.c #Object_Function : Data.s "ascii" ;{
      Data.i @Std_StringPointer_Ascii()
    Data.s "a"
    ;}
    Data.c #Object_Function : Data.s "char" ;{
      Data.i @Std_StringPointer_Character()
    Data.s "c"
    ;}
    Data.c #Object_Function : Data.s "index" ;{
      Data.i @Std_StringPointer_Index()
    ;}
    Data.c #Object_Function : Data.s "next" ;{
      Data.i @Std_StringPointer_Next()
    ;}
    Data.c #Object_Function : Data.s "prev" ;{
      Data.i @Std_StringPointer_Previous()
    ;}
    Data.c #Object_Function : Data.s _CM(new) ;{
      Data.i @Std_StringPointer__New()
    ;}
    Data.c #Object_Function : Data.s _CM(def) ;{
      Data.i @Std_StringPointer__Def()
    ;}
    Data.c #Object_Function : Data.s _CM(true?) ;{
      Data.i @Std_StringPointer__IsTrue()
    ;}
    Data.c #Object_Function : Data.s _CM(set) ;{
      Data.i @Std_StringPointer__Set()
    ;}
    Data.c #Object_Function : Data.s _CM(add) ;{
      Data.i @Std_StringPointer__Add()
    ;}
    Data.c #Object_Function : Data.s _CM(sub) ;{
      Data.i @Std_StringPointer__Sub()
    ;}
    Data.c #Object_Function : Data.s _CM(inc) ;{
      Data.i @Std_StringPointer__Inc()
    ;}
    Data.c #Object_Function : Data.s _CM(dec) ;{
      Data.i @Std_StringPointer__Dec()
    ;}
    Data.c #Object_Function : Data.s _CM(form) ;{
      Data.i @Std_StringPointer__Form()
    ;}
    Data.c #Null
EndDataSection
; IDE Options = PureBasic 4.41 RC 1 (Windows - x86)
; CursorPosition = 149
; FirstLine = 107
; Folding = g-HAg
; EnableXP