;*************************************************************************************
;          ______  ______  ______  ______  __   __  ______
;         / __  / / __  / / ____/ / __  / /  | / / / __  /
;        / / / / / /_/ / / /___  / /_/ / /   |/ / / / / /
;       / / / / / ____/ / ____/ /    _/ / /|   / / / / /
;      / /_/ / / /     / /___  / /| |  / / |  / / /_/ /
;     /_____/ /_/     /_____/ /_/ |_| /_/  |_/ /_____/
;      A dynamic And object-oriented programming language
;
;     E D I T O R
;
;*************************************************************************************
;   (c) 2010 Josef Sniatecki
; 
; This program is free software; you can redistribute it and/or modify it under the
; terms of the GNU General Public License As published by the Free Software Foundation;
; either version 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be useful, but WITHOUT ANY
; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
; PARTICULAR PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public License along With this
; program; if not, see "http://www.gnu.org/licenses/".
;*************************************************************************************



EnableExplicit

;*******************************************************
; Includes
;*******************************************************

IncludePath "..\..\..\..\Includes"
IncludeFile "Standard.pbi"

IncludeFile "EditorGadget.pbi"
IncludeFile "Cursor.pbi"

  Structure QueueItem_Memory
    StructureUnion
      *Memory
      *Value.OpernoValue
      *Object.OpernoObject
      *Item.QueueItem
    EndStructureUnion
  EndStructure
  #Templates_Queue_Memory = #True
IncludeFile "Queue.pbi"

IncludeFile "Operno.pbi"



;******************************************
; Structures
;******************************************

Structure Panel
  Name.s
  Modified.i
  Find_Text.s
  Find_Flags.i
  Gadget.i
EndStructure



;******************************************
; Variables
;******************************************

Global User_Name.s
Global User_Language.s

Global IDE_Font.s
Global IDE_FontSize.i
Global IDE_ForegroundColor.i
Global IDE_BackgroundColor.i

Global NewList Panel.Panel()

Global Event.i, Window.i, Gadget.i, Type.i, Menu.i

Global Integer.i, Integer2.i, String.s, String2.s
Global *Panel.Panel, Parameter.s, Program.i



;******************************************
; Declarations
;******************************************

Declare.i ReadUserOptions()
Declare.i ReadIDEOptions()
Declare.i WriteIDEOptions()

Declare.i Callback(Event.i, lParam.i, wParam.i)

Declare.i UpdateWindowTitle()
Declare.i UpdateViewer()
Declare.i UpdateViewerTree(*Object.OpernoObject, Level.i = 0)

Declare.i Open(Name.s = "")
Declare.i SaveAsCurrentFile()
Declare.i SaveCurrentFile()
Declare.i Close()
Declare.i CloseCurrentFile()
Declare.i CloseAllFiles()

Declare.s CamelCase(Text.s)
Declare.s SpacedCase(Text.s)

Declare.i EditorCallback(Window.i, Message.i, wParam.i, lParam.i)



;******************************************
; Sources
;******************************************

IncludePath ""
IncludeFile "Common.pb"



;******************************************
; Functions
;******************************************

Procedure.i ReadUserOptions()
   OpenPreferences("Data\Options.txt")
     PreferenceGroup("User")
       User_Name     = ReadPreferenceString("Name", "Unknow")
       User_Language = ReadPreferenceString("Language", "German")
   ClosePreferences()
EndProcedure
Procedure.i ReadIDEOptions()
  OpenPreferences("Data\IDE options.txt")
    PreferenceGroup("Style")
      IDE_Font            = ReadPreferenceString("Font", "Courier New")
      IDE_FontSize        = ReadPreferenceInteger("Font size", 10)
      IDE_ForegroundColor = Val("$" + RSet(ReadPreferenceString("Foreground color", "000000"), 6, "0"))
      IDE_BackgroundColor = Val("$" + RSet(ReadPreferenceString("Background color", "FFFFFF"), 6, "0"))
  ClosePreferences()
EndProcedure
Procedure.i WriteIDEOptions()
  CreatePreferences("Data\IDE options.txt")
    PreferenceGroup("Style")
      WritePreferenceString("Font", IDE_Font)
      WritePreferenceInteger("Font size", IDE_FontSize)
      WritePreferenceString("Foreground color", RSet(Hex(IDE_ForegroundColor, #PB_Integer), 6, "0"))
      WritePreferenceString("Background color", RSet(Hex(IDE_BackgroundColor, #PB_Integer), 6, "0"))
  ClosePreferences()
EndProcedure

Procedure.i Callback(Event.i, lParam.i, wParam.i)
  Static Input.s, Start.i
  
  Select Event
    Case #OpernoCallback_Output
      EditorGadgetAppendText(#Main_Output, PeekS(lParam))
    Case #OpernoCallback_Input
      Input = InputRequester("Input", "", "")
      EditorGadgetAppendText(#Main_Output, Input + #N)
      ProcedureReturn @Input
    Case #OpernoCallback_ClearOutput
      SetGadgetText(#Main_Output, "")
    Case #OpernoCallback_Wait
      Start = ElapsedMilliseconds()
      Repeat
        WindowEvent()
        Sleep_(1)
      Until ElapsedMilliseconds() - Start > lParam
  EndSelect
EndProcedure

Procedure.i UpdateWindowTitle()
  Protected Title.s
  
  If Panel()\Name
    If Panel()\Modified
      Title = GetFilePart(Panel()\Name) + "*"
    Else
      Title = GetFilePart(Panel()\Name)
    EndIf
  ElseIf GetGadgetText(Panel()\Gadget)
    Title = "<New>*"
  Else
    Title = "<New>"
  EndIf
  
  SetGadgetItemText(#Main_Panel, ListIndex(Panel()), Title)
EndProcedure
Procedure.i UpdateViewer()
  If IsWindow(#ViewerWindow)
    ClearGadgetItems(#Viewer_Tree)
    UpdateViewerTree(*OpernoMainObject)
    UpdateViewerTree(*OpernoPreprocessorObject)
    UpdateViewerTree(*OpernoStandardLibrary)
    UpdateViewerTree(*OpernoStandardClassLibrary)
  EndIf
EndProcedure
Procedure.i UpdateViewerTree(*Object.OpernoObject, Level.i = 0)
  Protected Text.s, *Item.QueueItem
  
  If Not *Object\Reference
    Select *Object\Type
      Case #OpernoObject_Variable
        If *Object\Variable\Value\Class\Group\ClassMethod[#OpernoClassMethod_Format]
          Text = *Object\Name + "." + OpernoClassName(*Object\Variable\Value\Class) + " = " + OpernoFormatValue(*Object\Variable\Value)
        Else
          Text = *Object\Name + "." + OpernoClassName(*Object\Variable\Value\Class)
        EndIf
      Case #OpernoObject_Function
        Text = *Object\Name + "()"
      Case #OpernoObject_Group
        Text = *Object\Name
    EndSelect
  Else
    Text = *Object\Name + " -> " + OpernoObjectPath(*Object\Reference)
  EndIf
  
  AddGadgetItem(#Viewer_Tree, -1, Text, 0, Level)
  SetGadgetItemData(#Viewer_Tree, CountGadgetItems(#Viewer_Tree) - 1, *Object)
  
  If Not *Object\Reference
    Select *Object\Type
      Case #OpernoObject_Variable
        If *Object\Variable\Value\Scope&#OpernoValue_Searchable And Not *Object\Variable\Value\Scope&#OpernoValue_Reference
          *Item = *Object\Variable\Value\Object\Group\ObjectQueue\First
          While *Item
            UpdateViewerTree(*Item\Object, Level + 1)
            _Move(*Item, N)
          Wend
        EndIf
      Case #OpernoObject_Group
        *Item = *Object\Group\ObjectQueue\First
        While *Item
          UpdateViewerTree(*Item\Object, Level + 1)
          _Move(*Item, N)
        Wend
      Case #OpernoObject_Function
        If OpernoClassMethodID(*Object\Name) <> #None
          SetGadgetItemColor(#Viewer_Tree, CountGadgetItems(#Viewer_Tree) - 1, #PB_Gadget_FrontColor, $A000A0)
        EndIf
    EndSelect
  Else
    SetGadgetItemColor(#Viewer_Tree, CountGadgetItems(#Viewer_Tree) - 1, #PB_Gadget_FrontColor, $808080)
  EndIf
EndProcedure

Procedure.i Open(Name.s = "")
  Static First.i
  
  If LastElement(Panel()) And Panel()\Name = "" And Not GetGadgetText(Panel()\Gadget)
    If Not Name
      ProcedureReturn
    Else
      Close()
    EndIf
  EndIf
  
  AddElement(Panel())
  
  Panel()\Name = Name
  
  If Not Name
    Name = "<New>"
  Else
    Name = GetFilePart(Name)
  EndIf
  
  OpenGadgetList(#Main_Panel)
  AddGadgetItem(#Main_Panel, -1, Name)
    Panel()\Gadget = EditorGadget(#PB_Any, 0, 0, GetGadgetAttribute(#Main_Panel, #PB_Panel_ItemWidth), GetGadgetAttribute(#Main_Panel, #PB_Panel_ItemHeight))
    SetWindowLongPtr_(GadgetID(Panel()\Gadget), #GWLP_USERDATA, SetWindowLongPtr_(GadgetID(Panel()\Gadget), #GWLP_WNDPROC, @EditorCallback()))
  CloseGadgetList()
  
  EditorGadgetModified(Panel()\Gadget)
  
  SetGadgetFont(Panel()\Gadget, FontID(#MainFont))
  SetGadgetColor(Panel()\Gadget, #PB_Gadget_FrontColor, IDE_ForegroundColor)
  SetGadgetColor(Panel()\Gadget, #PB_Gadget_BackColor, IDE_BackgroundColor)
    
  SetGadgetState(#Main_Panel, ListIndex(Panel()))
  SetActiveGadget(Panel()\Gadget)
EndProcedure
Procedure.i SaveAsCurrentFile()
  Panel()\Name = SaveFileRequester("Save", GetCurrentDirectory(), "*.op_c", 0)
  If Panel()\Name
    If Not GetExtensionPart(Panel()\Name)
      Panel()\Name + ".op_c"
    EndIf
    UpdateWindowTitle()
  Else
    ProcedureReturn #False
  EndIf
    
  EditorGadgetSave(Panel()\Gadget, Panel()\Name)
  Panel()\Modified = #False
  UpdateWindowTitle()
  ProcedureReturn #True
EndProcedure
Procedure.i SaveCurrentFile()
  If Not Panel()\Name
    Panel()\Name = SaveFileRequester("Save", GetCurrentDirectory(), "*.op_c", 0)
    If Panel()\Name
      If Not GetExtensionPart(Panel()\Name)
        Panel()\Name + ".op_c"
      EndIf
    Else
      ProcedureReturn #False
    EndIf
  EndIf
  
  EditorGadgetSave(Panel()\Gadget, Panel()\Name)
  Panel()\Modified = #False
  UpdateWindowTitle()
  ProcedureReturn #True
EndProcedure
Procedure.i Close()
  RemoveGadgetItem(#Main_Panel, ListIndex(Panel()))
  DeleteElement(Panel())
EndProcedure
Procedure.i CloseCurrentFile()
  If (Panel()\Name And Panel()\Modified) Or Not Panel()\Name
    If Not Panel()\Name And Not GetGadgetText(Panel()\Gadget)
      Close()
      ProcedureReturn #True
    EndIf
    Select MessageRequester("Warning", "Do you want to save this file?", #PB_MessageRequester_YesNoCancel)
      Case #PB_MessageRequester_Yes
        ProcedureReturn SaveCurrentFile()
      Case #PB_MessageRequester_No
        Close()
        ProcedureReturn #True
      Case #PB_MessageRequester_Cancel
        ProcedureReturn #False
    EndSelect
  Else
    Close()
    ProcedureReturn #True
  EndIf
EndProcedure
Procedure.i CloseAllFiles()
  ForEach Panel()
    If Not CloseCurrentFile()
      ProcedureReturn #False
    EndIf
  Next
  ProcedureReturn #True
EndProcedure

Procedure.s CamelCase(Text.s)
  Protected *Character.Character, Result.s, *Finish
  
  *Finish    = @Text + Len(Text) - 1
  *Character = @Text
  
  While *Character\C
    If *Character\C = ' ' And *Character <> *Finish
      *Character + 1
      If *Character\C <> ' '
        Result + UCase(Chr(*Character\C))
      Else
        *Character - 1
      EndIf
    Else
      Result + Chr(*Character\C)
    EndIf
    *Character + 1
  Wend
  
  ProcedureReturn Result
EndProcedure
Procedure.s SpacedCase(Text.s)
  Protected *Character.Character, Result.s
  
  *Character = @Text
  While *Character\C
    Select *Character\C
      Case 'A' To 'Z'
        Result + " " + LCase(Chr(*Character\C))
      Default
        Result + Chr(*Character\C)
    EndSelect
    *Character + 1
  Wend
  
  ProcedureReturn Result
EndProcedure

Procedure.i EditorCallback(Window.i, Message.i, wParam.i, lParam.i)
  ; Thanks to iostream form www.pure-board.de!
  
  Protected Range.CHARRANGE
  Protected PreviousLine.i, PreviousLineContent.s
  Protected Indent.s, *Pointer.Character
  
  If Message = #WM_CHAR And wParam = #CR
    SendMessage_(Window, #EM_EXGETSEL, 0, @Range)
    
    PreviousLine        = SendMessage_(Window, #EM_EXLINEFROMCHAR, 0, Range\cpMin) - 1
    PreviousLineContent = GetGadgetItemText(Panel()\Gadget, PreviousLine)
    *Pointer            = @PreviousLineContent
    Indent              = ""
    
    While *Pointer\c And (*Pointer\c = 9 Or *Pointer\c = 32)
      *Pointer + _Sz(Character)
    Wend
    Indent = PeekS(@PreviousLineContent, *Pointer - @PreviousLineContent)
    
    SendMessage_(Window, #EM_REPLACESEL, 0, @Indent)
    ProcedureReturn 0
  EndIf
  
  ProcedureReturn CallWindowProc_(GetWindowLongPtr_(Window, #GWLP_USERDATA), Window, Message, wParam, lParam)
EndProcedure



;******************************************
; Configurations
;******************************************

SetCurrentDirectory(GetPathPart(ProgramFilename()))

InitOperno("DLL\Operno.dll")
OpernoCallback(@Callback())



ReadUserOptions()
ReadIDEOptions()



OpenMainWindow()

Parameter = ProgramParameter()
While Parameter
  Open(Parameter)
  EditorGadgetLoad(Panel()\Gadget, Parameter)
  EditorGadgetModified(Panel()\Gadget)
  Parameter = ProgramParameter()
Wend



;******************************************
; Main
;******************************************

Repeat
  ; Get informations about the actual event:
  Event     = WaitWindowEvent()
  Window    = EventWindow()
  Gadget    = EventGadget()
  Type      = EventType()
  Menu      = EventMenu()
  
  Select Window
    Case #MainWindow
      Select Event
        Case #PB_Event_Gadget
          Select Gadget
            Case Panel()\Gadget
              If EditorGadgetModified(Panel()\Gadget)
                Panel()\Modified = #True
                UpdateWindowTitle()
              EndIf
            Case #Main_Splitter
              *Panel = @Panel()
              ForEach Panel()
                ResizeGadget(Panel()\Gadget, #PB_Ignore, #PB_Ignore, GetGadgetAttribute(#Main_Panel, #PB_Panel_ItemWidth), GetGadgetAttribute(#Main_Panel, #PB_Panel_ItemHeight))
              Next
              ChangeCurrentElement(Panel(), *Panel)
            Case #Main_Panel
              SelectElement(Panel(), GetGadgetState(#Main_Panel))
          EndSelect
        Case #PB_Event_Menu
          Select Menu
            Case #MainMenu_New
              Open()
            Case #MainMenu_Open
              String = OpenFileRequester("Open", GetCurrentDirectory(), "*.op_c;*.txt", 0)
              If String
                Open(String)
                EditorGadgetLoad(Panel()\Gadget, String) : EditorGadgetModified(Panel()\Gadget)
                UpdateWindowTitle()
              EndIf
            Case #MainMenu_Save
              SaveCurrentFile()
            Case #MainMenu_SaveAs
              SaveAsCurrentFile()
            Case #MainMenu_Close
              CloseCurrentFile()
              If ListSize(Panel()) = 0
                Open()
              ElseIf ListIndex(Panel()) = -1
                LastElement(Panel())
                SetGadgetState(#Main_Panel, ListIndex(Panel()))
                SetActiveGadget(Panel()\Gadget)
              Else
                SetGadgetState(#Main_Panel, ListIndex(Panel()))
                SetActiveGadget(Panel()\Gadget)
              EndIf
            Case #MainMenu_Quit
              If CloseAllFiles()
                Break
              EndIf
            Case #MainMenu_Undo
              EditorGadgetUndo(Panel()\Gadget)
            Case #MainMenu_Redo
              EditorGadgetRedo(Panel()\Gadget)
            Case #MainMenu_Cut
              EditorGadgetCut(Panel()\Gadget)
            Case #MainMenu_Copy
              EditorGadgetCopy(Panel()\Gadget)
            Case #MainMenu_Paste
              EditorGadgetPaste(Panel()\Gadget)
            Case #MainMenu_SelectAll
              EditorGadgetSelectAll(Panel()\Gadget)
            Case #MainMenu_FindReplace
              OpenFindWindow()
            Case #MainMenu_FindNext
              If Panel()\Find_Text
                If Not EditorGadgetFindText(Panel()\Gadget, Panel()\Find_Text, Panel()\Find_Flags)
                  MessageRequester("Info", "No more found.")
                EndIf
              EndIf
            Case #MainMenu_Date
              EditorGadgetReplaceSelection(Panel()\Gadget, FormatDate("%dd.%mm.%yyyy", Date()))
            Case #MainMenu_Header
              EditorGadgetReplaceSelection(Panel()\Gadget, "'{ ************************ }" + #N + "  " + #N + " { ************************ }'" + #N + #N)
              EditorGadgetSelectXY(Panel()\Gadget, EditorGadgetCursorY(Panel()\Gadget) - 4, 3, EditorGadgetCursorY(Panel()\Gadget) - 4, 3)
            Case #MainMenu_Run
              OpernoOutput(#OpernoSymbol_Message, "Running...")
              EditorGadgetSave(Panel()\Gadget, "Data\Temp.op_c")
              Program = RunProgram("Operno.exe", #Q + "Data\Temp.op_c" + #Q, "", #PB_Program_Open)
              While ProgramRunning(Program)
                WindowEvent()
                Delay(1)
              Wend
              OpernoOutput(#OpernoSymbol_Message, "Instance closed.")
            Case #MainMenu_CreateTokenFile
              String = SaveFileRequester("Compile", GetCurrentDirectory(), "*.op_t", 0)
              If String
                If Not GetExtensionPart(String)
                  String + ".op_t"
                EndIf
                OpernoOutput(#OpernoSymbol_Message, "Generating token file...")
                EditorGadgetSave(Panel()\Gadget, "Data\Temp.op_c")
                Program = RunProgram("Operno.exe", "compile " + #Q + "Data\Temp.op_c" + #Q + #Q + String + #Q, "", #PB_Program_Open)
                While ProgramRunning(Program)
                  WindowEvent()
                  Delay(1)
                Wend
                OpernoOutput(#OpernoSymbol_Message, "Generating token file finished.")
              EndIf
            Case #MainMenu_RunInInstance
              String = GetGadgetText(Panel()\Gadget)
              OpernoOutput(5, String)
              OpernoEvalCommands(String)
              If OpernoError()
                MessageRequester("Operno error", OpernoErrorMessage())
              Else
                SetGadgetText(Panel()\Gadget, "")
              EndIf
              UpdateViewer()
              If OpernoTruncation() And CloseCurrentFile()
                Break
              Else
                OpernoSetTruncation(0)
              EndIf
            Case #MainMenu_NewInstance
              OpernoOutput(#OpernoSymbol_Message, "Reloading...")
              ReloadOperno()
              OpernoCallback(@Callback())
              OpernoOutput(#OpernoSymbol_Message, "Reloading finished.")
              UpdateViewer()
            Case #MainMenu_ClearOutput
              SetGadgetText(#Main_Output, "")
            Case #MainMenu_OpenOperno
              RunProgram("Operno.exe", "cns", "")
            Case #MainMenu_Font
              If FontRequester(IDE_Font, IDE_FontSize, 0)
                IDE_Font     = SelectedFontName()
                IDE_FontSize = SelectedFontSize()
                FreeFont(#MainFont)
                LoadFont(#MainFont, IDE_Font, IDE_FontSize)
                *Panel = @Panel()
                ForEach Panel()
                  SetGadgetFont(Panel()\Gadget, FontID(#MainFont))
                Next
                ChangeCurrentElement(Panel(), *Panel)
                SetGadgetFont(#Main_Output, FontID(#MainFont))
              EndIf
            Case #MainMenu_ForegroundColor
              Integer = ColorRequester(IDE_ForegroundColor)
              If Integer <> -1
                IDE_ForegroundColor = Integer
                *Panel = @Panel()
                ForEach Panel()
                  SetGadgetColor(Panel()\Gadget, #PB_Gadget_FrontColor, IDE_ForegroundColor)
                Next
                ChangeCurrentElement(Panel(), *Panel)
                SetGadgetColor(#Main_Output, #PB_Gadget_FrontColor, IDE_ForegroundColor)
              EndIf
            Case #MainMenu_BackgroundColor
              Integer = ColorRequester(IDE_BackgroundColor)
              If Integer <> -1
                IDE_BackgroundColor = Integer
                *Panel = @Panel()
                ForEach Panel()
                  SetGadgetColor(Panel()\Gadget, #PB_Gadget_BackColor, IDE_BackgroundColor)
                Next
                ChangeCurrentElement(Panel(), *Panel)
                SetGadgetColor(#Main_Output, #PB_Gadget_BackColor, IDE_BackgroundColor)
              EndIf
            Case #MainMenu_Viewer
              OpenViewerWindow()
            Case #MainMenu_FunctionPlotter
              RunProgram("Applications\Function Plotter\FunctionPlotter.exe")
            Case #MainMenu_Naltor
              RunProgram("Applications\Naltor\Naltor.exe")
            Case #MainMenu_GenerateHTML
              EditorGadgetSave(Panel()\Gadget, "Data\Temp.op_c")
              OpernoOutput(#OpernoSymbol_Message, "Generate HTML file...")
              OpernoGenerateHTMLFile("Data\Temp.op_c", "Data\Temp.html")
              OpernoOutput(#OpernoSymbol_Message, "Finished")
              RunProgram(GetPathPart(ProgramFilename()) + "Data\Temp.html")
            Case #MainMenu_ConfigureGenerator
              DisableWindow(#MainWindow, #True)
              OpernoConfigureHTMLGenerator()
              DisableWindow(#MainWindow, #False)
            Case #MainMenu_Help
              RunProgram(GetPathPart(ProgramFilename()) + "Help\" + LCase(User_Language) + "\Index.html")
            Case #MainMenu_Info
              MessageRequester("Info", "Operno Editor (Version 1.10)" + #N + "Copyright " + #C + " 2010 Josef Sniatecki" + #N + #N + "Operno Version: " + OpernoVersion())
            Case #MainMenu_Tutorial
              RunProgram("Operno.exe", #Q + "Tutorials\" + LCase(User_Language) + "\action.op_c" + #Q, GetPathPart(ProgramFilename()))
          EndSelect
        Case #WM_RBUTTONDOWN
          If CursorOverGadget(#MainWindow, Panel()\Gadget)
            DisplayPopupMenu(#MainPopupMenu, WindowID(#MainWindow))
          EndIf
        Case #PB_Event_SizeWindow
          ResizeGadget(#Main_Splitter, #PB_Ignore, #PB_Ignore, WindowWidth(#MainWindow), WindowHeight(#MainWindow) - MenuHeight())
          *Panel = @Panel()
          ForEach Panel()
            ResizeGadget(Panel()\Gadget, #PB_Ignore, #PB_Ignore, GetGadgetAttribute(#Main_Panel, #PB_Panel_ItemWidth), GetGadgetAttribute(#Main_Panel, #PB_Panel_ItemHeight))
          Next
          ChangeCurrentElement(Panel(), *Panel)
        Case #PB_Event_GadgetDrop
          Select EventDropType()
            Case #PB_Drop_Files
              String = EventDropFiles()
              Integer2 = CountString(String, Chr(10)) + 1
              For Integer = 1 To Integer2
                String2 = StringField(String, Integer, Chr(10))
                Open(String2)
                EditorGadgetLoad(Panel()\Gadget, String2)
                EditorGadgetModified(Panel()\Gadget)
              Next
            Case #PB_Drop_Text
              Open()
              SetGadgetText(Panel()\Gadget, EventDropText())
              EditorGadgetModified(Panel()\Gadget)
          EndSelect
        Case #PB_Event_CloseWindow
          If CloseAllFiles()
            Break
          EndIf
      EndSelect
    Case #FindWindow
      Select Event
        Case #PB_Event_Gadget
          Select Gadget
            Case #Find_DoFind
              Integer = (#PB_FindText_CaseSensitive*Is(GetGadgetState(#Find_CheckCaseSensitive)))|(#PB_FindText_WholeWord*Is(GetGadgetState(#Find_CheckWholeWords)))
              If Not EditorGadgetFindText(Panel()\Gadget, GetGadgetText(#Find_ToFind), Integer)
                MessageRequester("Info", "No more found.")
              EndIf
            Case #Find_DoReplaceAll
              Integer = (#PB_FindText_CaseSensitive*Is(GetGadgetState(#Find_CheckCaseSensitive)))|(#PB_FindText_WholeWord*Is(GetGadgetState(#Find_CheckWholeWords)))
              Integer = EditorGadgetReplaceAll(Panel()\Gadget, GetGadgetText(#Find_ToFind), GetGadgetText(#Find_ToReplace), Integer)
              MessageRequester("Info", Form(Integer, "word", "words") + " replaced.")
            Case #Find_DoReplace
              Integer = (#PB_FindText_CaseSensitive*Is(GetGadgetState(#Find_CheckCaseSensitive)))|(#PB_FindText_WholeWord*Is(GetGadgetState(#Find_CheckWholeWords)))
              If EditorGadgetFindText(Panel()\Gadget, GetGadgetText(#Find_ToFind), Integer)
                EditorGadgetReplaceSelection(Panel()\Gadget, GetGadgetText(#Find_ToReplace))
              Else
                MessageRequester("Info", "Not found!")
              EndIf
            Case #Find_DoCancel
              Panel()\Find_Text  = GetGadgetText(#Find_ToFind)
              Panel()\Find_Flags = (#PB_FindText_CaseSensitive*Is(GetGadgetState(#Find_CheckCaseSensitive)))|(#PB_FindText_WholeWord*Is(GetGadgetState(#Find_CheckWholeWords)))
              CloseWindow(#FindWindow)
          EndSelect
        Case #PB_Event_CloseWindow
          Panel()\Find_Text  = GetGadgetText(#Find_ToFind)
          Panel()\Find_Flags = (#PB_FindText_CaseSensitive*Is(GetGadgetState(#Find_CheckCaseSensitive)))|(#PB_FindText_WholeWord*Is(GetGadgetState(#Find_CheckWholeWords)))
          CloseWindow(#FindWindow)
      EndSelect
    Case #ViewerWindow
      Select Event
        Case #PB_Event_Gadget
          If Gadget = #Viewer_Tree
            Select Type
              Case #PB_EventType_LeftDoubleClick
                Integer = GetGadgetState(#Viewer_Tree)
                If Integer > -1
                  String = OpernoObjectPath(GetGadgetItemData(#Viewer_Tree, Integer))
                  If Left(String, 4) = "pre\"
                    String = "#" + SStr(String, -4)
                  EndIf
                  EditorGadgetReplaceSelection(Panel()\Gadget, SpacedCase(String))
                EndIf
            EndSelect
          EndIf
        Case #PB_Event_SizeWindow
          ResizeGadget(#Viewer_Tree, #PB_Ignore, #PB_Ignore, WindowWidth(#ViewerWindow), WindowHeight(#ViewerWindow))
        Case #PB_Event_CloseWindow
          CloseWindow(#ViewerWindow)
      EndSelect
  EndSelect
ForEver



WriteIDEOptions()



End
; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 641
; FirstLine = 348
; Folding = DAA-
; EnableXP
; UseIcon = C:\Users\Josef Sniatecki\Pictures\Operno\Application Icon.ico