;**************************************************************
;     Executers
;
; Author: Josef Sniatecki
; Creation date: 07.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i Execute(Code.s, *Object.Object = 0, SharedStack.i = #False)
  Protected *TokenQueue.Queue, *Result.Value, *Value.Value
  
  *TokenQueue = CreateQueue(@FreeToken())
  Parse(*TokenQueue, @Code) : _E("", FreeQueue(*TokenQueue))
  
  *Result = PushVValue()
  
  OpenLevel(*TokenQueue, 0, *Object, 0, SharedStack)
    *Value = Run()
    If Not ErrorMessage And *Value
      If SharedStack And Not *Value\Scope&#Value_Literal
        *Result = *Value
      ElseIf *Value\Class <> *VariantClass
        SetValue(*Result, *Value)
      EndIf
    Else
      *Result = #Null
    EndIf
  CloseLevel()
  
  FreeQueue(*TokenQueue)
  ProcedureReturn *Result
EndProcedure
Procedure.i ExecuteI(Code.s, *Object.Object = 0, SharedStack.i = #False)
  Protected *TokenQueue.Queue, Integer.i, *Value.Value
  
  *TokenQueue = CreateQueue(@FreeToken())
  Parse(*TokenQueue, @Code) : _E("", FreeQueue(*TokenQueue))
  
  OpenLevel(*TokenQueue, 0, *Object, 0, SharedStack)
    *Value = Run()
    If Not Error And *Value
      Integer = ValueI(*Value)
    EndIf
  CloseLevel()
  
  FreeQueue(*TokenQueue)
  ProcedureReturn Integer
EndProcedure
Procedure.f ExecuteF(Code.s, *Object.Object = 0, SharedStack.i = #False)
  Protected *TokenQueue.Queue, Float.f, *Value.Value
  
  *TokenQueue = CreateQueue(@FreeToken())
  Parse(*TokenQueue, @Code) : _E("", FreeQueue(*TokenQueue))
  
  OpenLevel(*TokenQueue, 0, *Object, 0, SharedStack)
    *Value = Run()
    If Not Error And *Value
      Float = ValueF(*Value)
    EndIf
  CloseLevel()
  
  FreeQueue(*TokenQueue)
  ProcedureReturn Float
EndProcedure
Procedure.q ExecuteQ(Code.s, *Object.Object = 0, SharedStack.i = #False)
  Protected *TokenQueue.Queue, Quad.q, *Value.Value
  
  *TokenQueue = CreateQueue(@FreeToken())
  Parse(*TokenQueue, @Code) : _E("", FreeQueue(*TokenQueue))
  
  OpenLevel(*TokenQueue, 0, *Object, 0, SharedStack)
    *Value = Run()
    If Not Error And *Value
      Quad = ValueQ(*Value)
    EndIf
  CloseLevel()
  
  FreeQueue(*TokenQueue)
  ProcedureReturn Quad
EndProcedure
Procedure.d ExecuteD(Code.s, *Object.Object = 0, SharedStack.i = #False)
  Protected *TokenQueue.Queue, Double.d, *Value.Value
  
  *TokenQueue = CreateQueue(@FreeToken())
  Parse(*TokenQueue, @Code) : _E("", FreeQueue(*TokenQueue))
  
  OpenLevel(*TokenQueue, 0, *Object, 0, SharedStack)
    *Value = Run()
    If Not Error And *Value
      Double = ValueD(*Value)
    EndIf
  CloseLevel()
  
  FreeQueue(*TokenQueue)
  ProcedureReturn Double
EndProcedure  
Procedure.s ExecuteS(Code.s, *Object.Object = 0, SharedStack.i = #False)
  Protected *TokenQueue.Queue, String.s, *Value.Value
  
  *TokenQueue = CreateQueue(@FreeToken())
  Parse(*TokenQueue, @Code) : _E("", FreeQueue(*TokenQueue))
  
  OpenLevel(*TokenQueue, 0, *Object, 0, SharedStack)
    *Value = Run()
    If Not Error And *Value
      String = ValueS(*Value)
    EndIf
  CloseLevel()
  
  FreeQueue(*TokenQueue)
  ProcedureReturn String
EndProcedure
Procedure.i ExecuteCommands(Code.s, *Object.Object = 0, SharedStack.i = #False)
  Protected *TokenQueue.Queue, *Result.Value, *Value.Value
  
  *TokenQueue = CreateQueue(@FreeToken())
  Parse(*TokenQueue, @Code) : _E("", FreeQueue(*TokenQueue))
  
  *Result = PushVValue()
  
  OpenLevel(*TokenQueue, 0, *Object, 0, SharedStack)
    *Value = RunCommands()
    If Not Error And *Value
      If SharedStack And Not *Value\Scope&#Value_Literal
        *Result = *Value
      Else
        SetValue(*Result, *Value)
      EndIf
    Else
      *Result = #Null
    EndIf
  CloseLevel()
  
  FreeQueue(*TokenQueue)
  ProcedureReturn *Result
EndProcedure
Procedure.i ExecuteObject(Path.s, *Super.Object, Mode.i, *ResultThis.Integer = 0, EnableError.i = #True)
  Protected *TokenQueue.Queue, *Object.Object
  
  *TokenQueue = CreateQueue(@FreeToken())
  Parse(*TokenQueue, @Path) : _E("", FreeQueue(*TokenQueue))
  
  OpenLevel(*TokenQueue, 0, 0, 0)
    *Object = RunObjectPath(*Super, Mode, "", 0, *ResultThis, EnableError)
  CloseLevel()
  
  FreeQueue(*TokenQueue)
  ProcedureReturn *Object
EndProcedure
Procedure.i ExecuteFile(Name.s)
  Protected File.i, *Code, *TokenQueue.Queue
  Protected Window.i, TextGadget.i
  
  Select GetExtensionPart(Name)
    Case "op_t"
      *TokenQueue = CreateQueue(@FreeToken())
      LoadTokenQueue(Name, *TokenQueue)
    Default
      File = ReadFile(#Any, Name)
      If File
        *Code = AllocateMemory(Lof(File) + 1)
        ReadData(File, *Code, Lof(File))
        
        *TokenQueue = CreateQueue(@FreeToken())
        Parse(*TokenQueue, *Code)
        FreeMemory(*Code)
        CloseFile(File)
      Else
        _Error("File not found: " + #Q + Name + #Q, CloseWindow(Window))
      EndIf
  EndSelect
  
  _E()
  
  OpenLevel(*TokenQueue, 0, 0, 0)
    RunCommands()
  CloseLevel()
  
  FreeQueue(*TokenQueue)
EndProcedure

Procedure.i ImportLibrary(Name.s)
  Protected Library.i
  
  Library = OpenLibrary(#PB_Any, GetCurrentDirectory() + "Libraries\" + Name + ".dll")
  If Not Library
    _Error("Can not import the specified library: " + #Q + Name + #Q)
  EndIf
  
  CallFunction(Library, "OpernoGetFunctions", ?FunctionTable)
  CallFunction(Library, "InitLibrary")
EndProcedure
; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 208
; FirstLine = 171
; Folding = --
; EnableXP