;**************************************************************
;     HTML generator
;
; Autor: Josef Sniatecki
; Creation date: 15.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i GenerateHTMLFile(SourceFilename.s, HTMLFilename.s)
  Protected SourceFile.i, File.i, *Code
  
  SourceFile = ReadFile(#PB_Any, SourceFilename)
  If SourceFile
    *Code = AllocateMemory(Lof(SourceFile) + 1)
    ReadData(SourceFile, *Code, Lof(SourceFile))
    
    File = CreateFile(#PB_Any, HTMLFilename)
      WriteStringN(File, "<html>")
      WriteStringN(File, "<style type=" + #Q + "text/css" + #Q + ">")
      HTMLCodeStyles(File)
      WriteStringN(File, "</style>")
      WriteStringN(File, "<body style=" + #Q + "font-family:'" + HTMLGenerator_Font + "',Consolas,'Courier New'" + #Q + ">")
        GenerateHTMLCode(File, *Code)
      WriteStringN(File, "</body></html>")
    CloseFile(File)
    
    FreeMemory(*Code)
    CloseFile(SourceFile)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i GenerateHTMLCode(File.i, *Pointer.Character)
  Protected *Start.Character, String.s, Integer.i
  
  Repeat
    Select *Pointer\C
      Case 'a' To 'z', 'A' To 'Z', '_', 192 To 255, ''
        ; Name:
        *Start = *Pointer
        *Pointer + 1
        Repeat
          Select *Pointer\C
            Case 'a' To 'z', 'A' To 'Z', '_', '0' To '9', 192 To 255, '!', '$', '?', ''
              *Pointer + 1
            Default
              *Pointer-1
              Break
          EndSelect
        ForEver
        ; Read name:
        String = PeekS(*Start, *Pointer-*Start + 1)
        ; Generate Code:
        If KeywordID(String) <> -1 Or String = "as" Or String = "begin" Or String = "finish" Or String = "method"
          WriteString(File, GenerateHTMLString(String, #HTMLGenerator_Tag_Keyword))
        ElseIf ClassMethodID(String) <> -1
          WriteString(File, GenerateHTMLString(String, #HTMLGenerator_Tag_ClassMethod))
        Else
          WriteString(File, GenerateHTMLString(String, #HTMLGenerator_Tag_Default))
        EndIf
      Case '0' To '9'
        ; Number:
        *Start = *Pointer
        *Pointer + 1
        Select *Pointer\C
          Case '$'
            ; Binary:
            *Pointer + 1
            Repeat
              Select *Pointer\C
                Case '0', 'o', 'O', 'Q', '1', 'i', 'I'
                  *Pointer + 1
                Default
                  *Pointer - 1
                  Break
              EndSelect
            ForEver
          Case '#'
            ; Hex:
            *Pointer + 1
            Repeat
              Select *Pointer\C
                Case '0' To '9', 'a' To 'f', 'A' To 'F'
                  *Pointer + 1
                Default
                  *Pointer - 1
                  Break
              EndSelect
            ForEver
          Case '0' To '9', '.', ''
            ; Decimal:
            Repeat
              Select *Pointer\C
                Case '0' To '9'
                  *Pointer + 1
                Case '.'
                  *Pointer + 1
                  If *Pointer\C = '.'
                    *Pointer - 2
                    Break
                  EndIf
                Case ''
                  Break
                Default
                  *Pointer - 1
                  Break
              EndSelect
            ForEver
          Default
            ; Only null:
            *Pointer - 1
        EndSelect
        ; Generate Code:
        WriteString(File, GenerateHTMLString(PeekS(*Start, *Pointer - *Start + 1), #HTMLGenerator_Tag_Number))
      Case '"'
        ; String:
        *Start = *Pointer
        *Pointer = Parser_IgnoreString(*Pointer)
        WriteString(File, GenerateHTMLString(PeekS(*Start, *Pointer - *Start + 1), #HTMLGenerator_Tag_String))
      Case '+', '-', '*', '/', '=', '<', '>', '&', '%', '|', '!'
        ; Operator:
        *Start = *Pointer
        *Pointer + 1
        Repeat
          Select *Pointer\C
            Case '+', '-', '*', '/', '=', '<', '>', '&', '%', '|', '!'
              *Pointer + 1
            Default
              *Pointer - 1
              Break
          EndSelect
        ForEver
        ; Generate code:
        WriteString(File, GenerateHTMLString(PeekS(*Start, *Pointer-*Start + 1), #HTMLGenerator_Tag_Operator))
      Case '(', ')', '[', ']', '{', '}'
        ; Bracket:
        WriteString(File, GenerateHTMLString(Chr(*Pointer\C), #HTMLGenerator_Tag_Bracket))
      Case '\'
        *Pointer + 1
        If *Pointer\C = '\'
          WriteString(File, GenerateHTMLString("\\", #HTMLGenerator_Tag_Character))
        Else
          *Pointer-1
          WriteString(File, GenerateHTMLString("\", #HTMLGenerator_Tag_Character))
        EndIf
      Case '.'
        *Pointer + 1
        If *Pointer\C = '.'
          *Pointer + 1
          If *Pointer\C = '.'
            WriteString(File, GenerateHTMLString("...", #HTMLGenerator_Tag_Character))
          Else
            *Pointer - 1
            WriteString(File, GenerateHTMLString("..", #HTMLGenerator_Tag_Character))
          EndIf
        Else
          *Pointer - 1
          WriteString(File, GenerateHTMLString(".", #HTMLGenerator_Tag_Character))
        EndIf
      Case ':'
        *Pointer + 1
        If *Pointer\C = ':'
          WriteString(File, GenerateHTMLString("::", #HTMLGenerator_Tag_Character))
        Else
          *Pointer-1
          WriteString(File, GenerateHTMLString(":", #HTMLGenerator_Tag_Character))
        EndIf
      Case '@'
        WriteString(File, GenerateHTMLString("@", #HTMLGenerator_Tag_Character))
      Case ',', ';'
        WriteString(File, GenerateHTMLString(Chr(*Pointer\C), #HTMLGenerator_Tag_Seperator))
      Case '#'
        ; Preprozessor symbol:
        *Pointer + 1
        Select *Pointer\C
          Case 'a' To 'z', 'A' To 'Z', '_', 192 To 255
            *Start   = *Pointer
            *Pointer = Parser_GetName(*Pointer)
            WriteString(File, GenerateHTMLString("#" + PeekS(*Start, *Pointer-*Start + 1), #HTMLGenerator_Tag_Symbol))
          Case '$'
            *Pointer + 1
            WriteString(File, GenerateHTMLString("#$" + Chr(*Pointer\C), #HTMLGenerator_Tag_Symbol))
          Default
            *Pointer - 1
            WriteString(File, GenerateHTMLString("#", #HTMLGenerator_Tag_Symbol))
        EndSelect
      Case 39
        *Start = *Pointer
        *Pointer = Parser_IgnoreComment(*Start)
        WriteString(File, GenerateHTMLString(PeekS(*Start, *Pointer-*Start + 1), #HTMLGenerator_Tag_Comment))
      Case ' '
        WriteString(File, "&nbsp;")
      Case 9
        WriteString(File, "&nbsp;&nbsp;")
      Case 13
        *Pointer + 1
        WriteStringN(File, "<br/>")
      Case 0, 1
        Break
      Default
        WriteString(File, GenerateHTMLString(Chr(*Pointer\C), #HTMLGenerator_Tag_Default))
    EndSelect
    *Pointer + 1
  ForEver
  
  WriteStringN(File, "") ;End last line
    
  ProcedureReturn *Pointer
EndProcedure
Procedure.i HTMLCodeStyles(File.i)
  Protected Tag.i, Style.s
  
  For Tag = 0 To #HTMLGenerator_TagCount
    Style = "span." + LCase(Left(HTMLGenerator_TagName(Tag), 3)) + "{"
    If HTMLGenerator_Tag(Tag)\Bold
      Style + "font-weight:bold;"
    EndIf
    If HTMLGenerator_Tag(Tag)\Italic
      Style + "font-style:italic;"
    EndIf
    If HTMLGenerator_Tag(Tag)\Underline
      Style + "text-decoration:underline;"
    EndIf
    If HTMLGenerator_Tag(Tag)\Color
      Style + "color:#" + RSet(Hex(TurnColor(HTMLGenerator_Tag(Tag)\Color)), 6, "0") + ";"
    EndIf
    Style + "}"
    WriteStringN(File, Style)
  Next
  
  WriteStringN(File, "h2{font-family:Sans-Serif;text-decoration:underline;margin-left:20px;}")
  WriteStringN(File, "p.info{font-family:Sans-Serif;background-color:#E0E0E0;padding:5px;margin:0px 10px 0px 10px;font-style:italic;font-size:14px;line-height:15px;border-width:1px;border-style:solid;border-color:#A0A0A0;text-align:justify;}")
EndProcedure
Procedure.s HTMLString(String.s)
  Protected NewString.s, *Pointer.Character
  
  *Pointer = @String
  While *Pointer\C
    Select *Pointer\C
      Case ' '
        NewString + "&nbsp;"
      Case 34
        NewString + "&quot;"
      Case '&'
        NewString + "&amp;"
      Case '<'
        NewString + "&lt;"
      Case '>'
        NewString + "&gt;"
      Case 13
        NewString + "<br/>"
        *Pointer + 1
      Default
        NewString + Chr(*Pointer\C)
    EndSelect
    *Pointer + _Sz(Character)
  Wend
  
  ProcedureReturn NewString
EndProcedure
Procedure.s GenerateHTMLString(String.s, Tag.i)
  ProcedureReturn "<span class=" + #Q + LCase(Left(HTMLGenerator_TagName(Tag), 3)) + #Q + ">" + HTMLString(String) + "</span>"
EndProcedure
Procedure.i ExtractHTMLGeneratorTags(Array Tag.HTMLGenerator_Tag(1), *Memory.Integer)
  Protected Index.i, *Tag.HTMLGenerator_Tag
  
  *Tag = @Tag(0)
  For Index = 0 To #HTMLGenerator_TagCount
    *Tag\Bold      = *Memory\I : *Memory + _Sz(Integer)
    *Tag\Italic    = *Memory\I : *Memory + _Sz(Integer)
    *Tag\Underline = *Memory\I : *Memory + _Sz(Integer)
    *Tag\Color     = *Memory\I : *Memory + _Sz(Integer)
    *Tag + _Sz(HTMLGenerator_Tag)
  Next
EndProcedure
Procedure.i HTMLGeneratorConfiguration()
  Protected Window.i
  Protected OKButton.i, CancelButton.i, DefaultsButton.i
  Protected GenericFrame.i, FontNameText.i, FontNameString.i, SelectFontButton.i
  Protected TagConfigurationFrame.i, BoldCheckBox.i, ItalicCheckBox.i, UnderlineCheckBox.i
  Protected HexString.i, SelectColorButton.i
  Protected TagList.i, PreviewFrame.i, PreviewText.i, PreviewFont.i
  Protected Font.s
  Protected Dim Tag.HTMLGenerator_Tag(#HTMLGenerator_TagCount)
  Protected *Tag.HTMLGenerator_Tag
  Protected Event.i, EventWindow.i, EventGadget.i, EventType.i, Refresh.i
  Protected Index.i, String.s, SelectedColor.i
  
  ; Open the window and all gadgets:
  Window = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, 360, 250, "HTML configuration", #PB_Window_ScreenCentered|#PB_Window_SystemMenu|#PB_Window_TitleBar)
    OKButton       = ButtonGadget(#PB_Any, 10, 210, 70, 30, "OK")
    CancelButton   = ButtonGadget(#PB_Any, 80, 210, 70, 30, "Cancel")
    DefaultsButton = ButtonGadget(#PB_Any, 150, 210, 70, 30, "Defaults")
    
    GenericFrame     = Frame3DGadget(#PB_Any, 10, 10, 210, 40, "Generic")
    FontNameText     = TextGadget(#PB_Any, 20, 28, 30, 13, "Font:")
    FontNameString   = StringGadget(#PB_Any, 50, 25, 120, 20, "", #PB_String_ReadOnly)
    SelectFontButton = ButtonGadget(#PB_Any, 170, 25, 40, 20, "...")
    
    TagConfigurationFrame = Frame3DGadget(#PB_Any, 10, 50, 210, 150, "Tag-configuration")
    BoldCheckBox          = CheckBoxGadget(#PB_Any, 20, 70, 40, 20, "Bold")
    ItalicCheckBox        = CheckBoxGadget(#PB_Any, 20, 90, 40, 20, "Italic")
    UnderlineCheckBox     = CheckBoxGadget(#PB_Any, 20, 110, 60, 20, "Underline")
    HexString             = StringGadget(#PB_Any, 20, 165, 150, 20, "", #PB_String_ReadOnly)
    SelectColorButton     = ButtonGadget(#PB_Any, 170, 165, 40, 20, "...")
    
    TagList      = ListViewGadget(#PB_Any, 230, 10, 120, 150)
    PreviewFrame = Frame3DGadget(#PB_Any, 230, 160, 120, 80, "Preview")
    PreviewText  = TextGadget(#PB_Any, 240, 180, 100, 50, "", #PB_Text_Center)
  
  
  
  ; Initializations:
  SetGadgetText(FontNameString, HTMLGenerator_Font) : Font = HTMLGenerator_Font
  For Index = 0 To #HTMLGenerator_TagCount
    Tag(Index)\Bold      = HTMLGenerator_Tag(Index)\Bold
    Tag(Index)\Italic    = HTMLGenerator_Tag(Index)\Italic
    Tag(Index)\Underline = HTMLGenerator_Tag(Index)\Underline
    Tag(Index)\Color     = HTMLGenerator_Tag(Index)\Color
    
    AddGadgetItem(TagList, -1, HTMLGenerator_TagName(Index))
  Next
  SetGadgetState(TagList,  0) : *Tag = @Tag(0)
  
  SetGadgetState(BoldCheckBox, *Tag\Bold)
  SetGadgetState(ItalicCheckBox, *Tag\Italic)
  SetGadgetState(UnderlineCheckBox, *Tag\Underline)
  SetGadgetText(HexString, RSet(Hex(*Tag\Color), 6, "0"))
  
  PreviewFont = LoadFont(#PB_Any, Font, 12, (#PB_Font_Bold**Tag\Bold)|(#PB_Font_Italic**Tag\Italic)|(#PB_Font_Underline**Tag\Underline))
  SetGadgetFont(PreviewText, FontID(PreviewFont))
  SetGadgetColor(PreviewText, #PB_Gadget_FrontColor, *Tag\Color)
  SetGadgetText(PreviewText, HTMLGenerator_Example(0))
  
  
  
  ; Event loop:
  Repeat
    Event       = WaitWindowEvent()
    EventWindow = EventWindow()
    EventGadget = EventGadget()
    EventType   = EventType()
    
    Select EventWindow
      Case Window
        Select Event
          Case #PB_Event_Gadget
            Select EventGadget
              Case OKButton
                ; Apply all configurations:
                HTMLGenerator_Font     = GetGadgetText(FontNameString)
                For Index = 0 To #HTMLGenerator_TagCount
                  HTMLGenerator_Tag(Index)\Bold      = Tag(Index)\Bold
                  HTMLGenerator_Tag(Index)\Italic    = Tag(Index)\Italic
                  HTMLGenerator_Tag(Index)\Underline = Tag(Index)\Underline
                  HTMLGenerator_Tag(Index)\Color     = Tag(Index)\Color
                Next
                Break
              Case CancelButton
                ; Do not apply all configurations:
                Break
              Case DefaultsButton
                ; Set all temporary configurations into default configurations:
                Font = "Courier New"
                ExtractHTMLGeneratorTags(Tag(), ?HTMLGenerator_StandardTags)
                Refresh = #True
                
              Case SelectFontButton
                ; Select a font:
                If FontRequester(GetGadgetText(FontNameString), 12, 0)
                  Font = SelectedFontName()
                  Refresh = #True
                EndIf
              
              Case BoldCheckBox
                ; Enable/Disable bolding of the current tag:
                *Tag\Bold = GetGadgetState(BoldCheckBox)
                Refresh = #True
              Case ItalicCheckBox
                ; Enable/Disable italicizing of the current tag:
                *Tag\Italic = GetGadgetState(ItalicCheckBox)
                Refresh = #True
              Case UnderlineCheckBox
                ; Enable/Disable underlining of the current tag:
                *Tag\Underline = GetGadgetState(UnderlineCheckBox)
                Refresh = #True
              Case SelectColorButton
                ; Select an other color for the current tag:
                SelectedColor = ColorRequester(*Tag\Color)
                If SelectedColor <> -1
                  *Tag\Color = SelectedColor
                  Refresh = #True
                EndIf
              
              Case TagList
                Select EventType
                  Case #PB_EventType_LeftClick
                    ; Select an other tag:
                    Index = GetGadgetState(TagList)
                    If Index <> -1
                      *Tag = @Tag(Index)
                      SetGadgetText(PreviewText, HTMLGenerator_Example(Index))
                      Refresh = #True
                    EndIf
                EndSelect
            EndSelect
          Case #PB_Event_CloseWindow
            Break
        EndSelect
    EndSelect
    
    ; Refresh all informations on the window:
    If Refresh
      SetGadgetText(FontNameString, Font)
      
      SetGadgetState(BoldCheckBox, *Tag\Bold)
      SetGadgetState(ItalicCheckBox, *Tag\Italic)
      SetGadgetState(UnderlineCheckBox, *Tag\Underline)
      SetGadgetText(HexString, RSet(Hex(TurnColor(*Tag\Color)), 6, "0"))
      
      FreeFont(PreviewFont)
      PreviewFont = LoadFont(#PB_Any, Font, 12, (#PB_Font_Bold**Tag\Bold)|(#PB_Font_Italic**Tag\Italic)|(#PB_Font_Underline**Tag\Underline))
      SetGadgetFont(PreviewText, FontID(PreviewFont))
      SetGadgetColor(PreviewText, #PB_Gadget_FrontColor, *Tag\Color)
      
      Refresh = #False
    EndIf
  ForEver
  
  CloseWindow(Window)
EndProcedure

Procedure.i GenerateHelpIndex(SourceFileName.s, DestinationFileName.s)
  Protected SourceFile.i, DestinationFile.i
  Protected String.s, Text.s, Line.i, Column.i
  
  SourceFile      = ReadFile(#PB_Any, SourceFileName)
  DestinationFile = CreateFile(#PB_Any, DestinationFileName)
  
  If SourceFile And DestinationFile
    WriteStringN(DestinationFile, "<!DOCTYPE HTML PUBLIC " + #Q + "-//W3C//DTD HTML 4.0 Transitional//EN" + #Q + ">")
    WriteStringN(DestinationFile, "<html>")
    WriteStringN(DestinationFile, "<head><title>Operno - Index</title><link rel=" + #Q + "stylesheet" + #Q + " type=" + #Q + "text/css" + #Q + " href=" + #Q + "..\..\Resources\Style.css" + #Q + "></head>")
    WriteStringN(DestinationFile, "<body>")
    WriteStringN(DestinationFile, "<div class=" + #Q + "header" + #Q + ">" + ReadString(SourceFile) + "</div>")
    WriteStringN(DestinationFile, "<div class=" + #Q + "back" + #Q + "><div class=" + #Q + "page" + #Q + ">")
    
    While Not Eof(SourceFile)
      String = ReadString(SourceFile)
      Select String
        Case "#TABLE"
          WriteStringN(DestinationFile, "<table class=" + #Q + "list" + #Q + " width=" + #Q + "90%" + #Q + " align=" + #Q + "center" + #Q + " cellspacing=" + #Q + "2px" + #Q + " cellpadding=" + #Q + "3px" + #Q + ">")
        Case "#END TABLE"
          WriteStringN(DestinationFile, "</table>")
          
        Case "#UNIT"
          WriteStringN(DestinationFile, "<tr><th colspan=" + #Q + "2" + #Q + "><h2>" + ReadString(SourceFile) + "</h2></th></tr>")
        Case "#LINKS"
          WriteStringN(DestinationFile, "<tr><td valign=" + #Q + "top" + #Q + "><ul>")
          Text   = ""
          Line   = 0
          Column = 0
          While Not Eof(SourceFile)
            String = ReadString(SourceFile)
            If String = "#END LINKS"
              Break
            Else
              If Line = 5
                Line = 1
                WriteStringN(DestinationFile, Text)
                Text = ""
                If Column = 1
                  Column = 0
                  WriteStringN(DestinationFile, "</ul></td></tr></tr><td><ul>")
                Else
                  Column + 1
                  WriteStringN(DestinationFile, "</ul></td><td><ul>")
                EndIf
              Else
                Line + 1
              EndIf
              If Text
                Text + #N
              EndIf
              Text + "<li><a href=" + #Q + String + ".html" + #Q + ">" + String + "</a>" + "</li>"
            EndIf
          Wend
          WriteStringN(DestinationFile, Text)
          WriteStringN(DestinationFile, "</ul></td></tr>")
          
        Case ""
          Continue
          
        Default
          WriteStringN(DestinationFile, "<p>" + String + "</p>")
      EndSelect
    Wend
    
    WriteStringN(DestinationFile, "</div></div>")
    WriteStringN(DestinationFile, "<div class=" + #Q + "foot" + #Q + "><a href=" + #Q + "#top" + #Q + ">Top</a> | <a href=" + #Q + "../Index.html" + #Q + ">Home</a></div>")
    WriteStringN(DestinationFile, "</body></html>")
    
    CloseFile(SourceFile)
    CloseFile(DestinationFile)
  EndIf
EndProcedure
Procedure.i GenerateHelp(SourceFileName.s, DestinationFileName.s)
  Protected SourceFile.i, DestinationFile.i
  Protected String.s, Text.s
  
  SourceFile      = ReadFile(#PB_Any, SourceFileName)
  DestinationFile = CreateFile(#PB_Any, DestinationFileName)
  
  If SourceFile And DestinationFile
    WriteStringN(DestinationFile, "<!DOCTYPE HTML PUBLIC " + #Q + "-//W3C//DTD HTML 4.0 Transitional//EN" + #Q + ">")
    WriteStringN(DestinationFile, "<html>")
    WriteStringN(DestinationFile, "<head><title>Operno - " + StringField(GetFilePart(DestinationFileName), 1, ".") + "</title><link rel=" + #Q + "stylesheet" + #Q + " type=" + #Q + "text/css" + #Q + " href=" + #Q + "..\..\Resources\Style.css" + #Q + "></head>")
    WriteStringN(DestinationFile, "<body>")
    WriteStringN(DestinationFile, "<div class=" + #Q + "header" + #Q + ">" + StringField(GetFilePart(DestinationFileName), 1, ".") + "</div>")
    WriteStringN(DestinationFile, "<div class=" + #Q + "back" + #Q + "><div class=" + #Q + "page" + #Q + ">")
    
    While Not Eof(SourceFile)
      String = ReadString(SourceFile)
      Select String
        Case "#CODE"
          Text = ""
          While Not Eof(SourceFile)
            String = ReadString(SourceFile)
            If String = "#END CODE"
              Break
            ElseIf Text
              Text + #N
            EndIf
            Text + String
          Wend
          WriteStringN(DestinationFile, "<p class=" + #Q + "code" + #Q + ">")
          GenerateHTMLCode(DestinationFile, @Text)
          WriteStringN(DestinationFile, "</p>")
        Case "#OUTPUT"
          Text = ""
          While Not Eof(SourceFile)
            String = ReadString(SourceFile)
            If String = "#END OUTPUT"
              Break
            ElseIf Text
              Text + #N
            EndIf
            Text + String
          Wend
          WriteStringN(DestinationFile, "<p class=" + #Q + "out" + #Q + ">" + HTMLString(Text) + "</p>")
        Case "#BLACK CODE"
          Text = ""
          While Not Eof(SourceFile)
            String = ReadString(SourceFile)
            If String = "#END BLACK CODE"
              Break
            ElseIf Text
              Text + #N
            EndIf
            Text + String
          Wend
          WriteStringN(DestinationFile, "<p class=" + #Q + "code" + #Q + ">" + HTMLString(Text) + "</p>")
        
        Case "#COURIER"
          WriteStringN(DestinationFile, "<p><code>" + ReadString(SourceFile) + "</code></p>")
        Case "#SUBTITLE", "#TITLE"
          WriteStringN(DestinationFile, "<h2>" + ReadString(SourceFile) + "</h2>")
        Case "#CAPTION"
          WriteStringN(DestinationFile, "<h3>" + ReadString(SourceFile) + ":" + "</h3>")
        Case "#COMMENT"
          WriteStringN(DestinationFile, "<p class=" + #Q + "comment" + #Q + ">" + ReadString(SourceFile) + "</p>")
        Case "#INFO"
          WriteStringN(DestinationFile, "<p class=" + #Q + "info" + #Q + ">" + ReadString(SourceFile) + "</p>")
        Case "#LINKS"
          WriteStringN(DestinationFile, "<p class=" + #Q + "links" + #Q + ">" + ReadString(SourceFile) + "</p>")
          
        Case "#OBJECT"
          WriteStringN(DestinationFile, "<p class=" + #Q + "obj" + #Q + ">" + ReadString(SourceFile) + "</p>")
          
        Case "#HR"
          WriteStringN(DestinationFile, "<hr>")
          
        Case "#HTML"
          Text = ""
          While Not Eof(SourceFile)
            String = ReadString(SourceFile)
            If String = "#END HTML"
              Break
            ElseIf Text
              Text + #N
            EndIf
            Text + String
          Wend
          WriteStringN(DestinationFile, "<p>" + Text + "</p>")
          
        Case ""
          Continue
          
        Default
          WriteStringN(DestinationFile, "<p>" + String + "</p>")
      EndSelect
    Wend
    
    WriteStringN(DestinationFile, "</div></div>")
    WriteStringN(DestinationFile, "<div class=" + #Q + "foot" + #Q + "><a href=" + #Q + "#top" + #Q + ">Top</a> | <a href=" + #Q + "../Index.html" + #Q + ">Home</a> | <a href=" + #Q + "Index.html" + #Q + ">Index</a></div>")
    WriteStringN(DestinationFile, "</body></html>")
    
    CloseFile(SourceFile)
    CloseFile(DestinationFile)
  EndIf
EndProcedure
; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 71
; FirstLine = 62
; Folding = X-
; EnableXP
; CompileSourceDirectory