;**************************************************************
;     Interface
;
; Author: Josef Sniatecki
; Creation date: 07.04.2009
;**************************************************************
;   Copyright (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



; Options:
Procedure.i LoadOptions()
  Protected New.i, Index.i
  
  If Not OpenPreferences("Data\Options.txt")
    New = #True
    CreatePreferences("Data\Options.txt")
    
    ExtractHTMLGeneratorTags(HTMLGenerator_Tag(), ?HTMLGenerator_StandardTags)
  EndIf
  
  PreferenceGroup("User")
    User_Name     = ReadPreferenceString("Name", "Unknow")
    User_Language = ReadPreferenceString("Language", "German")
    User_New      = ReadPreferenceInteger("New", 1)
    
  PreferenceGroup("Preview")
    Preview_Display  = ReadPreferenceInteger("Display", #True)
    Preview_Font     = ReadPreferenceString("Font", "Courier New")
    Preview_FontSize = ReadPreferenceInteger("Font size", 12)
    
  PreferenceGroup("Console")
    Console_FrontColor = ReadPreferenceInteger("Front color", 7)
    Console_BackColor  = ReadPreferenceInteger("Back color", 0)
    
  PreferenceGroup("GUI")
    GUI_DisplayInput   = ReadPreferenceInteger              ("Display input", #True)
    GUI_FrontColor     = Val("$" + RSet(ReadPreferenceString("Front color", "FFFFFF"), 6, "0"))
    GUI_BackColor      = Val("$" + RSet(ReadPreferenceString("Back color", "000000"), 6, "0"))
    GUI_Font           = ReadPreferenceString               ("Font", "Courier New")
    GUI_FontSize       = ReadPreferenceInteger              ("Font size", 8)
    GUI_Width          = ReadPreferenceInteger              ("Width", 550)
    GUI_Height         = ReadPreferenceInteger              ("Height", 600)
    
  PreferenceGroup("HTMLGenerator")
    HTMLGenerator_Font = ReadPreferenceString("Font", "Courier New")
    If New = #False
      For Index = 0 To #HTMLGenerator_TagCount
        HTMLGenerator_Tag(Index)\Bold      = ReadPreferenceInteger              (HTMLGenerator_TagName(Index) + " bold", 0)
        HTMLGenerator_Tag(Index)\Italic    = ReadPreferenceInteger              (HTMLGenerator_TagName(Index) + " italic", 0)
        HTMLGenerator_Tag(Index)\Underline = ReadPreferenceInteger              (HTMLGenerator_TagName(Index) + " underline", 0)
        HTMLGenerator_Tag(Index)\Color     = Val("$" + RSet(ReadPreferenceString(HTMLGenerator_TagName(Index) + " color", "000000"), 6, "0"))
      Next
    EndIf
    
  ClosePreferences()
EndProcedure
Procedure.i SaveOptions()
  Protected Index.i
  
  CreatePreferences("Data\Options.txt")
  
  PreferenceGroup("User")
    WritePreferenceString ("Name"    , User_Name)
    WritePreferenceString ("Language", User_Language)
    WritePreferenceInteger("New"     , User_New)
    
  PreferenceGroup("Preview")
    WritePreferenceInteger("Display"  , Preview_Display)
    WritePreferenceString ("Font face", Preview_Font)
    WritePreferenceInteger("Font size", Preview_FontSize)
    
  PreferenceGroup("Console")
    WritePreferenceInteger("Front color", Console_FrontColor)
    WritePreferenceInteger("Back color" , Console_BackColor)
    
  PreferenceGroup("GUI")
    WritePreferenceInteger("Display input", GUI_DisplayInput)
    WritePreferenceString ("Front color"  , RSet(Hex(GUI_FrontColor, #PB_Integer), 6, "0"))
    WritePreferenceString ("Back color"   , RSet(Hex(GUI_BackColor, #PB_Integer), 6, "0"))
    WritePreferenceString ("Font"         , GUI_Font)
    WritePreferenceInteger("Font size"    , GUI_FontSize)
    WritePreferenceInteger("Width"        , GUI_Width)
    WritePreferenceInteger("Height"       , GUI_Height)
    
  PreferenceGroup("HTMLGenerator")
    WritePreferenceString("Font", HTMLGenerator_Font)
    For Index = 0 To #HTMLGenerator_TagCount
      WritePreferenceInteger(HTMLGenerator_TagName(Index) + " bold"     , HTMLGenerator_Tag(Index)\Bold)
      WritePreferenceInteger(HTMLGenerator_TagName(Index) + " italic"   , HTMLGenerator_Tag(Index)\Italic)
      WritePreferenceInteger(HTMLGenerator_TagName(Index) + " underline", HTMLGenerator_Tag(Index)\Underline)
      WritePreferenceString (HTMLGenerator_TagName(Index) + " color"    , RSet(Hex(HTMLGenerator_Tag(Index)\Color, #PB_Integer), 6, "0"))
    Next
    
  ClosePreferences()
EndProcedure

; Output channels:
Procedure.i DisplayConsoleOutput(Text.s)
  Protected *Char.Character, Attribute.s, Char.s
  Protected *Start.Character
  
  Text = CharToConsole(Text)
  
  *Char = @Text
  While *Char\c
    Select *Char\c
      Case '$'
        *Char + 1
        *Start = *Char
        Repeat
          Select *Char\c
            Case '$'
              Break
            Case 0
              Break 2
          EndSelect
          *Char + 1
        ForEver
        
        Attribute = LCase(PeekS(*Start, *Char - *Start))
        Select Attribute
          Case "n", "nc"
            ConsoleColor(Console_FrontColor, Console_BackColor)
          Case "red"
            ConsoleColor(12, Console_BackColor)
          Case "green"
            ConsoleColor(10, Console_BackColor)
          Case "blue"
            ConsoleColor(9, Console_BackColor)
          Case "yellow"
            ConsoleColor(14, Console_BackColor)
          Case "magenta"
            ConsoleColor(13, Console_BackColor)
          Case "cyan"
            ConsoleColor(11, Console_BackColor)
          Case "white"
            ConsoleColor(15, Console_BackColor)
          Case "gray"
            ConsoleColor(8, Console_BackColor)
          Case ""
            Print("$")
        EndSelect
      Case 13
        *Char + 1
        PrintN("")
        ConsoleColor(Console_FrontColor, Console_BackColor)
      Default
        Print(Chr(*Char\c))
    EndSelect
    *Char + 1
  Wend
  
  ConsoleColor(Console_FrontColor, Console_BackColor)
EndProcedure
Procedure.i DisplayOutput(Text.s)
  If Console
    DisplayConsoleOutput(Text)
  ElseIf GUI
    DisplayGUIOutput(Text)
  ElseIf *MasterCallback
    _Call(*MasterCallback, #Callback_Output, @Text, 0)
  Else
    OpenGUIWindow()
    DisplayGUIOutput(Text)
  EndIf
EndProcedure
Procedure.s ConvertOutput(Text.s, Indent.i)
  If OutputSymbols
    Text = ReplaceString(Text, #N, #N + Space(#SymbolLength) + #SymbolSeperator + Space(Indent*#IndentLength))
  Else
    Text = ReplaceString(Text, #N, #N + Space(Indent*#IndentLength))
  EndIf
  Text = ReplaceString(Text, #T, Space(#IndentLength))
  ProcedureReturn Text
EndProcedure
Procedure.i OutputSymbol(Symbol.i)
  If OutputSymbols
    DisplayOutput(Symbol(Symbol) + Space(#SymbolLength-Len(Symbol(Symbol))))
    DisplayOutput(#SymbolSeperator)
  EndIf
EndProcedure

; Output:
Procedure.i OutputT(Text.s, Indent.i = 0)
  DisplayOutput(ConvertOutput(Text, Indent))
EndProcedure
Procedure.i OutputL(Symbol.i, Text.s = "", Indent.i = 0)
  OutputSymbol(Symbol)
  DisplayOutput(ConvertOutput(Text, Indent))
EndProcedure
Procedure.i OutputFL(Symbol.i, Text.s, Indent.i = 0)
  OutputSymbol(Symbol)
  DisplayOutput(Space(Indent*#IndentLength) + ConvertOutput(Text, Indent) + #N)
EndProcedure
Procedure.i OutputN()
  DisplayOutput(#N)
EndProcedure
Procedure.i OutputE()
  ; Outputs an occurred error:
  If Error
    OutputFL(#Symbol_Error, ErrorMessage)
    Error = #False
    ErrorMessage = ""
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i OutputHL()
  DisplayOutput("-------------------------------------------------------------------------" + #N)
EndProcedure
Procedure.i OutputFLW(Symbol.i, Text.s, Indent.i = 0)
  Protected *Pointer.Character, MaximalDuration.i
  
  OutputSymbol(Symbol)
  DisplayOutput(Space(Indent*#IndentLength))
  
  *Pointer = @Text
  
  ; Skip the writing of the code:
  If GetAsyncKeyState_(#VK_RETURN) = -32767
    DisplayOutput(ConvertOutput(PeekS(*Pointer), Indent) + #N)
    ProcedureReturn
  EndIf
  
  Wait(500)
  
  While *Pointer\C
    ; Skip the writing of the code:
    If GetAsyncKeyState_(#VK_RETURN) = -32767
      DisplayOutput(ConvertOutput(PeekS(*Pointer), Indent) + #N)
      ProcedureReturn
    EndIf
      
    If *Pointer > @Text And PeekC(*Pointer - 1) = *Pointer\C
      ; Write a character once again:
      Wait(80)
      DisplayOutput(Chr(*Pointer\C))
    ElseIf *Pointer\C = 13
      ; Write a new line:
      Wait(100)
      DisplayOutput(#N)
      OutputSymbol(#Symbol_None)
      DisplayOutput(Space(Indent*#IndentLength))
      Wait(250)
      *Pointer + 1
    Else
      ; Select a maximal duration of various characters:
      Select *Pointer\C
        Case '@', '', '', '~', '\', '[', ']', '{', '}'
          MaximalDuration = 300
        Case '(', ')'
          MaximalDuration = 200
        Case '"', '#', '$'
          MaximalDuration = 120
        Case ':', '.', ';', ','
          MaximalDuration = 80
        Default
          MaximalDuration = 30
      EndSelect
      Wait(80 + Random(MaximalDuration))
      DisplayOutput(Chr(*Pointer\C))
    EndIf
    *Pointer + 1
  Wend
  
  Wait(100)
  DisplayOutput(#N)
EndProcedure

; Input:
Procedure.s ReadInput()
  Protected Input.s, SecondInput.s
  
  If Console
    ProcedureReturn ConsoleToChar(Input())
  ElseIf GUI
    ProcedureReturn ReadGUIInput()
  ElseIf *MasterCallback
    ProcedureReturn PeekS(_Call(*MasterCallback, #Callback_Input, 0, 0))
  Else
    ProcedureReturn InputRequester("Operno input", "", "")
  EndIf
EndProcedure
Procedure.s InputFL(Symbol.i, Indent.i = 0)
  OutputSymbol(Symbol)
  DisplayOutput(Space(Indent*#IndentLength))
  ProcedureReturn ReadInput()
EndProcedure
Procedure.s InputFLT(Symbol.i, Text.s, Indent.i = 0)
  OutputSymbol(Symbol)
  If Text
    DisplayOutput(Space(Indent*#IndentLength) + Text + ": ")
  Else
    DisplayOutput(Space(Indent*#IndentLength))
  EndIf
  ProcedureReturn ReadInput()
EndProcedure
Procedure.i InputYN(Question.s, Indent.i = 0)
  If Not GUI And Not Console
    If MessageRequester("Question", Question, #PB_MessageRequester_YesNo) = #PB_MessageRequester_Yes
      OutputFL(#Symbol_Input, "yes")
      ProcedureReturn #True
    Else
      OutputFL(#Symbol_Input, "no")
      ProcedureReturn #False
    EndIf
  Else
    OutputL(#Symbol_Message, ConvertOutput(Question + " (y/n): ", Indent))
    Repeat
      Select LCase(ReadInput())
        Case "y", "yes"
          ProcedureReturn #True
        Case "n", "no"
          ProcedureReturn #False
        Default
          OutputFL(#Symbol_Error, "Type only " + #Q + "yes/y" + #Q + " to apply or " + #Q + "no/n" + #Q + "!")
      EndSelect
    ForEver
  EndIf
EndProcedure
Procedure.s InputC(Indent.i = 0)
  Protected Input.s, Code.s, Level.i
  
  Repeat
    If Code
      Input = InputFL(#Symbol_ETC, Indent + Level)
      Code + #N
    Else
      Input = InputFL(#Symbol_Input, Indent + Level)
    EndIf
    Code + Space(#IndentLength * Level) + Input
    If Right(Input, 1) = "#"
      Continue
    EndIf
    Level + Parser_CountLevel(Input) : _E()
    If Level <= 0
      ProcedureReturn Code
    EndIf
  ForEver
EndProcedure
Procedure.i WaitForReturn()
  Protected Event.i, Window.i, Menu.i, Gadget.i
  
  If Console
    While Not(Inkey() And RawKey() = 13)
      Delay(1)
    Wend
  ElseIf GUI
    DisableGadget(#GUI_Input, #True)
    SetGadgetText(#GUI_Input, "Press [F5] to continue!")
    SetGadgetColor(#GUI_Input, #PB_Gadget_FrontColor, #Black)
    Repeat
      Event  = WindowEvent()
      Window = EventWindow()
      Menu   = EventMenu()
      Gadget = EventGadget()
      
      Select Window
        Case #GUIWindow
          Select Event
            Case #PB_Event_CloseWindow
              Truncation = #Truncation_End
              ProcedureReturn
            Case #PB_Event_Menu
              If Menu = #GUIMenu_Execute_Input Or Menu = #GUIMenu_Execute_InputPlusResult
                Break
              EndIf
          EndSelect
      EndSelect
      
      Delay(1)
    ForEver
    SetGadgetText(#GUI_Input, "")
    SetGadgetColor(#GUI_Input, #PB_Gadget_FrontColor, GUI_FrontColor)
    DisableGadget(#GUI_Input, #False)
  ElseIf *MasterCallback
    _Call(*MasterCallback, #Callback_WaitForReturn, 0 ,0)
  Else
    While Not GetAsyncKeyState_(#VK_RETURN) = -32767
      Delay(1)
    Wend
  EndIf
EndProcedure

; User interface:
Procedure.i OpenUI(EnableHeader.i = #True)
  Protected Code.s, *Result
  Protected Program.i
  Protected AutoDisplay.i
  
  ; Open the console:
  If Console
    If EnableHeader
      OutputFL(#Symbol_Message, "Console has started.")
    EndIf
  Else
    Console = #True
    OpenConsole()
    ConsoleTitle("Operno")
    
    EnableGraphicalConsole(#True)
    ConsoleColor(Console_FrontColor, Console_BackColor)
    ClearConsole()
    EnableGraphicalConsole(#False)
    
    If EnableHeader
      Interface_Head()
      OutputFL(#Symbol_None, "For a summary of special commands enter " + #Q + ".help" + #Q + ".")
    EndIf
  EndIf
  If EnableHeader
    Interface_Startup()
  EndIf
  
  ; Prompt:
  Repeat
    ClearStack()
    ClearLevels()
    
    Code = InputC() : _E("", OutputE(), Continue)
    If Not Code
      Continue
    EndIf
    
    If Left(Code, 1) = "."
      ; Check console commands:
      Select Right(Code, Len(Code)-1)
        Case "help"
          OutputFL(#Symbol_None , "Each command starts with a dot!")
          OutputFL(#Symbol_Point, "new   - Open a new instance of Operno")
          OutputFL(#Symbol_Point, "exec  - Execute scripts in new Operno instances")
          OutputFL(#Symbol_Point, "clp   - Execute the clipboard contents")
          OutputFL(#Symbol_Point, "cmp   - Create a token file")
          OutputFL(#Symbol_Point, "html  - Generate sytax-hilighted scripts")
          OutputFL(#Symbol_Point, "tut   - Open and start a tutorial")
          OutputFL(#Symbol_Point, "htmlc - Configure html-generator styles for tags")
          OutputFL(#Symbol_Point, "ad    - Enable/Disable automatic displaying of results")
          OutputFL(#Symbol_Point, "c     - Clear the whole console buffer")
          OutputFL(#Symbol_Point, "a     - Display all informations of Operno")
          OutputFL(#Symbol_Point, "h     - Open help")
          OutputFL(#Symbol_Point, "gh    - Generate help files")
        Case "new"
          RunProgram("Operno.exe", "NOPREVIEW", GetCurrentDirectory())
        Case "exec"
          Interface_Execute()
        Case "clp"
          Interface_ExecuteClipboard()
        Case "cmp"
          Interface_Compile()
        Case "html"
          Interface_HTML()
        Case "tut"
          Interface_Tutorial() : OutputE()
        Case "htmlc"
          HTMLGeneratorConfiguration()
        Case "ad"
          AutoDisplay = 1 - AutoDisplay
        Case "clear", "c"
          EnableGraphicalConsole(#True)
          ClearConsole()
          EnableGraphicalConsole(#False)
        Case "h"
          Interface_OpenHelp()
        Case "gh"
          Interface_GenerateHelp()
        Case "a"
          Interface_About()
      EndSelect
    Else
      ; Normal code:
      *Result = ExecuteCommands(Code, 0, #True)
      If Truncation = #Truncation_End
        Break
      EndIf
      Truncation = 0
      If Not OutputE() And AutoDisplay
        OutputFL(#Symbol_Output, FormatValue(*Result))
      EndIf
    EndIf
  ForEver
  
  CloseConsole()
  Console = #False
EndProcedure
Procedure.i OpenPreview()
  Protected Window.i, ImageGadget.i, Image.i, File.i
  Protected Maximal.i, String.s, CodeImage.i
  Protected CodeFont.i, CommentFont.i, Y.i
  Protected *PositionQueue.Queue = CreateQueue(#Queue_FreeMemory), *Integer.Integer
  
  If Not Preview_Display
    ProcedureReturn
  EndIf
  
  UseJPEGImageDecoder()
  Image = LoadImage(#PB_Any, "Data\Banner.jpg")
  If Random(5)
    If User_New = #False
      File = ReadFile(#PB_Any, "Previews\" + User_Language + ".txt")
    EndIf
    If File
      CodeFont = LoadFont(#PB_Any, Preview_Font, Preview_FontSize, #PB_Font_Bold)
      CommentFont = LoadFont(#PB_Any, "Microsoft Sans Serif", 12)
      ; Detect all previews of the file:
      While Not Eof(File)
        If ReadString(File) = "#PREVIEW"
          *Integer   = _New(Integer)
          *Integer\I = Loc(File)
          AppendQueueItem(*PositionQueue, *Integer)
        EndIf
      Wend
      ; Select a random preview of the file:
      SelectQueueItem(*PositionQueue, Random(*PositionQueue\Count-1), @*Integer)
      FileSeek(File, *Integer\I)
      FreeQueue(*PositionQueue)
      
      ; Start drawing of a preview:
      StartDrawing(ImageOutput(Image))
        DrawingMode(#PB_2DDrawing_AlphaBlend)
        Box(11 , 0  , 320, 324, $FAFFFFFF)
        Box(337, 0  , 438, 132, $FAFFFFFF)
        Box(337, 138, 438, 186, $FDFFFFFF)
        
        DrawingMode(#PB_2DDrawing_Outlined)
        Box(0, 0, ImageWidth(Image), ImageHeight(Image), #Blue)
        
        DrawingMode(#PB_2DDrawing_Transparent)
        ; Draw the code:
        DrawingFont(FontID(CodeFont))
        Y = 13
        While Not Eof(File)
          String = ReadString(File)
          Select String
            Case "#COMMENT", "#PREVIEW"
              Break
            Case "#IMAGE"
              CodeImage = LoadImage(#PB_Any, "Previews\Images\" + ReadString(File) + ".bmp")
              DrawImage(ImageID(CodeImage), 12, Y)
              Y + ImageHeight(CodeImage) + 3
              FreeImage(CodeImage)
            Default
              DrawText(12, Y, String, $000000)
              Y + 15
          EndSelect
        Wend
        ; Draw the comment:
        DrawingFont(FontID(CommentFont))
        Y = 139
        While Not Eof(File)
          String = ReadString(File)
          Select String
            Case "#PREVIEW"
              Break
            Default
              DrawText(338, Y, String, $000000)
              Y + 15
          EndSelect
        Wend
        ; Draw important informations:
        DrawText(355, 43, "Operno " + #Version, $000000)
      StopDrawing()
      
      ; Free some data:
      FreeFont(CodeFont)
      FreeFont(CommentFont)
      CloseFile(File)
    EndIf
  Else
    StartDrawing(ImageOutput(Image))
      DrawingMode(#PB_2DDrawing_Outlined)
      Box(0, 0, ImageWidth(Image), ImageHeight(Image), #Blue)
    StopDrawing()
  EndIf
  
  ; Open the window:
  Window = OpenWindow(#PB_Any, 0, 0, ImageWidth(Image), ImageHeight(Image), "Operno", #PB_Window_BorderLess|#PB_Window_ScreenCentered)
    ImageGadget = ImageGadget(#PB_Any, 0, 0, ImageWidth(Image), ImageHeight(Image), ImageID(Image))
  
  Repeat
    Select WaitWindowEvent()
      Case 13100, 256 ;Pushing on image, Key down
        Break
    EndSelect
  ForEver
  
  CloseWindow(Window)
  FreeImage(Image)
EndProcedure

; Interface:
Procedure.i Interface_Head()
  OutputFL(#Symbol_None, "$b$Operno$n$ (Version: " + #Version + ")")
  OutputFL(#Symbol_None, "Copyright " + #C + " " + FormatDate("%yyyy", Date()) + " Josef Sniatecki")
EndProcedure
Procedure.i Interface_Startup()
  OutputHL()
  If User_New
    User_New = #False
    StartTutorial("Tutorials\" + User_Language + "\start.txt") : OutputE()
  EndIf
EndProcedure
Procedure.i Interface_Execute()
  Protected Filename.s, Program.i
  
  Filename = OpenFileRequester("Execute", "", "*.*", 0, #PB_Requester_MultiSelection)
  While Filename
    Program = RunProgram("Operno.exe", #Q + Filename + #Q, GetCurrentDirectory() + "\", #PB_Program_Open)
    
    OutputFL(#Symbol_Message, #Q + GetFilePart(Filename) + #Q + " started.")
    WaitProgram(Program)
    OutputFL(#Symbol_Message, "Finished.")
    KillProgram(Program)
    CloseProgram(Program)
    
    Filename = NextSelectedFileName()
  Wend
EndProcedure
Procedure.i Interface_ExecuteClipboard()
  Protected Code.s, File.i, Program.i
  
  Code = GetClipboardText()
  
  File = CreateFile(#PB_Any, "Temp source.txt")
  If File
    WriteString(File, Code)
    CloseFile(File)
    
    Program = RunProgram("Operno.exe", #Q + "Temp source.txt" + #Q, GetCurrentDirectory(), #PB_Program_Open)
    
    OutputFL(#Symbol_Message, #Q + GetFilePart("Temp source.txt") + #Q + " started.")
    WaitProgram(Program)
    OutputFL(#Symbol_Message, "Finished.")
    KillProgram(Program)
    CloseProgram(Program)
  EndIf
EndProcedure
Procedure.i Interface_Compile()
  Protected Filename.s, Program.i
  
  Filename = OpenFileRequester("Compile", "", "*.*", 0, #PB_Requester_MultiSelection)
  While Filename
    Program = RunProgram("Operno.exe", "compile " + #Q + Filename + #Q + " " + #Q + StringField(Filename, 1, ".") + ".op_t" + #Q, GetCurrentDirectory(), #PB_Program_Open)
    
    OutputFL(#Symbol_Message, "Generate " + #Q + GetFilePart(Filename) + #Q + " as token file.")
    WaitProgram(Program)
    OutputFL(#Symbol_Message, "Finished.")
    KillProgram(Program)
    CloseProgram(Program)
    
    Filename = NextSelectedFileName()
  Wend
EndProcedure
Procedure.i Interface_HTML()
  Protected FileName.s, HTMLFileName.s, Program.i
  
  FileName = OpenFileRequester("Convert to HTML", "", "*.*", 0, #PB_Requester_MultiSelection)
  While FileName
    HTMLFileName = StringField(FileName, 1, ".") + ".html"
    
    OutputFL(#Symbol_Message, "Generate " + #Q + GetFilePart(FileName) + #Q + " as HTML.")
    GenerateHTMLFile(FileName, HTMLFileName)
    OutputFL(#Symbol_Message, "Finished.")
    
    OutputFL(#Symbol_Message, "Opening...")
    RunProgram(HTMLFileName)
    
    Filename = NextSelectedFileName()
  Wend
EndProcedure
Procedure.i Interface_GenerateHelp(Path.s = "")
  Protected Directory.i, FileName.s, HTMLFileName.s
  
  Directory = ExamineDirectory(#PB_Any, "Help\german\Source" + Path, "*.*")
  While NextDirectoryEntry(Directory)
    If DirectoryEntryType(Directory) = #PB_DirectoryEntry_File
      FileName = DirectoryEntryName(Directory)
      HTMLFileName = StringField(FileName, 1, ".") + ".html"
      OutputFL(#Symbol_Message, "Generating " + #Q + GetFilePart(FileName) + #Q + ".")
      If FileName = "Index.txt"
        GenerateHelpIndex("Help\german\Source" + Path + "\" + FileName, "Help\german" + Path + "\" + HTMLFilename)
      Else
        GenerateHelp("Help\german\Source" + Path + "\" + FileName, "Help\german" + Path + "\" + HTMLFileName)
      EndIf
      OutputFL(#Symbol_Message, "finished.")
    ElseIf CountString(DirectoryEntryName(Directory), ".") = 0
      CreateDirectory("Help\german" + Path + "\" + DirectoryEntryName(Directory))
      Interface_GenerateHelp(Path + "\" + DirectoryEntryName(Directory))
    EndIf
  Wend
  FinishDirectory(Directory)
EndProcedure
Procedure.i Interface_Tutorial()
  Protected File.i, FileName.s
  Protected Text.s, String.s, Event.i
  
  FileName = OpenFileRequester("Tutorial", "Tutorials\" + User_Language + "\", "*.txt", 0)
  If Filename
    StartTutorial(FileName) : OutputE()
  EndIf
EndProcedure
Procedure.i Interface_OpenHelp()
  RunProgram(GetStandardBrowser(), #Q + GetCurrentDirectory() + "Help\" + LCase(User_Language) + "\Index.html" + #Q, "")
EndProcedure
Procedure.i Interface_About()
  Protected License.s
  
  OutputFL(#Symbol_None , "Operno " + #Version, 1)
  OutputFL(#Symbol_None , "")
  OutputFL(#Symbol_Point, "Main developer: Josef Sniatecki")
  OutputFL(#Symbol_Point, "Supporters    : www.pure-board.de (forum)")
  OutputFL(#Symbol_None , "")
  
  License = "This program is free software; you can redistribute it and/or"       + #N
  License + "modify it under the terms of the $b$GNU General Public License$n$"   + #N
  License + "As published by the Free Software Foundation; either version"        + #N
  License + "3 of the License, Or (at your option) any later version."            + #N
  License                                                                         + #N
  License + "This program is distributed in the hope that it will be"             + #N
  License + "useful, but $b$WITHOUT ANY WARRANTY$n$; without even the implied"    + #N
  License + "warranty of $b$MERCHANTABILITY Or FITNESS For A PARTICULAR$n$"       + #N
  License + "$b$PURPOSE$n$. See the GNU General Public License For more details." + #N
  License                                                                         + #N
  License + "You should have received a copy of the GNU General Public"           + #N
  License + "License along With this program;"                                    + #N
  License + "If Not, see $u$" + #Q + "http://www.gnu.org/licenses/" + #Q + "$n$."
  OutputFL(#Symbol_None, License, 1)
  
  OutputHL()
EndProcedure

; Misc:
Procedure.i Wait(Milliseconds.i)
  Protected Start.i
  
  If GUI
    Start = ElapsedMilliseconds()
    Repeat
      WindowEvent()
      Sleep_(1)
    Until ElapsedMilliseconds() - Start > Milliseconds
  ElseIf *MasterCallback
    _Call(*MasterCallback, #Callback_Wait, Milliseconds, 0)
  Else
    Sleep_(Milliseconds)
  EndIf
EndProcedure

; Record:
Procedure.i InitRecord()
  Record = "'{" + #N
  Record + "  Program name" + #N
  Record + "  " + FormatDate("%dd.%mm.%yyyy", Date()) + #N
  Record + "}'"
EndProcedure
Procedure.i AddRecord(Code.s)
  If Record
    Record + #N
  EndIf
  Record + Code
EndProcedure
Procedure.i SaveRecord()
  Protected File.i, FileName.s
  
  FileName = SaveFileRequester("Save record", "", "*.*", 0)
  If FileName
    File = CreateFile(#PB_Any, FileName)
    If File
      WriteString(File, Record)
      Record = ""
      CloseFile(File)
    EndIf
  EndIf
EndProcedure

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 482
; FirstLine = 147
; Folding = AAALAg
; EnableXP
; Executable = ..\Operno.exe
; CompileSourceDirectory