;**************************************************************
;     Statements
;
; Autor: Josef Sniatecki
; Creation date: 07.04.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i Statement_If(*Value.Value = #Null)
  Protected *Result.Value, *Token.Token
  
  If IsValueTrue(RunControlBlock())
    ; The expression is true:
    If *Value
      *Result = *Value
    Else
      *Result = RunControlBlock() : _R(0, ProcedureReturn *Result)
    EndIf
  Else
    ; The expression is false:
    If Not *Value
      _N() ;Ignore controll-block
    EndIf
    Repeat
      _SN(*Token)
      If *Token\Type = #Token_Keyword
        Select *Token\Keyword
          Case #Keyword_ElseIf
            If IsValueTrue(RunControlBlock())
              *Result = RunControlBlock()  : _R(0, ProcedureReturn *Result)
              Break
            Else
              _N() ;Ignore controll-block
            EndIf
            Continue
          Case #Keyword_Else
            ProcedureReturn RunControlBlock()
        EndSelect
      EndIf
      _P()
      ProcedureReturn PushVValue()
    ForEver
  EndIf
  
  ; Ignore the whole if-controll:
  Repeat
    _SN(*Token)
    If *Token\Type = #Token_Keyword
      Select *Token\Keyword
        Case #Keyword_ElseIf
          _N() ;Ignore expression
          _N() ;Ignore controll-block
          Continue
        Case #Keyword_Else
          _N() ;Ignore controll-block
          Continue
        Default
      EndSelect
    EndIf
    _P()
    Break
  ForEver
  
  ProcedureReturn *Result
EndProcedure
Procedure.i Statement_While(*Value.Value = #Null)
  Protected *Token.Token, *TokenQueue.Queue, *ExpressionToken.QueueItem
  Protected *Result.Value
  
  If *Value And *Value\Class = *BlockClass
    *TokenQueue = *Value\Queue
    *ExpressionToken = *Current\TokenItem
    Repeat
      OpenLevel(*TokenQueue, 0, 0, 0)
        RunBlock()
      CloseLevel() : _E() : _RB(1)
      *Result = RunControlBlock() : _E()
      If IsValueTrue(*Result)
        *Current\TokenItem = *ExpressionToken
      Else
        Break
      EndIf
    ForEver
  Else
    *ExpressionToken = *Current\TokenItem
    _N() : _SN(*Token)
    If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Curly
      *TokenQueue = *Token\Block\TokenQueue
    Else
      _Error("A block is missed.")
    EndIf
    Repeat
      *Current\TokenItem = *ExpressionToken
      *Result = RunControlBlock() : _E()
      If IsValueTrue(*Result)
        OpenLevel(*TokenQueue, 0, 0, 0)
          RunBlock()
        CloseLevel() : _E() : _RB(1)
      Else
        Break
      EndIf
    ForEver
    _N()
  EndIf
EndProcedure
Procedure.i Statement_Repeat()
  Protected *Token.Token, *TokenQueue.Queue
  Protected *StatementObject.Object, *Indentifer.Object, *ArgumentQueue.Queue, *Value.Value
  Protected Max.i, Steps.i, Index.i, *StartTokenItem.QueueItem
  Protected *Result.Value, *ResultQueue.Queue
  
  *Token = *Current\TokenItem\Memory
  _N()
  
  *StatementObject = CreateGroup("_repeat")
  
  If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Round
    *ArgumentQueue = RunArguments(*Token\Block\TokenQueue)
    Select *ArgumentQueue\Count
      Case 1
        Index = 1
        Max   = ValueI(*ArgumentQueue\First\Value)
        Steps = 1
      Case 2
        Index = ValueI(*ArgumentQueue\First\Value)
        Max   = ValueI(*ArgumentQueue\Last\Value)
        Steps = 1
      Case 3
        Index = ValueI(*ArgumentQueue\First\Value)
        Max   = ValueI(*ArgumentQueue\First\N\Value)
        Steps = ValueI(*ArgumentQueue\Last\Value)
      Default
        _Error("Invalid count of arguments: " + Form(*ArgumentQueue\Count, "argument", "arguments"))   
    EndSelect
    _SN(*Token)
  EndIf
  
  If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Square
    *Token = *Token\Block\TokenQueue\First\Token
    If *Token\Type = #Token_Name
      *Indentifer = CreateVariable(*Token\Name, #Null, *StatementObject)
    Else
      _Error("Indentifer not specified in given brackets!")
    EndIf
  Else
    *Indentifer = CreateVariable("index", #Null, *StatementObject)
    _P()
  EndIf
  
  *Value                     = _New(Value)
  *Indentifer\Variable\Value = *Value
  
  *Value\Class    = *IntegerClass
  *Value\Memory   = @Index
  *Value\Scope    = #Value_Binary|#Value_Variable
  *Value\Variable = *Indentifer
  
  OpenLevel(0, 0, *StatementObject, 0, #True)
  *StartTokenItem = *Current\TokenItem
  
  If Steps
    If Index <= Max
      While Index <= Max
        *Result = Statement_RunForCommands(*StartTokenItem, *Value) : _EB() : _RB(1)
        If *Result
          If *Result = *Value
            *Result = PushDuplicatedValue(*Result)
          EndIf
          If Not *ResultQueue
            *ResultQueue = CreateQueue(@FreeValue())
          EndIf
          AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
        EndIf
        Index + Steps
      Wend
    Else
      While Index => Max
        *Result = Statement_RunForCommands(*StartTokenItem, *Value) : _EB() : _RB(1)
        If *Result
          If *Result = *Value
            *Result = PushDuplicatedValue(*Result)
          EndIf
          If Not *ResultQueue
            *ResultQueue = CreateQueue(@FreeValue())
          EndIf
          AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
        EndIf
        Index - Steps
      Wend
    EndIf
  Else
    _SN(*Token)
    If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Curly
      *TokenQueue = *Token\Block\TokenQueue
    Else
      _Error("Missing a block.")
    EndIf
    Repeat
      OpenLevel(*TokenQueue, 0, 0, 0)
        RunBlock()
      CloseLevel()
      _EB()
      _RB(1)
      Index + 1
    ForEver
  EndIf
  
  CloseLevel()
  
  If *ArgumentQueue
    FreeQueue(*ArgumentQueue)
  EndIf
  _Free(*Indentifer\Variable\Value)
  *Indentifer\Variable\Value = #Null
  FreeObject(*StatementObject)
  
  If *ResultQueue
    ProcedureReturn PushQueueValue(*ResultQueue)
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i Statement_For(*Value.Value)
  Protected *ArgumentQueue.Queue, *ResultQueue.Queue, *Result.Value, *PreviousResult.Value
  Protected *Token.Token, *IndentiferTokenItem.QueueItem, *IndentiferToken.Token, *StartTokenItem.QueueItem
  Protected *StatementObject, *IndentiferQueue.Queue, *IndentiferObject.Object
  Protected *IndexItem.QueueItem, *ValueItem.QueueItem, *SubValue.Value, *SubItem.QueueItem, *Object.Object
  Protected Index.i, *Pointer.Integer
  
  *Token = *Current\TokenItem\Memory
  _N()
  
  If Not *Value
    If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Round
      *ArgumentQueue = RunArguments(*Token\Block\TokenQueue) : _E()
      If *ArgumentQueue\Count = 1
        *Value = *ArgumentQueue\First\Value
      ElseIf *ArgumentQueue\Count
        *Value = PushQueueValue(*ArgumentQueue, #True)
      Else
        _Error("No arguments typed.")
      EndIf
      _SN(*Token)
    Else
      _Error("No argument queue typed.")
    EndIf
  ElseIf *Value\Class <> *QueueClass
    *Result = CallClassMethod(*Value\Class, #ClassMethod_Iterate, 0, *Value, #True, #False)
    If *Result
      *Value = *Result
    EndIf
  EndIf
  
  *StatementObject = CreateGroup("_for")
  
  If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Square
    *IndentiferQueue = CreateQueue(#Queue_SaveMemory)
    
    *IndentiferTokenItem = *Token\Block\TokenQueue\First
    While *IndentiferTokenItem
      *IndentiferToken = *IndentiferTokenItem\Memory
      
      Select *IndentiferToken\Type
        Case #Token_Name
          AppendQueueItem(*IndentiferQueue, CreateVariable(*IndentiferToken\Name, 0, *StatementObject))
          _Move(*IndentiferTokenItem, N)
        Case #Token_Seperator
          Select *IndentiferToken\Seperator
            Case #Seperator_Block
              Break
            Case #Seperator_Argument
              _Move(*IndentiferTokenItem, N)
            Default
              _Error("Invalid token typed in an indentifer queue: " + #Q + FormatToken(*IndentiferToken) + #Q, Statement_FreeFor(*ArgumentQueue, *StatementObject, *IndentiferObject, *IndentiferQueue))
          EndSelect
        Default
          _Error("Invalid token typed in an indentifer queue: " + #Q + FormatToken(*IndentiferToken) + #Q, Statement_FreeFor(*ArgumentQueue, *StatementObject, *IndentiferObject, *IndentiferQueue))
      EndSelect
    Wend
    
    If *IndentiferQueue\Count = 1
      *IndentiferObject = *IndentiferQueue\First\Object
      FreeQueue(*IndentiferQueue)
      *IndentiferQueue = #Null
    EndIf
  Else
    *IndentiferObject = CreateVariable("current", #Null, *StatementObject)
    _P()
  EndIf
  
  OpenLevel(0, 0, *StatementObject, 0, #True)
  *StartTokenItem = *Current\TokenItem
  
  If *Value\Class = *QueueClass
    If *IndentiferObject
      *ValueItem = *Value\Queue\First
      While *ValueItem
        *IndentiferObject\Variable\Value = *ValueItem\Value
        *Result = Statement_RunForCommands(*StartTokenItem, *ValueItem\Value) : _E("", Statement_FreeFor(*ArgumentQueue, *StatementObject, *IndentiferObject, *IndentiferQueue)) : _RB(1)
        If *Result
          If Not *ResultQueue
            *ResultQueue = CreateQueue(@FreeValue())
          EndIf
          AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
        EndIf
        _Move(*ValueItem, N)
      Wend
    Else
      *ValueItem = *Value\Queue\First
      While *ValueItem
        *SubValue = *ValueItem\Value
        If *SubValue\Class = *QueueClass
          If *SubValue\Queue\Count = *IndentiferQueue\Count
            *SubItem   = *SubValue\Queue\First
            *IndexItem = *IndentiferQueue\First
            While *SubItem
              *IndexItem\Object\Variable\Value = *SubItem\Value
              _Move(*SubItem, N)
              _Move(*IndexItem, N)
            Wend
          Else
            _Error("Sub queue do not join in to the count of indentifers: " + #Q + Str(*SubValue\Queue\Count) + ":" + Str(*IndentiferQueue\Count) + #Q)
          EndIf
        Else
          _Error("No sub queue definied: " + #Q + ClassName(*SubValue\Class) + #Q)
        EndIf
        *Result = Statement_RunForCommands(*StartTokenItem, *SubValue) : _E("", Statement_FreeFor(*ArgumentQueue, *StatementObject, *IndentiferObject, *IndentiferQueue)) : _RB(1)
        If *Result
          If Not *ResultQueue
            *ResultQueue = CreateQueue(@FreeValue())
          EndIf
          AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
        EndIf
        _Move(*ValueItem, N)
      Wend
    EndIf
  Else
    If *IndentiferObject
      *IndentiferObject\Variable\Value = *Value
      *PreviousResult = Statement_RunForCommands(*StartTokenItem, *Value)
    Else
      _Error("More than one indentifer is not allowed for a single value.")
    EndIf
  EndIf
  
  Statement_FreeFor(*ArgumentQueue, *StatementObject, *IndentiferObject, *IndentiferQueue)
  CloseLevel()
  
  If *ResultQueue
    ProcedureReturn PushQueueValue(*ResultQueue)
  ElseIf *PreviousResult
    ProcedureReturn *PreviousResult
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i Statement_RunForCommands(*StartTokenItem.QueueItem, *Value.Value)
  Protected *Token.Token, *ResultQueue.Queue, *Result.Value, *PreviousResult.Value
  Protected *Object.Object, *OtherValue.Value
  
  *Current\TokenItem = *StartTokenItem
  *Token             = *StartTokenItem\Memory
  _N()
  
  Repeat
    Select *Token\Type
      Case #Token_Block
        Select *Token\Block\Bracket
          Case #Bracket_Curly
            OpenLevel(*Token\Block\TokenQueue, 0, 0, *Value)
              RunBlock() : _E() : _RB()
            CloseLevel()
          Case #Bracket_Round
            OpenLevel(*Token\Block\TokenQueue, 0, 0, *Value, #True)
              *Result = Run() : _E()
            CloseLevel()
            If *Result\Class = *VariantClass
              *Result = 0
            EndIf
        EndSelect
      Case #Token_Name
        If *Value\Scope&#Value_Searchable
          *Object = RunObjectPath(*Value\Object, 0, *Token\Name) : _E()
          *Result = RunObject(*Object, 0)
        Else
          _Error("Value is not searchable: " + #Q + ClassName(*Value\Class) + #Q)
        EndIf
      Case #Token_Operator
        *OtherValue = Run(-1)                                        : _E()
        *Result = OperateValue(*Token\Operator, *Value, *OtherValue) : _E()
      Case #Token_Control
        If *Token\Control = #Control_Class
          *Object = RunObjectPath(*Value\Class, 0, "", *Value, @*OtherValue) : _E()
          *Result = RunObject(*Object, *OtherValue)
        Else
          _Error("Invalid token typed: " + #Q + FormatToken(*Token) + #Q)
        EndIf
    EndSelect
    
    _SN(*Token)
    
    Select *Token\Type
      Case #Token_Seperator
        If *Result
          If *PreviousResult
            If Not *ResultQueue
              *ResultQueue = CreateQueue(@FreeValue())
              AppendQueueItem(*ResultQueue, CreateReferenceValue(*PreviousResult))
            EndIf
            AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
          Else
            *PreviousResult = *Result
          EndIf
        EndIf
        *Result = #Null
        
        Select *Token\Seperator
          Case #Seperator_Argument
            _SN(*Token)
          Default
            _P()
            Break
        EndSelect
      Default
        _Error("Invalid token typed: " + #Q + FormatToken(*Token) + #Q)
    EndSelect
  ForEver
  
  If *ResultQueue
    ProcedureReturn PushQueueValue(*ResultQueue)
  ElseIf *PreviousResult
    ProcedureReturn *PreviousResult
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i Statement_FreeFor(*ArgumentQueue.Queue, *StatementObject.Object, *IndentiferObject.Object, *IndentiferQueue.Queue)
  Protected *Item.QueueItem
  
  If *ArgumentQueue
    FreeQueue(*ArgumentQueue)
  EndIf
  If *IndentiferQueue
    *Item = *IndentiferQueue\First
    While *Item
      *Item\Object\Variable\Value = 0
      _Move(*Item, N)
    Wend
    FreeQueue(*IndentiferQueue)
  Else
    *IndentiferObject\Variable\Value = 0
  EndIf
  If *StatementObject
    FreeObject(*StatementObject)
  EndIf
EndProcedure
Procedure.i Statement_Switch(*Value.Value)
  Protected *Token.Token, *StartTokenItem.QueueItem, *Item.QueueItem
  Protected *StatementObject.Object, *Indentifer.Object, *ArgumentQueue.Queue
  Protected *Result.Value, *ResultQueue.Queue, *PreviousResult.Value
  
  If Not *Value
    _SN(*Token)
    If *Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Round
      *ArgumentQueue = RunArguments(*Token\Block\TokenQueue) : _EG(Statement_Switch)
      If *ArgumentQueue\Count = 1
        *Value = *ArgumentQueue\First\Value
      ElseIf *ArgumentQueue\Count
        *Value = PushQueueValue(*ArgumentQueue)
      Else
        _Error("No arguments typed.")
      EndIf
    Else
      _Error("No argument queue typed.")
    EndIf
  EndIf
  
  _SN(*Token)
  If Not(*Token\Type = #Token_Block And *Token\Block\Bracket = #Bracket_Curly)
    _Error("No block definied.")
  EndIf
  
  *StatementObject = CreateGroup("_switch")
    *Indentifer = CreateVariable("case", 0, *StatementObject)
  *StartTokenItem = *Token\Block\TokenQueue\First
  
  OpenLevel(*Token\Block\TokenQueue, 0, 0, 0, #True)
  
  If *Value\Class = *QueueClass
    *Item = *Value\Queue\First
    While *Item
      *Value = *Item\Value
      
      *Indentifer\Variable\Value = *Value
      *Result = Statement_SwitchBlock(*StartTokenItem, *Value) : _EB() : _RB()
      
      If *Result
        If *PreviousResult
          If Not *ResultQueue
            *ResultQueue = CreateQueue(@FreeValue())
            AppendQueueItem(*ResultQueue, CreateReferenceValue(*PreviousResult))
          EndIf
          AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
        Else
          *PreviousResult = *Result
        EndIf
      EndIf
      
      _Move(*Item, N)
    Wend
  Else
    *Indentifer\Variable\Value = *Value
    *Result = Statement_SwitchBlock(*StartTokenItem, *Value)
  EndIf
  
  Statement_Switch_Error:
  
  CloseLevel()
  
  If *StatementObject
    If *Indentifer
      *Indentifer\Variable\Value = #Null
    EndIf
    FreeObject(*StatementObject)
  EndIf
  If *ArgumentQueue
    FreeQueue(*ArgumentQueue)
  EndIf
  
  If *ResultQueue
    ProcedureReturn PushQueueValue(*ResultQueue)
  ElseIf *Result
    ProcedureReturn *Result
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i Statement_SwitchBlock(*StartTokenItem.QueueItem, *Value.Value)
  Protected *Token.Token, IsValueTrue.i
  Protected *Result.Value, *ResultQueue.Queue, *PreviousResult.Value
  
  *Current\TokenItem = *StartTokenItem\N
  *Token             = *StartTokenItem\Token
  
  Repeat
    Select *Token\Type
      Case #Token_Block, #Token_Value
        If IsValueTrue = #False
          *Result = Statement_SwitchCase(*Token, *Value) : _E() : _R()
          If *Result
            IsValueTrue = #True
          EndIf
        Else
          _N()
        EndIf
      Case #Token_Keyword
        If IsValueTrue = #False
          _SN(*Token)
          OpenLevel(*Token\Block\TokenQueue, 0, 0, *Value, #True)
            *Result = RunBlock()
          CloseLevel()
          _E()
          _R()
        Else
          _N()
        EndIf
      Case #Token_Seperator
        Select *Token\Seperator
          Case #Seperator_Command
            IsValueTrue = #Null
          Case #Seperator_Block
            Break
        EndSelect
    EndSelect
    
    If *Result
      If *PreviousResult
        If Not *ResultQueue
          *ResultQueue = CreateQueue(@FreeValue())
          AppendQueueItem(*ResultQueue, CreateReferenceValue(*PreviousResult))
        EndIf
        AppendQueueItem(*ResultQueue, CreateReferenceValue(*Result))
      Else
        *PreviousResult = *Result
      EndIf
      *Result = #Null
    EndIf
    
    _SN(*Token)
  ForEver
  
  If *ResultQueue
    ProcedureReturn PushQueueValue(*ResultQueue)
  ElseIf *Result
    ProcedureReturn *Result
  ElseIf *PreviousResult
    ProcedureReturn *PreviousResult
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i Statement_SwitchCase(*Token.Token, *Value.Value)
  Protected TrueCase.i, *Result.Value
  Protected *ArgumentQueue.Queue, *Item.QueueItem
  
  Select *Token\Type
    Case #Token_Block
      Select *Token\Block\Bracket
        Case #Bracket_Square
          *ArgumentQueue = RunArguments(*Token\Block\TokenQueue)
          *Item = *ArgumentQueue\First
          While *Item
            If IsValueEqual(*Value, *Item\Value)
              TrueCase = #True
              Break
            EndIf
            _Move(*Item, N)
          Wend
        Case #Bracket_Curly
          OpenLevel(*Token\Block\TokenQueue, 0, 0, *Value, #True)
            *Result = Run()
          CloseLevel()
          _E()
          TrueCase = IsValueTrue(*Result)
      EndSelect
    Case #Token_Value
      TrueCase = IsValueEqual(*Value, *Token\Value)
  EndSelect
  
  If TrueCase
    _SN(*Token)
    OpenLevel(*Token\Block\TokenQueue, 0, 0, *Value, #True)
      *Result = RunBlock()
    CloseLevel()
     _E()
     _R()
    ProcedureReturn *Result
  Else
    _N()
  EndIf
  
  ProcedureReturn #Null
EndProcedure

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 538
; FirstLine = 517
; Folding = --
; EnableXP