;**************************************************************
;     Term
;
; Autor: Josef Sniatecki
; Creation date: 09.12.2009
;**************************************************************
;   (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



#IncludeTerm = #False

CompilerIf #IncludeTerm



Procedure.i FreeTerm(*Term.Term)
  Select *Term\Type
    Case #Term_Sum, #Term_Product
      FreeQueue(*Term\Children)
  EndSelect
  _Free(*Term)
  
  ProcedureReturn #True
EndProcedure
Procedure.i CopyTerm(*SourceTerm.Term, *DestinationTerm.Term)
  *DestinationTerm\Type = *SourceTerm\Type
  
  Select *SourceTerm\Type
    Case #Term_Number
      *DestinationTerm\Number = *SourceTerm\Number
    Case #Term_Indentifer
      *DestinationTerm\Indentifer = *SourceTerm\Indentifer
    Case #Term_Sum, #Term_Product
      *DestinationTerm\Children = CreateQueue(@FreeTerm())
      CopyQueue(*SourceTerm\Children, *DestinationTerm\Children, @CopyTerm())
  EndSelect
  
  ProcedureReturn *DestinationTerm
EndProcedure

Procedure.i CreateTermNumber(Number.d)
  Protected *Term.Term
  
  *Term = _New(Term)
  
  *Term\Type   = #Term_Number
  *Term\Number = Number
  
  ProcedureReturn *Term
EndProcedure
Procedure.i CreateTermIndentifer(Name.s)
  Protected *Term.Term
  
  *Term = _New(Term)
  
  *Term\Type       = #Term_Indentifer
  *Term\Indentifer = Name
  
  ProcedureReturn *Term
EndProcedure
Procedure.i CreateTermSum(*Children.Queue = #Null)
  Protected *Term.Term
  
  *Term = _New(Term)
  
  *Term\Type     = #Term_Sum
  If *Children
    *Term\Children = *Children
  Else
    *Term\Children = CreateQueue(@FreeTerm())
  EndIf
  
  ProcedureReturn *Term
EndProcedure
Procedure.i CreateTermProduct(*Children.Queue = #Null)
  Protected *Term.Term
  
  *Term = _New(Term)
  
  *Term\Type     = #Term_Product
  If *Children
    *Term\Children = *Children
  Else
    *Term\Children = CreateQueue(@FreeTerm())
  EndIf
  
  ProcedureReturn *Term
EndProcedure
Procedure.i CreateTermDivision()
  Protected *Term.Term
  
  *Term = _New(Term)
  
  *Term\Type     = #Term_Division
  *Term\Children = CreateQueue(@FreeTerm())
  
  ProcedureReturn *Term
EndProcedure
Procedure.i CreateTermPower()
  Protected *Term.Term
  
  *Term = _New(Term)
  
  *Term\Type = #Term_Power
  
  ProcedureReturn *Term
EndProcedure

Procedure.i CreateTermSingleProduct(*Factor.Term)
  Protected *Term.Term
  
  *Term = _New(Term)
  
  *Term\Type     = #Term_Product
  *Term\Children = CreateQueue(@FreeTerm())
  AppendQueueItem(*Term\Children, *Factor)
  
  ProcedureReturn *Term
EndProcedure
Procedure.i CreateTermAddition(*A.Term, *B.Term)
  Protected *Term.Term
  
  *Term = CreateTermSum()
  AppendQueueItem(*Term\Children, *A)
  AppendQueueItem(*Term\Children, *B)
  
  ProcedureReturn *Term
EndProcedure
Procedure.i CreateTermMultiplication(*A.Term, *B.Term)
  Protected *Term.Term
  
  *Term = CreateTermProduct()
  AppendQueueItem(*Term\Children, *A)
  AppendQueueItem(*Term\Children, *B)
  
  ProcedureReturn *Term
EndProcedure



Procedure.i CompileTerm(*Level.CompileTerm, Priority.i = 0)
  Protected *Term.Term, *SubTerm.Term, *HeaderTerm.Term, *Token.Token, Level.CompileTerm
  Protected Number.d
  
  With *Level
  
  While \TokenItem
    *Token = \TokenItem\Memory
    
    Select *Token\Type
      Case #Token_Value
        Number   = ValueD(*Token\Value) : _E()
        *SubTerm = CreateTermNumber(Number)
      Case #Token_Name
        If *SubTerm
          *Term = CreateTermProduct()
          AppendQueueItem(*Term\Children, *SubTerm)
          AppendQueueItem(*Term\Children, CreateTermIndentifer(*Token\Name))
          *SubTerm = #Null
        Else
          *SubTerm = CreateTermIndentifer(*Token\Name)
        EndIf
      Case #Token_Operator
        Select *Token\Operator
          Case #Operator_Add
            If Priority < 1
              If Not (*Term And *Term\Type = #Term_Sum)
                If *Term
                  *SubTerm = *Term
                EndIf
                *Term = CreateTermSum()
              EndIf
              If *SubTerm
                AppendQueueItem(*Term\Children, *SubTerm)
              EndIf
              
              _Move(\TokenItem, N)
              *SubTerm = CompileTerm(*Level, 1) : _E()
              AppendQueueItem(*Term\Children, *SubTerm)
              *SubTerm = #Null
            Else
              _Move(\TokenItem, P)
              Break
            EndIf
          Case #Operator_Multiply
            If Priority < 2
              If Not (*Term And *Term\Type = #Term_Product)
                If *Term
                  *SubTerm = *Term
                EndIf
                *Term = CreateTermProduct()
              EndIf
              If *SubTerm
                AppendQueueItem(*Term\Children, *SubTerm)
              EndIf
              
              _Move(\TokenItem, N)
              *SubTerm = CompileTerm(*level, 2) : _E()
              AppendQueueItem(*Term\Children, *SubTerm)
              *SubTerm = #Null
            Else
              _Move(\TokenItem, P)
              Break
            EndIf
          Case #Operator_Divide
            If Priority < 2
              If Not (*Term And *Term\Type = #Term_Division)
                If *Term
                  *SubTerm = *Term
                EndIf
                *Term = CreateTermDivision()
              EndIf
              If *SubTerm
                AppendQueueItem(*Term\Children, *SubTerm)
              EndIf
              
              _Move(\TokenItem, N)
              *SubTerm = CompileTerm(*Level, 2) : _E()
              AppendQueueItem(*Term\Children, *SubTerm)
              *SubTerm = #Null
            Else
              _Move(\TokenItem, P)
              Break
            EndIf
        EndSelect
      Case #Token_Block
        If *SubTerm
          If *Term
            *HeaderTerm = *Term
          EndIf
          
          *Term = CreateTermProduct()
          AppendQueueItem(*Term\Children, *SubTerm)
          *SubTerm = #Null
          
          Level\TokenQueue = *Token\Block\TokenQueue
          Level\TokenItem  = *Token\Block\TokenQueue\First
          AppendQueueItem(*Term\Children, CompileTerm(@Level, 0))
          
          If *HeaderTerm
            AppendQueueItem(*HeaderTerm\Children, *Term)
            *Term = *HeaderTerm
          EndIf
        Else
          If *Term
            Level\TokenQueue = *Token\Block\TokenQueue
            Level\TokenItem  = *Token\Block\TokenQueue\First
            AppendQueueItem(*Term\Children, CompileTerm(@Level, 0))
          Else
            Level\TokenQueue = *Token\Block\TokenQueue
            Level\TokenItem  = *Token\Block\TokenQueue\First
            *SubTerm = CompileTerm(@Level, 0)
          EndIf
        EndIf
      Case #Token_Seperator
        Select *Token\Seperator
          Case #Seperator_Command, #Seperator_Block
            _Move(\TokenItem, P)
            Break
        EndSelect
    EndSelect
    
    _Move(\TokenItem, N)
  Wend
  
  If *SubTerm
    ProcedureReturn *SubTerm
  Else
    ProcedureReturn *Term
  EndIf
EndProcedure
Procedure.i CompileTermByTokens(*TokenQueue.Queue)
  Protected Level.CompileTerm
  
  Level\TokenQueue = *TokenQueue
  Level\TokenItem  = *TokenQueue\First
  
  ProcedureReturn CompileTerm(@Level)
EndProcedure

; Informations:
Procedure.i IsTermEqual(*A.Term, *B.Term)
  Protected *Item.QueueItem, *OtherItem.QueueItem, Equal.i
  Protected *Child.Term, *OtherChild.Term
  Protected *Checked, *CheckedPointer.Byte
  
  Select *A\Type
    Case #Term_Number
      If *B\Type = #Term_Number And *B\Number = *A\Number
        ProcedureReturn #True
      EndIf
    Case #Term_Indentifer
      If *B\Type = #Term_Indentifer And *B\Indentifer = *A\Indentifer
        ProcedureReturn #True
      EndIf
    Case #Term_Sum, #Term_Product
      If *A\Children\Count = *B\Children\Count
        Equal = #True
        *Checked = _Dim(*B\Children\Count * _Sz(Byte))
        
        *Item = *A\Children\First
        While *Item
          *Child = *Item\Memory
          *CheckedPointer = *Checked
          *OtherItem = *B\Children\First
          While *OtherItem
            If Not *CheckedPointer\B
              If Not IsTermEqual(*Child, *OtherItem\Memory)
                Equal = #False
                Break 2
              EndIf
            EndIf
            _Move(*OtherItem, N)
          Wend
          _Move(*Item, N)
        Wend
        
        _Free(*Checked)
        
        ProcedureReturn Equal
      EndIf
  EndSelect
  
  ProcedureReturn #False
EndProcedure

; Transformations:
Procedure.i AddTerm(*A.Term, *B.Term)
  Protected *Term.Term, *Term2.Term, *Term3.Term, *Item.QueueItem, *OtherItem.QueueItem
  Protected *Factors.Queue, *Factors2.Queue, *Queue.Queue, *Queue2.Queue
  Protected *Checked, *CheckedPointer.Byte
  Protected *Result.Term
  
  Debug "Simplify: " + FormatTerm(*A) + " + " + FormatTerm(*B)
  
  Select *A\Type
    Case #Term_Number ;{ number + ...
      Select *B\Type
        Case #Term_Number
          *Result = CreateTermNumber(*A\Number + *B\Number)
        Case #Term_Product
          Select *B\Type
            Case #Term_Number
              *Result = AddTerm(CreateTermSingleProduct(*A), *B)
              *B      = #Null
            Case #Term_Sum
              *Queue = CopyQueue(*B\Children, CreateQueue(@FreeTerm()), @CopyTerm())
              *Term  = CreateTermSum(*Queue)
              PrependQueueItem(*Queue, *A)
              *Result = SimplifyTermF(*Term)
              *A      = #Null
          EndSelect
      EndSelect
      ;}
    Case #Term_Sum ;{ (a+b) + ...
      *Queue = CopyQueue(*A\Children, CreateQueue(@FreeTerm()), @CopyTerm())
      *Term  = CreateTermSum(*Queue)
      AppendQueueItem(*Queue, *B)
      *Result = SimplifyTermF(*Term)
      *B      = #Null
    ;}
    Case #Term_Product ;{ a*b + ...
      Select *B\Type
        Case #Term_Product ;{ a*b + c*d
          *Checked = _Dim(*B\Children\Count * _Sz(Byte))
          *Factors  = CreateQueue()
          *Factors2 = CreateQueue()
          
          *Item = *A\Children\First
          While *Item
            *Term           = *Item\Memory
            *OtherItem      = *B\Children\First
            *CheckedPointer = *Checked
            While *OtherItem
              If Not *CheckedPointer\B
                *Term2 = *OtherItem\Memory
                If *Term\Type = *Term2\Type
                  If IsTermEqual(*Term, *Term2)
                    AppendQueueItem(*Factors, *Item)
                    AppendQueueItem(*Factors2, *OtherItem)
                    *CheckedPointer\B = #True
                    Break
                  EndIf
                EndIf
              EndIf
              *CheckedPointer + _Sz(Byte)
              _Move(*OtherItem, N)
            Wend
            _Move(*Item, N)
          Wend
          
          *Term = CreateTermProduct(CopyFactors(*Factors))
          Debug "Common factors: " + FormatTerm(*Term)
          FreeTerm(*Term)
          
          _Free(*Checked)
          
          If *Factors\Count = *A\Children\Count And *Factors2\Count = *B\Children\Count
            ; x*y + x*y = 2(x*y):
            *Term  = CreateTermProduct(CopyFactors(*Factors))
            PrependQueueItem(*Term\Children, CreateTermNumber(2))
            
            *Result = SimplifyTerm(*Term)
          Else
            ;a*x + a*y = a(x*y):
            *Term = SimplifyTermF(CreateTermProduct(CopyFactors(*Factors)))
            
            *Term2 = CreateTermSum()
            
            If *Factors\Count = *A\Children\Count
              AppendQueueItem(*Term2\Children, CreateTermProduct(RestFactors(*B\Children, *Factors2)))
              AppendQueueItem(*Term2\Children, CreateTermNumber(1))
            ElseIf *Factors2\Count = *B\Children\Count
              AppendQueueItem(*Term2\Children, CreateTermProduct(RestFactors(*A\Children, *Factors)))
              AppendQueueItem(*Term2\Children, CreateTermNumber(1))
            Else
              AppendQueueItem(*Term2\Children, CreateTermProduct(RestFactors(*A\Children, *Factors)))
              AppendQueueItem(*Term2\Children, CreateTermProduct(RestFactors(*B\Children, *Factors2)))
            EndIf
            
            *Result = CreateTermMultiplication(*Term, SimplifyTermF(*Term2))
          EndIf
          
          FreeQueue(*Factors)
          FreeQueue(*Factors2)
        ;}
        Case #Term_Number, #Term_Indentifer ;{
          *Result = AddTerm(*A, CreateTermSingleProduct(*B))
          *A      = #Null
        ;}
      EndSelect
      ;}
    Case #Term_Indentifer ;{ x + ...
      Select *B\Type
        Case #Term_Indentifer ;{ x + x
          If *B\Indentifer = *A\Indentifer
            *Result = CreateTermMultiplication(CreateTermNumber(2), *A)
            *A      = #Null
          EndIf
          ;}
        Case #Term_Product ;{ x + y*z
          *Result = AddTerm(CreateTermSingleProduct(*A), *B)
          *B      = #Null
        ;}
        Default ;{
          *Result = CreateTermAddition(*A, *B)
          *A      = #Null
          *B      = #Null
        ;}
      EndSelect
      ;}
  EndSelect
  
  If *A : FreeTerm(*A) : EndIf
  If *B : FreeTerm(*B) : EndIf
  
  If *Result
    Debug "Sum: " + FormatTerm(*Result)
  Else
    Debug "No sum"
  EndIf
  
  ProcedureReturn *Result
EndProcedure
Procedure.i MultiplyTerm(*A.Term, *B.Term)
  Protected *Result.Term, *Temp.Term
  
  Select *A\Type
    Case #Term_Number
      Select *B\Type
        Case #Term_Number
          ProcedureReturn CreateTermNumber(*A\Number * *B\Number)
        Case #Term_Sum
          *B = SimplifyTerm(*B)
          Select *B\Type
            Case #Term_Number
              ProcedureReturn CreateTermNumber(*A\Number * *B\Number)
          EndSelect
        Case #Term_Product
          *B = SimplifyTerm(*B)
          Select *B\Type
            Case #Term_Number
              ProcedureReturn CreateTermNumber(*A\Number * *B\Number)
            Case #Term_Product
              *Result = CreateTermProduct()
              AppendQueueItem(*Result\Children, *A)
              CopyQueue(*B\Children, *Result\Children, @CopyTerm(), *Result\Children\Last)
              *A = SimplifyTerm(*Result)
              FreeTerm(*Result)
              ProcedureReturn *A
          EndSelect
      EndSelect
    Case #Term_Sum
      *A = SimplifyTerm(*A)
      *B = SimplifyTerm(*B)
      If *A\Type = #Term_Sum
        ProcedureReturn CreateTermMultiplication(*A, *B)
      Else
        ProcedureReturn MultiplyTerm(*A, *B)
      EndIf
  EndSelect
EndProcedure

Procedure.i RestFactors(*AllFactors.Queue, *Factors.Queue)
  Protected *Result.Queue
  
  *Factors = SubtractQueue(*AllFactors, *Factors)
  *Result  = CopyQueue(*Factors, CreateQueue(@FreeTerm()), @CopyTerm())
  FreeQueue(*Factors)
  
  ProcedureReturn *Result
EndProcedure
Procedure.i CopyFactors(*Factors.Queue)
  Protected *Result.Queue
  
  *Factors = MemoryQueue(*Factors)
  *Result  = CopyQueue(*Factors, CreateQueue(@FreeTerm()), @CopyTerm())
  FreeQueue(*Factors)
  
  ProcedureReturn *Result
EndProcedure

Procedure.i SimplifyTermF(*Term.Term)
  Protected *Result.Term
  
  *Result = SimplifyTerm(*Term)
  FreeTerm(*Term)
  
  ProcedureReturn *Result
EndProcedure
Procedure.i SimplifyTerm(*Term.Term)
  Protected *Item.QueueItem, *OtherItem.QueueItem
  Protected *ResultTerm.Term, *Result.Term, *PreviousResult.Term
  Protected *ChildTerm.Term, *OtherChildTerm.Term
  Protected *Checked, *CheckedTerm.Byte, *CheckedTerm2.Byte
  
  Select *Term\Type
    Case #Term_Number, #Term_Indentifer
      ProcedureReturn CopyTerm(*Term, _New(Term))
      
    Case #Term_Sum
      *Checked     = _Dim(*Term\Children\Count * _Sz(Byte))
      *CheckedTerm = *Checked
      
      *Item = *Term\Children\First
      While *Item
        If Not *CheckedTerm\B
          *ChildTerm  = SimplifyTerm(*Item\Memory)
          
          If *ResultTerm
            *OtherItem = *ResultTerm\Children\First
            While *OtherItem
              *OtherChildTerm = CopyTerm(*OtherItem\Memory, _New(Term))
              *Result = AddTerm(*ChildTerm, *OtherChildTerm)
              If *Result
                FreeQueueItem(*ResultTerm\Children, *OtherItem)
                *CheckedTerm\B  = #True
                Break
              EndIf
              _Move(*OtherItem, N)
            Wend
          ElseIf *PreviousResult
            *Result = AddTerm(*ChildTerm, CopyTerm(*PreviousResult, _New(Term)))
            If *Result
              FreeTerm(*PreviousResult)
              *PreviousResult = #Null
              *CheckedTerm\B  = #True
            EndIf
          EndIf
          
          If Not *Result
            *CheckedTerm2 = *Checked
            *OtherItem    = *Term\Children\First
            While *OtherItem
              If Not (*OtherItem = *Item Or *CheckedTerm2\B)
                *OtherChildTerm = SimplifyTerm(*OtherItem\Memory)
                *Result = AddTerm(*ChildTerm, *OtherChildTerm)
                If *Result
                  *CheckedTerm\B  = #True
                  *CheckedTerm2\B = #True
                  Break
                EndIf
              EndIf
              *CheckedTerm2 + _Sz(Byte)
              _Move(*OtherItem, N)
            Wend
          EndIf
          
          If Not *Result
            *Result = CopyTerm(*ChildTerm, _New(Term))
          EndIf
          
          If *PreviousResult
            If Not *ResultTerm
              *ResultTerm = CreateTermSum()
              AppendQueueItem(*ResultTerm\Children, *PreviousResult)
            EndIf
            AppendQueueItem(*ResultTerm\Children, *Result)
          EndIf
          *PreviousResult = *Result
          *Result         = #Null
        EndIf
        
        *CheckedTerm + _Sz(Byte)
        _Move(*Item, N)
      Wend
      
      *CheckedTerm = *Checked
      *Item        = *Term\Children\First
      While *Item
        If Not *CheckedTerm\B
          *ChildTerm = CopyTerm(*Item\Memory, _New(Term))
          If *PreviousResult
            If Not *ResultTerm
              *ResultTerm = CreateTermSum()
              AppendQueueItem(*ResultTerm\Children, *PreviousResult)
            EndIf
            AppendQueueItem(*ResultTerm\Children, *ChildTerm)
          Else
            *PreviousResult = *ChildTerm
          EndIf
        EndIf
        _Move(*Item, N)
      Wend
      
      _Free(*Checked)
    Case #Term_Product
      *Checked     = _Dim(*Term\Children\Count * _Sz(Byte))
      *CheckedTerm = *Checked
      
      *Item = *Term\Children\First
      While *Item
        If Not *CheckedTerm\B
          *ChildTerm = SimplifyTerm(*Item\Memory)
          
          If *ResultTerm
            *OtherItem = *ResultTerm\Children\First
            While *OtherItem
              *OtherChildTerm = CopyTerm(*OtherItem\Memory, _New(Term))
              *Result = MultiplyTerm(*ChildTerm, *OtherChildTerm)
              If *Result
                FreeQueueItem(*ResultTerm\Children, *OtherItem)
                *CheckedTerm\B  = #True
                Break
              EndIf
              _Move(*OtherItem, N)
            Wend
          ElseIf *PreviousResult
            *Result = MultiplyTerm(*ChildTerm, CopyTerm(*PreviousResult, _New(Term)))
            If *Result
              FreeTerm(*PreviousResult)
              *PreviousResult = #Null
              *CheckedTerm\B  = #True
            EndIf
          EndIf
          
          If Not *Result
            *CheckedTerm2 = *Checked
            *OtherItem = *Term\Children\First
            While *OtherItem
              If Not (*OtherItem = *Item Or *CheckedTerm2\B)
                *OtherChildTerm = SimplifyTerm(*OtherItem\Memory)
                *Result = MultiplyTerm(*ChildTerm, *OtherChildTerm)
                If *Result
                  *CheckedTerm\B  = #True
                  *CheckedTerm2\B = #True
                  Break
                EndIf
              EndIf
              *CheckedTerm2 + _Sz(Byte)
              _Move(*OtherItem, N)
            Wend
          EndIf
          
          If Not *Result
            *Result = CopyTerm(*ChildTerm, _New(Term))
          EndIf
          
          If *PreviousResult
            If Not *ResultTerm
              *ResultTerm = CreateTermProduct()
              AppendQueueItem(*ResultTerm\Children, *PreviousResult)
            EndIf
            AppendQueueItem(*ResultTerm\Children, *Result)
          EndIf
          *PreviousResult = *Result
          *Result         = #Null
        EndIf
        
        *CheckedTerm + _Sz(Byte)
        _Move(*Item, N)
      Wend
      
      *CheckedTerm = *Checked
      *Item        = *Term\Children\First
      While *Item
        If Not *CheckedTerm\B
          *ChildTerm = CopyTerm(*Item\Memory, _New(Term))
          If *PreviousResult
            If Not *ResultTerm
              *ResultTerm = CreateTermProduct()
              AppendQueueItem(*ResultTerm\Children, *PreviousResult)
            EndIf
            AppendQueueItem(*ResultTerm\Children, *ChildTerm)
          Else
            *PreviousResult = *ChildTerm
          EndIf
        EndIf
        _Move(*Item, N)
      Wend
      
      _Free(*Checked)
  EndSelect
  
  If *ResultTerm
    ProcedureReturn *ResultTerm
  ElseIf *PreviousResult
    ProcedureReturn *PreviousResult
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure

Procedure.s FormatTerm(*Term.Term, *Parent.Term = #Null)
  Protected *Item.QueueItem, *SubTerm.Term, Result.s
  
  Select *Term\Type
    Case #Term_Number
      ProcedureReturn StrDN(*Term\Number)
    Case #Term_Indentifer
      ProcedureReturn *Term\Indentifer
    Case #Term_Sum
      *Item = *Term\Children\First
      Result + "("
      While *Item
        *SubTerm = *Item\Memory
        If *Item\P
          Result + " + "
        EndIf
        Result + FormatTerm(*SubTerm, *Term)
        _Move(*Item, N)
      Wend
      Result + ")"
    Case #Term_Product
      *Item = *Term\Children\First
      Result + "("
      While *Item
        *SubTerm = *Item\Memory
        If *Item\P
          Result + "*"
        EndIf
        Result + FormatTerm(*SubTerm, *Term)
        _Move(*Item, N)
      Wend
      Result + ")"
    Case #Term_Division
      *Item = *Term\Children\First
      Result + "("
      While *Item
        *SubTerm = *Item\Memory
        If *Item\P
          Result + "/"
        EndIf
        Result + FormatTerm(*SubTerm)
        _Move(*Item, N)
      Wend
      Result + ")"
    Case #Term_Power
      Result + "(" + FormatTerm(*Term\FirstChild) + ")" + "^(" + FormatTerm(*Term\SecondChild)
  EndSelect
  
  ProcedureReturn Result
EndProcedure



Define *TokenQueue.Queue
Define *Term.Term


*TokenQueue = CreateQueue(@FreeToken())
Parse(*TokenQueue, @"3*x + 2*x*y + x")
*Term = CompileTermByTokens(*TokenQueue)

Debug "Formula: " + FormatTerm(*Term)
Debug "Result: " + FormatTerm(SimplifyTerm(*Term))

End

CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 1
; Folding = AA9-5-
; EnableXP