;**************************************************************
;     Token
;
; Author: Josef Sniatecki
; Creation date: 03.04.2009
;**************************************************************
;   Copyright (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



; Main:
Procedure.i AddIntegerToken(*Queue.Queue, Type.i, Integer.i)
  Protected *Token.Token
  
  *Token = _New(Token)
  AppendQueueItem(*Queue, *Token)
  
  *Token\Type    = Type
  *Token\Integer = Integer
  
  ProcedureReturn *Token
EndProcedure
Procedure.i AddValueToken(*Queue.Queue, Type.i, *Value.Value)
  Protected *Token.Token
  
  *Token = _New(Token)
  AppendQueueItem(*Queue, *Token)
  
  *Token\Type  = Type
  *Token\Value = *Value
  *Value\Scope |#Value_Literal
  
  ProcedureReturn *Token
EndProcedure
Procedure.i AddStringToken(*Queue.Queue, Type.i, String.s)
  Protected *Token.Token
  
  *Token = _New(Token)
  AppendQueueItem(*Queue, *Token)
  
  *Token\Type   = Type
  *Token\String = String
  
  ProcedureReturn *Token
EndProcedure
Procedure.i AddBlockToken(*Queue.Queue, Type.i, BracketID.i, *TokenQueue.Queue)
  Protected *Token.Token
  
  *Token = _New(Token)
  AppendQueueItem(*Queue, *Token)
  
  *Token\Type             = Type
  *Token\Block            = _New(Token_Block)
  *Token\Block\Bracket    = BracketID
  *Token\Block\TokenQueue = *TokenQueue
  
  ProcedureReturn *Token
EndProcedure
Procedure.i FreeToken(*Token.Token)
  Select *Token\Type
    Case #Token_Value
      FreeHardValue(*Token\Value)
    Case #Token_Block
      If *Token\Block\TokenQueue
        FreeQueue(*Token\Block\TokenQueue)
      EndIf
      _Free(*Token\Block)
  EndSelect
  
  ClearStructure(*Token, Token)
  _Free(*Token)
  
  ProcedureReturn #True
EndProcedure
Procedure.i CopyToken(*Source.Token, *Destination.Token)
  *Destination\Type = *Source\Type
  Select *Source\Type
    Case #Token_Name
      *Destination\String = *Source\String
    Case #Token_Value
      *Destination\Value = DuplicateValue(*Source\Value)
    Case #Token_Block
      *Destination\Block            = _New(Token_Block)
      *Destination\Block\Bracket    = *Source\Block\Bracket
      *Destination\Block\TokenQueue = CopyQueue(*Source\Block\TokenQueue, CreateQueue(@FreeToken()), @CopyToken())
    Default
      *Destination\Integer = *Source\Integer
  EndSelect
  ProcedureReturn *Destination
EndProcedure
; Formators:
Procedure.s FormatToken(*Token.Token)
  Protected String.s
  
  Select *Token\Type
    Case #Token_Name
      String = *Token\Name
    Case #Token_Value
      Select *Token\Value\Class
        Case *IntegerClass
          String = Str(*Token\Value\Integer\I)
        Case *FloatClass
          String = StrF(*Token\Value\Float\F)
        Case *QuadClass
          String = Str(*Token\Value\Quad\Q)
        Case *DoubleClass
          String = StrD(*Token\Value\Double\D)
        Case *StringClass
          String = *Token\Value\String\S
          String = ReplaceString(String, #N, "#n")
          String = ReplaceString(String, #Q, "#q")
          String = #Q + String + #Q
        Case *BooleanClass
          If *Token\Value\Integer\I
            String = "#true"
          Else
            String = "#false"
          EndIf
      EndSelect
    Case #Token_Operator
      String = Operator(*Token\Operator)
    Case #Token_Control
      Select *Token\Control
        Case #Control_Next
          String = "\"
        Case #Control_Previous
          String = "\\"
        Case #Control_Class
          String = ":"
        Case #Control_ObjectClass
          String = "::"
        Case #Control_Type
          String = "."
        Case #Control_ECT
          String = "..."
        Case #Control_Address
          String = "@"
      EndSelect
    Case #Token_Seperator
      Select *Token\Seperator
        Case #Seperator_Command
          String = ";"
        Case #Seperator_Argument
          String = ", "
        Case #Seperator_Block
          String = "}"
      EndSelect
    Case #Token_Block
      String = "{"
    Case #Token_Keyword
      String = Keyword(*Token\Keyword)
  EndSelect
  
  ProcedureReturn String
EndProcedure
Procedure.s FormatTokenQueue(*Queue.Queue, Indent.i = 0)
  Protected *Token.Token, *Item.QueueItem
  Protected Code.s, FormatedToken.s, Space.i, *PreviousToken.Token, *NextToken.Token
  
  *Item = *Queue\First
  While *Item
    *Token = *Item\Memory
    
    Select *Token\Type
      Case #Token_Name
        FormatedToken = *Token\Name
      Case #Token_Value
        Select *Token\Value\Class
          Case *IntegerClass
            FormatedToken = Str(*Token\Value\Integer\I)
          Case *FloatClass
            FormatedToken = StrF(*Token\Value\Float\F)
          Case *QuadClass
            FormatedToken = Str(*Token\Value\Quad\Q)
          Case *DoubleClass
            FormatedToken = StrD(*Token\Value\Double\D)
          Case *StringClass
            FormatedToken = *Token\Value\String\S
            FormatedToken = ReplaceString(FormatedToken, #N, "#n")
            FormatedToken = ReplaceString(FormatedToken, #Q, "#q")
            FormatedToken = #Q + FormatedToken + #Q
          Case *BooleanClass
            If *Token\Value\Integer\I
              FormatedToken = "#true"
            Else
              FormatedToken = "#false"
            EndIf
          Default
            FormatedToken = FormatValue(*Token\Value)
        EndSelect
      Case #Token_Operator
        FormatedToken = Operator(*Token\Operator)
      Case #Token_Control
        Select *Token\Control
          Case #Control_Next
            FormatedToken = "\"
          Case #Control_Previous
            FormatedToken = "\\"
          Case #Control_Class
            FormatedToken = ":"
          Case #Control_ObjectClass
            FormatedToken = "::"
          Case #Control_Type
            FormatedToken = "."
          Case #Control_ECT
            FormatedToken = "..."
          Case #Control_Address
            FormatedToken = "@"
        EndSelect
      Case #Token_Seperator
        Select *Token\Seperator
          Case #Seperator_Command
            If *Item\N
              FormatedToken = #N + Space(Indent*#IndentLength)
            Else
              ProcedureReturn Code
            EndIf
          Case #Seperator_Argument
            FormatedToken = ", "
          Case #Seperator_Block
            ProcedureReturn Code
        EndSelect
      Case #Token_Block
        FormatedToken = FormatTokenQueue(*Token\Block\TokenQueue, Indent + 1)
        If Not FindString(FormatedToken, #N, 1)
          If *Token\Block\Bracket = #Bracket_Curly
            FormatedToken = Bracket(*Token\Block\Bracket) + " " + FormatedToken + " " + ClosingBracket(*Token\Block\Bracket)
          Else
            FormatedToken = Bracket(*Token\Block\Bracket) + FormatedToken + ClosingBracket(*Token\Block\Bracket)
          EndIf
        Else
          FormatedToken = Bracket(*Token\Block\Bracket) + #N + Space((Indent + 1)*#IndentLength) + FormatedToken + #N + Space(Indent*#IndentLength) + ClosingBracket(*Token\Block\Bracket)
        EndIf
      Case #Token_Keyword
        FormatedToken = Keyword(*Token\Keyword)
    EndSelect
    
    If *Item\P
      *PreviousToken = *Item\P\Token
    Else
      *PreviousToken = 0
    EndIf
    If *Item\N
      *NextToken = *Item\N\Token
    Else
      *NextToken = 0
    EndIf
    
    If *Token\Type = #Token_Operator And OperatorLeftValue(*Token\Operator)
      If *NextToken And *NextToken\Type<>#Token_Seperator
        Space = #True
      EndIf
    ElseIf *Token\Type = #Token_Keyword And *PreviousToken And (*PreviousToken\Type = #Token_Name Or *PreviousToken\Type = #Token_Value)
      Space = #True
    ElseIf *PreviousToken
      If *PreviousToken\Type = #Token_Operator And Not (*Item\P\P And *Item\P\P\Token\Type = #Token_Operator)
        Space = #True
      ElseIf *PreviousToken\Type = #Token_Keyword And *Token\Type<>#Token_Block
        Space = #True
      EndIf
    EndIf
    
    If Space
      Code + " " + FormatedToken
      Space = #False
    Else
      Code + FormatedToken
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn Code
EndProcedure
; Token-file:
Procedure.i SaveTokenQueue(Filename.s, *TokenQueue.Queue)
  Protected File.i
  
  File = CreateFile(#PB_Any, Filename)
  If File
    WriteTokenQueue(File, *TokenQueue)
    CloseFile(File)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i LoadTokenQueue(Filename.s, *TokenQueue.Queue)
  Protected File.i
  
  File = ReadFile(#PB_Any, Filename)
  If File
    ReadTokenQueue(File, *TokenQueue)
    CloseFile(File)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i WriteTokenQueue(File.i, *TokenQueue.Queue)
  Protected *Token.Token, *Item.QueueItem
  
  *Item = *TokenQueue\First
  While *Item
    *Token = *Item\Memory
    WriteByte(File, *Token\Type)
    Select *Token\Type
      Case #Token_Name
        WriteNTString(File, *Token\String)
      Case #Token_Value
        Select *Token\Value\Class
          Case *IntegerClass
            WriteByte(File, #Type_Integer)
            WriteInteger(File, *Token\Value\Integer\I)
          Case *FloatClass
            WriteByte(File, #Type_Float)
            WriteFloat(File, *Token\Value\Float\F)
          Case *QuadClass
            WriteByte(File, #Type_Quad)
            WriteQuad(File, *Token\Value\Quad\Q)
          Case *DoubleClass
            WriteByte(File, #Type_Double)
            WriteDouble(File, *Token\Value\Double\D)
          Case *StringClass
            WriteByte(File, #Type_String)
            WriteNTString(File, *Token\Value\String\S)
          Case *BooleanClass
            WriteByte(File, #Type_Bool)
            WriteByte(File, *Token\Value\Integer\I)
        EndSelect
      Case #Token_Operator, #Token_Control, #Token_Keyword
        WriteByte(File, *Token\Integer)
      Case #Token_Seperator
        WriteByte(File, *Token\Seperator)
        If *Token\Seperator = #Seperator_Block
          ProcedureReturn #True
        EndIf
      Case #Token_Block
        WriteByte(File, *Token\Block\Bracket)
        WriteTokenQueue(File, *Token\Block\TokenQueue)
    EndSelect
    _Move(*Item, N)
  Wend
EndProcedure
Procedure.i ReadTokenQueue(File.i, *TokenQueue.Queue)
  Protected *Token.Token, Type.i
  
  While Eof(File) = #False
    *Token = _New(Token)
    AppendQueueItem(*TokenQueue, *Token)
    *Token\Type = ReadByte(File)
    Select *Token\Type
      Case #Token_Name
        *Token\Name = ReadNTString(File)
      Case #Token_Value
        Type = ReadByte(File)
        Select Type
          Case #Type_Integer
            *Token\Value = CreateIValue(ReadInteger(File))
          Case #Type_Float
            *Token\Value = CreateFValue(ReadFloat(File))
          Case #Type_Quad
            *Token\Value = CreateQValue(ReadQuad(File))
          Case #Type_Double
            *Token\Value = CreateDValue(ReadDouble(File))
          Case #Type_String
            *Token\Value = CreateSValue(ReadNTString(File))
          Case #Type_Bool
            *Token\Value = CreateBValue(ReadByte(File))
        EndSelect
      Case #Token_Operator, #Token_Control, #Token_Keyword
        *Token\Integer = ReadByte(File)
      Case #Token_Seperator
        *Token\Seperator = ReadByte(File)
        If *Token\Seperator = #Seperator_Block
          ProcedureReturn #True
        EndIf
      Case #Token_Block
        *Token\Block            = _New(Token_Block)
        *Token\Block\Bracket    = ReadByte(File)
        *Token\Block\TokenQueue = CreateQueue(@FreeToken())
        ReadTokenQueue(File, *Token\Block\TokenQueue)
    EndSelect
  Wend
EndProcedure

; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 163
; FirstLine = 129
; Folding = ---
; EnableXP